/*
 * @(#)Abacus.java
 *
 * Copyright 1994 - 2011  David A. Bagley, bagleyd@tux.org
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

/**
 * The <code>Abacus</code> class extends the
 * <code>AbacusCanvas</code> for handling the responses to the user
 * for the <code>AbacusApplet</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd@tux.org
 * @author	http:/www.tux.org/~bagleyd/abacus.html
 */

public class Abacus extends AbacusCanvas implements
		MouseListener, KeyListener {
	private static final long serialVersionUID = 42L;

	public Abacus(Applet applet,
			Color fg, Color bg, Color borderc,
			Color beadc1, Color beadc2, Color beadc3,
			Color railc1, Color railc2, Color railc3,
			int delay, int rails, boolean vertical,
			int colorScheme, boolean slot,
			boolean diamond, int railIndex,
			boolean topOrient, boolean bottomOrient,
			int topNumber, int bottomNumber,
			int topFactor, int bottomFactor,
			int topSpaces, int bottomSpaces,
			int topPiece, int bottomPiece,
			int topPiecePercent, int bottomPiecePercent,
			int shiftPercent, int subdeck, int subbead,
			boolean sign, int decimalPosition,
			boolean group, int groupSize,
			boolean decimalComma, int base, int subbase,
			int anomaly, int shiftAnomaly,
			int anomalySq, int shiftAnomalySq,
			int displayBase, int pressOffsetY,
			boolean romanNumerals, boolean latin,
			boolean ancientRoman, boolean modernRoman,
			int mode, int submode) {

		this.applet = applet;
		this.foreground = fg; this.background = bg;
		this.currentForeground = this.borderColor = borderc;
		this.railColor[0] = railc1;
		this.railColor[1] = railc2;
		this.railColor[2] = railc3;
		this.symbolColor = this.beadColor[0] = beadc1;
		this.beadColor[1] = beadc2;
		this.beadColor[2] = beadc3;
		this.delay = delay;
		this.rails = rails;
		this.vertical = vertical;
		this.colorScheme = colorScheme;
		this.slot = slot; this.diamond = diamond;
		this.railIndex = railIndex;
		this.topOrient = topOrient; this.bottomOrient = bottomOrient;
		this.topNumber = topNumber; this.bottomNumber = bottomNumber;
		this.topFactor = topFactor; this.bottomFactor = bottomFactor;
		this.topSpaces = topSpaces; this.bottomSpaces = bottomSpaces;
		decks = new AbacusDeck[MAX_DECKS];
		decks[TOP] = new AbacusDeck(rails, topNumber,
			topOrient, topFactor, topSpaces);
		decks[BOTTOM] = new AbacusDeck(rails, bottomNumber,
			bottomOrient, bottomFactor, bottomSpaces);
		decks[TOP].piece = topPiece;
		decks[BOTTOM].piece = bottomPiece;
		decks[TOP].piecePercent = topPiecePercent;
		decks[BOTTOM].piecePercent = bottomPiecePercent;
		this.shiftPercent = shiftPercent;
		this.subdeck = subdeck;
		this.subbead = subbead;
		this.sign = sign;
		this.decimalPosition = decimalPosition;
		this.group = group;
		this.groupSize = groupSize;
		this.decimalComma = decimalComma;
		this.base = base;
		this.subbase = subbase;
		this.anomaly = anomaly;
		this.shiftAnomaly = shiftAnomaly;
		this.anomalySq = anomalySq;
		this.shiftAnomalySq = shiftAnomalySq;
		this.displayBase = displayBase;
		this.pressOffsetY = pressOffsetY;
		this.romanNumerals = romanNumerals;
		this.latin = latin;
		this.ancientRoman = ancientRoman;
		this.modernRoman = modernRoman;
		this.mode = mode;
		this.submode = submode;
		firstPaint = true;
		this.addMouseListener(this);
		this.addKeyListener(this);
		setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	public void processKey(char key) {
		switch (key) {
		case 'h':
		case 'H':
			((AbacusApplet)applet).popApplet();
			break;
		case '>':
		case '.':
			speedUpAbacus();
			break;
		case '<':
		case ',':
			slowDownAbacus();
			break;
		case '@':
			((AbacusApplet)applet).toggleSound();
			break;
		case '?':
			((AbacusApplet)applet).descriptionHelp();
			break;
		case '!':
			((AbacusApplet)applet).featuresHelp();
			break;
		case '^':
			((AbacusApplet)applet).referencesHelp();
			break;
		case 'a':
		case 'A':
			((AbacusApplet)applet).aboutHelp();
			break;
		case ' ':
			if (demo)
				showMoreAbacus();
			break;
		case 'c':
		case 'C':
			clearAbacus();
			break;
		case '~':
		case '`':
			complementAbacus();
			break;
		case 'i':
		case 'I':
			incrementAbacus();
			break;
		case 'd':
		case 'D':
			decrementAbacus();
			break;
		case 'f':
		case 'F':
			changeFormatAbacus();
			break;
		case 'm':
		case 'M':
			changeMuseumAbacus();
			break;
		case 'v':
		case 'V':
			toggleRomanNumeralsAbacus();
			break;
		case 'g':
		case 'G':
			toggleGroupingAbacus();
			break;
		case 's':
		case 'S':
			toggleNegativeSignAbacus();
			break;
		case 'u':
		case 'U':
			toggleQuartersAbacus();
			break;
		case 'p':
		case 'P':
			toggleQuarterPercentsAbacus();
			break;
		case 't':
		case 'T':
			toggleTwelfthsAbacus();
			break;
		case 'b':
		case 'B':
			toggleSubdecksAbacus();
			break;
		case 'e':
		case 'E':
			toggleEighthsAbacus();
			break;
		case 'l':
		case 'L':
			toggleAnomalyAbacus();
			break;
		case 'w':
		case 'W':
			toggleWatchAbacus();
			break;
		case 'o':
		case 'O':
			toggleDemoAbacus();
			break;
		case '$':
			toggleTeachAbacus();
			break;
		case '+':
			toggleRightToLeftAbacusAdd();
			break;
		case '*':
			toggleRightToLeftAbacusMult();
			break;
		case 'n':
		case 'N':
			if (demo)
				showNextAbacus();
			break;
		case 'r':
		case 'R':
			if (demo)
				showRepeatAbacus();
			break;
		case 'j':
		case 'J':
			if (demo)
				showJumpAbacus();
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		/*case '6':
		case '7':
		case '8':*/
			if (demo)
				showChapterAbacus(key - '1');
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		case '\03': /* Control-C */
			if (demo) {
				toggleDemoAbacus();
				break;
			}
			((AbacusApplet)applet).quit();
			break;
		default:
		}
	}

	public void mousePressed(MouseEvent event) {
		int modifiers = event.getModifiers();

		if (mouseDown)
			return;
		// would need to reset Teach but modal
		if (demo) {
			abacusDemo.moreDemo();
			return;
		}
		if (((modifiers & Event.META_MASK) == 0) &&
				((modifiers & Event.ALT_MASK) == 0)) {
			selectAbacus(event.getX(), event.getY());
			mouseDown = true;
		} else {
			if ((event.getClickCount() & 1) == 1)
				clearWithQueryAbacus();
			else
				clearAbacus();
		}
	}

	/* Method required but mousePressed does work */
	public void mouseClicked(MouseEvent event) {
	}

	public void mouseReleased(MouseEvent event) {
		if (mouseDown) { // Do not care which mouse button
			releaseAbacus(/*event.getX(), event.getY()*/);
			mouseDown = false;
		}
	}

	public void mouseEntered(MouseEvent event) {
		currentForeground = foreground;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	public void mouseExited(MouseEvent event) {
		if (mouseDown && currentDeck >= 0) {
			currentSpace = -1;
			releasePaint = true;
		}
		currentForeground = borderColor;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	public void keyPressed(KeyEvent event) {
		if (event.getKeyCode() == KeyEvent.VK_ESCAPE)
			((AbacusApplet)applet).shuffleDown();
		else
			processKey(event.getKeyChar());
	}

	/* Method required but keyPressed does work */
	public void keyReleased(KeyEvent event) {
	}

	/* Method required but keyPressed does work */
	public void keyTyped(KeyEvent event) {
	}
}
