/*
 * @(#)AbacusApplet.java
 *
 * Copyright 1994 - 2011  David A. Bagley, bagleyd@tux.org
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.URL;
import java.util.Random;
import java.util.EnumSet;
/*import static java.awt.GraphicsDevice.WindowTranslucency.*;*/

/**
 * The <code>AbacusApplet</code> class is to be embedded in a Web page
 * or run as an application.
 *
 * @author	David A. Bagley
 * @author	bagleyd@tux.org
 * @author	http:/www.tux.org/~bagleyd/abacus.html
 */

/* http://www.codemiles.com/java/jframe-on-applet-t385.html */

public class AbacusApplet extends JApplet implements ActionListener {
	private static final long serialVersionUID = 42L;
	public static final String nl = System.getProperty("line.separator");
	static final int MAX_RAILS = 24;
	static final String[] commandString = new String[] {
		"Clear", "Complement ~", "Increment", "Decrement",
		"Format", "Museum", "Roman Nvmerals", "Group", "Sign",
		"Quarter", "Quarter Percent", "Twelfth",
		"Subdeck", "Eighth", "Anomaly", "Watch",
		"Right To Left Add +", "Right To Left Mult *",
		"Speed >", "Slow <", "Sound @", "Demo", "Teach $",
		"Description ?", "Features !", "References ^", "About",
		"Detach", "Exit"};
	static final char[] commandChar = new char[] {
		'c', '~', 'i', 'd',
		'f', 'm', 'v', 'g', 's',
		'u', 'p', 't',
		'b', 'e', 'l', 'w',
		'+', '*',
		'>', '<', '@', 'o', '$',
		'?', '!', '^', 'a', 'h', 'x'};

	static ArgumentParser argumentParser = null;
	Icon mediumIcon = null;
	Icon largeIcon = null;
	int widthValue = 400;
	int heightValue = 400;
	Color fgColor = AbacusInterface.tan;
	Color bgColor = AbacusInterface.steelBlue;
	Color borderColor = AbacusInterface.gray25;
	Color primaryBeadColor = AbacusInterface.darkRed;
	Color leftAuxBeadColor = AbacusInterface.limeGreen;
	Color rightAuxBeadColor = AbacusInterface.gainsboro;
	Color secondaryBeadColor = AbacusInterface.brown;
	Color highlightBeadColor = AbacusInterface.magenta;
	Color primaryRailColor = AbacusInterface.gold;
	Color secondaryRailColor = AbacusInterface.silver;
	Color highlightRailColor = AbacusInterface.purple;
	boolean sound = false;
	int delayValue = 50; // msec
	int testValue = 0; // minutes
	boolean script = false;
	boolean controlValue = true;
	boolean demoValue = false;
	boolean rightToLeftAddValue = false, rightToLeftMultValue = false;
	boolean leeValue = false;
	int railsValue = AbacusInterface.DEFAULT_RAILS;
	int leftAuxRailsValue = AbacusInterface.DEFAULT_LEFT_AUX_RAILS;
	int rightAuxRailsValue = AbacusInterface.DEFAULT_RIGHT_AUX_RAILS;
	int totalAuxRailsValue = leftAuxRailsValue + rightAuxRailsValue;
	boolean verticalValue = false;
	int colorSchemeValue = 0;
	boolean slotValue = false;
	boolean diamondValue = false;
	int railIndexValue = 0;
	boolean topOrientValue = AbacusInterface.DEFAULT_TOP_ORIENT;
	boolean bottomOrientValue = AbacusInterface.DEFAULT_BOTTOM_ORIENT;
	int topNumberValue = AbacusInterface.DEFAULT_TOP_NUMBER;
	int bottomNumberValue = AbacusInterface.DEFAULT_BOTTOM_NUMBER;
	int topFactorValue = AbacusInterface.DEFAULT_TOP_FACTOR;
	int bottomFactorValue = AbacusInterface.DEFAULT_BOTTOM_FACTOR;
	int topSpacesValue = AbacusInterface.DEFAULT_TOP_SPACES;
	int bottomSpacesValue = AbacusInterface.DEFAULT_BOTTOM_SPACES;
	int topPieceValue = 0;
	int bottomPieceValue = 0;
	int topPiecePercentValue = 0;
	int bottomPiecePercentValue = 0;
	int shiftPercentValue = 2;
	int subdeckValue = 0;
	int subbeadValue = 4;
	boolean signValue = false;
	int decimalPositionValue = 2; /* Good default for most currencies */
	boolean groupValue = false;
	int groupSizeValue = AbacusInterface.DEFAULT_GROUP_SIZE;
	boolean decimalCommaValue = false;
	int baseValue = AbacusInterface.DEFAULT_BASE;
	int subbaseValue = 12;
	int anomalyValue = 0;
	int shiftAnomalyValue = 2;
	int anomalySqValue = 0;
	int shiftAnomalySqValue = 2;
	int displayBaseValue = AbacusInterface.DEFAULT_BASE;
	boolean pressOffsetValue = false;
	boolean romanNumeralsValue = false;
	boolean latinValue = false;
	boolean ancientRomanValue = false;
	boolean modernRomanValue = false;
	int modeValue = AbacusInterface.Modes.Chinese.ordinal();
	int submodeValue = -1;
	Random generator;
	int valueTextLength = 42 - 14; /* Roman Numerals fit */
	boolean debug = false;
	JPanel topPanel = new JPanel();
	Slider railSlider = null, baseSlider = null, displaySlider = null;
	JComboBox/*<Object>*/ modeComboBox = new JComboBox/*<Object>*/();
	Abacus abacus = null, leftAuxAbacus = null, rightAuxAbacus = null;
	AbacusCalc abacusCalc = new AbacusCalc(abacus);
	AbacusCalc leftAuxAbacusCalc = new AbacusCalc(leftAuxAbacus);
	AbacusCalc rightAuxAbacusCalc = new AbacusCalc(rightAuxAbacus);
	JTextField leftAuxValueTextField = new JTextField("0.0");
	JTextField rightAuxValueTextField = new JTextField("0.0");
	JTextField valueTextField = new JTextField("0.0");
	AudioClip bumpAudioClip = null;
	AudioClip moveAudioClip = null;
	OKDialog aDialog = null;
	DemoDialog demoDialog = null;
	TestDialog testDialog = null;
	TeachDialog teachDialog = null;
	StringBuffer stringBufferSave;
	boolean mathDone = true;
	JMenuBar menuBar;
	JMenuItem detachMenuItem;
	JPanel mainPanel = new JPanel();
	JPanel secondaryPanel = new JPanel();
	JFrame frame;
	boolean isPopped = false;
	Font font = new Font("Verdana", Font.PLAIN, 12);

	/* Thanks to Brian Pipa 08Feb2004
         * http://www.javakb.com/Uwe/Forum.aspx/java-gui/1533/Detaching-Applet-to-a-Frame-the-Frame-is-iconized */

	private void popoutApplet() {
		// Comment out next line to DEBUG
		if (getApplication()) {
			return;
		}
		if (frame == null) {
			frame = new JFrame(AbacusInterface.TITLE);
			frame.addWindowListener(new WindowAdapter() {
                                public void windowClosing(WindowEvent e) {
                                        popinApplet();
                                }
                        });
			// possible if debugging
			if (!getApplication())
				frame.setIconImage(getImage(getCodeBase(),
					(nl.compareTo("\n") == 0) ?
					AbacusInterface.LARGE_ICON :
					AbacusInterface.TINY_ICON));
		}
		getContentPane().remove(mainPanel);
		frame.getContentPane().add(mainPanel);
		int ht = heightValue + topPanel.getSize().height;

		if (controlValue)
			ht += menuBar.getSize().height;
		Insets inset = frame.getInsets();
		int titleBarHeight = (controlValue) ?
			menuBar.getSize().height : // a good guess for titleHeight
			16; // a bad guess for titleHeight
		frame.setSize(widthValue + inset.left + inset.right,
			ht + inset.top + inset.bottom +
			titleBarHeight);
		frame.setLocation(0, 0);
		isPopped = true;
		invalidate();
		validate();
		/* Thanks to Alex, move if null in popinApplet removed
		 * http://stackoverflow.com/questions/2106367/listen-to-jframe-resize-events-as-the-user-drags-their-mouse */
		mainPanel.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				int topOffset = topPanel.getSize().height;

				if (controlValue)
					topOffset += menuBar.getSize().height;
				setSize(mainPanel.getSize().width,
					mainPanel.getSize().height - topOffset);
				//setShape(new Rectangle2D.Double(0, 0, getWidth(), getHeight());
			}
		});
		frame.setVisible(true);
		repaint();
		detachMenuItem.setText("Attach");
	}

	void popinApplet() {
		frame.setVisible(false);
		frame.getContentPane().remove(mainPanel);
		frame = null; // not my original idea, but this gets rid of
			// a drawing error on second detachment.
		getContentPane().add(mainPanel);
		isPopped = false;
		invalidate();
		validate();
		repaint();
		detachMenuItem.setText("Detach");
	}

	public boolean popApplet() {
		if (isPopped) {
			popinApplet();
		} else {
			popoutApplet();
		}
		return isPopped;
	}

	public String getDemoString() {
		return this.stringBufferSave.toString();
	}

	public void setDemo(String s) {
		abacus.stringSave = s;
	}

	public boolean getDemo() {
		return demoValue;
	}

	public boolean getTest() {
		return (testValue != 0);
	}

	public boolean getTeach() {
		return (teachDialog != null);
	}

	public boolean getRightToLeftAdd() {
		return rightToLeftAddValue;
	}

	public void setRightToLeftAdd(boolean rightToLeftAdd) {
		rightToLeftAddValue = rightToLeftAdd;
	}

	public boolean getRightToLeftMult() {
		return rightToLeftMultValue;
	}

	public void setRightToLeftMult(boolean rightToLeftMult) {
		rightToLeftMultValue = rightToLeftMult;
	}

	public boolean getLee() {
		return leeValue;
	}

	public Abacus getAbacus(int aux) {
		if (leeValue && aux == AbacusInterface.LEFT_AUX)
			return leftAuxAbacus;
		else if (leeValue && aux == AbacusInterface.RIGHT_AUX)
			return rightAuxAbacus;
		else /*if (aux == AbacusInterface.PRIMARY)*/
			return abacus;
	}

	public boolean getApplication() {
		return (argumentParser != null);
	}

	public boolean mainAbacus(AbacusCanvas ab) {
		return abacus.equals(ab);
	}

	public boolean getToggleSound() {
		return sound;
	}

	/* This sometimes hangs so put it here and it runs only if you
	   really want sound */
	public void initSound() {
	}

	public void toggleSound() {
		sound = !sound;
	}

	public boolean getScript() {
		return script;
	}

	public String getOpt(String string) {
		if (getApplication()) {
			return argumentParser.getOption(string);
		}
		return getParameter(string);
	}

	public int getOpt(String param, int value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public boolean getOpt(String param, boolean value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public Color getOpt(String param, Color value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public void getOpts() {
		String string;

		widthValue = getOpt("windowWidth", widthValue);
		heightValue = getOpt("windowHeight", heightValue);
		fgColor = getOpt("fgColor", fgColor);
		bgColor = getOpt("bgColor", bgColor);
		borderColor = getOpt("borderColor", borderColor);
		primaryBeadColor = getOpt("primaryBeadColor", primaryBeadColor);
		leftAuxBeadColor = getOpt("leftAuxBeadColor",
			leftAuxBeadColor);
		rightAuxBeadColor = getOpt("rightAuxBeadColor",
			rightAuxBeadColor);
		secondaryBeadColor = getOpt("secondaryBeadColor",
			secondaryBeadColor);
		highlightBeadColor = getOpt("highlightBeadColor",
			highlightBeadColor);
		primaryRailColor = getOpt("primaryRailColor",
			primaryRailColor);
		secondaryRailColor = getOpt("secondaryRailColor",
			secondaryRailColor);
		highlightRailColor = getOpt("highlightRailColor",
			highlightRailColor);
		sound = getOpt("sound", sound);
		script = getOpt("script", script);
		controlValue = getOpt("control", controlValue);
		rightToLeftAddValue = getOpt("rightToLeftAdd",
			rightToLeftAddValue);
		rightToLeftMultValue = getOpt("rightToLeftMult",
			rightToLeftMultValue);
		leeValue = getOpt("lee", leeValue);
		railsValue = getOpt("rails", railsValue);
		if (railsValue < AbacusInterface.MIN_RAILS)
			railsValue = AbacusInterface.DEFAULT_RAILS;
		leftAuxRailsValue = getOpt("leftAuxRails", leftAuxRailsValue);
		if (leftAuxRailsValue < AbacusInterface.MIN_RAILS)
			leftAuxRailsValue =
				AbacusInterface.DEFAULT_LEFT_AUX_RAILS;
		rightAuxRailsValue = getOpt("rightAuxRails",
			rightAuxRailsValue);
		if (rightAuxRailsValue < AbacusInterface.MIN_RAILS)
			rightAuxRailsValue =
				AbacusInterface.DEFAULT_RIGHT_AUX_RAILS;
		totalAuxRailsValue = leftAuxRailsValue + rightAuxRailsValue;
		verticalValue = getOpt("vertical", verticalValue);
		colorSchemeValue = getOpt("colorScheme", colorSchemeValue);
		slotValue = getOpt("slot", slotValue);
		diamondValue = getOpt("diamond", diamondValue);
		railIndexValue = getOpt("railIndex", railIndexValue);
		topOrientValue = getOpt("topOrient", topOrientValue);
		bottomOrientValue = getOpt("bottomOrient", bottomOrientValue);
		topNumberValue = getOpt("topNumber", topNumberValue);
		bottomNumberValue = getOpt("bottomNumber", bottomNumberValue);
		topFactorValue = getOpt("topFactor", topFactorValue);
		bottomFactorValue = getOpt("bottomFactor", bottomFactorValue);
		topSpacesValue = getOpt("topSpaces", topSpacesValue);
		bottomSpacesValue = getOpt("bottomSpaces", bottomSpacesValue);
		topPieceValue = getOpt("topPiece", topPieceValue);
		bottomPieceValue = getOpt("bottomPiece", bottomPieceValue);
		topPiecePercentValue = getOpt("topPiecePercent",
			topPiecePercentValue);
		bottomPiecePercentValue = getOpt("bottomPiecePercent",
			bottomPiecePercentValue);
		shiftPercentValue = getOpt("shiftPercent", shiftPercentValue);
		subdeckValue = getOpt("subdeck", subdeckValue);
		subbeadValue = getOpt("subbead", subbeadValue);
		signValue = getOpt("sign", signValue);
		decimalPositionValue = getOpt("decimalPosition",
			decimalPositionValue);
		groupValue = getOpt("group", groupValue);
		groupSizeValue = getOpt("groupSize", groupSizeValue);
		decimalCommaValue = getOpt("decimalComma", decimalCommaValue);
		baseValue = getOpt("base", baseValue);
		if (baseValue < AbacusInterface.MIN_BASE ||
				baseValue > AbacusInterface.MAX_BASE)
			baseValue = AbacusInterface.DEFAULT_BASE;
		subbaseValue = (getOpt("eighth", subbaseValue == 8)) ? 8 : 12;
		anomalyValue = getOpt("anomaly", anomalyValue);
		shiftAnomalyValue = getOpt("shiftAnomaly", shiftAnomalyValue);
		anomalySqValue = getOpt("anomalySq", anomalySqValue);
		shiftAnomalySqValue = getOpt("shiftAnomalySq",
			shiftAnomalySqValue);
		displayBaseValue = getOpt("displayBase", displayBaseValue);
		if (displayBaseValue < AbacusInterface.MIN_BASE ||
				displayBaseValue > AbacusInterface.MAX_BASE)
			displayBaseValue = AbacusInterface.DEFAULT_BASE;
		pressOffsetValue = getOpt("pressOffset", pressOffsetValue);
		romanNumeralsValue = getOpt("romanNumerals",
			romanNumeralsValue);
		latinValue = getOpt("latin", latinValue);
		ancientRomanValue = getOpt("ancientRoman", ancientRomanValue);
		modernRomanValue = getOpt("modernRoman", modernRomanValue);
		string = getOpt("format");
		if (string != null) {
			modeValue = AbacusInterface.setModeFromFormat(string);
		}
		modeValue = getOpt("mode", modeValue);
		if (modeValue < AbacusInterface.Modes.Chinese.ordinal() ||
				modeValue > AbacusInterface.Modes.Generic.ordinal())
			modeValue = AbacusInterface.Modes.Chinese.ordinal();
		string = getOpt("museum");
		if (string != null) {
			submodeValue = AbacusInterface.setMuseumFromFormat(string);
		}
		submodeValue = getOpt("submode", submodeValue);
		if (submodeValue < 0 ||
				submodeValue >= AbacusInterface.MAX_MUSEUMS)
			submodeValue = generator.nextInt(AbacusInterface.MAX_MUSEUMS);
		delayValue = getOpt("delay", delayValue);
		testValue = getOpt("test", testValue);
	}

	public void showDemoMessage(String msg) {
		if (demoDialog == null) {
			createDemoDialog(msg);
		} else {
			demoDialog.showDemoMessage(msg);
		}
	}

	public void createDemoDialog(String msg) {
		if (demoDialog == null) {
//			FontMetrics fm = getFontMetrics(this.getFont());
			FontMetrics fm = getFontMetrics(font);

			demoDialog = new DemoDialog(ComponentUtil.findFrame(this),
				this, AbacusInterface.TITLE + ": Demo",
				msg + "                               ",
				largeIcon);
			demoDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2,
				425, (4 + ((getApplication()) ? 4 : 6)) *
				fm.getHeight());
			demoDialog.setBackground(Color.white);
			demoDialog.setVisible(true);
		}
	}

	public void killDemoDialog() {
		if (demoDialog != null) {
			demoDialog.dispose();
			demoDialog = null;
		}
	}

	public void setDemoMsg(String m1, String m2, String m3, String m4) {
		showDemoMessage(m1 + nl + m2 + nl + m3 + nl + m4);
	}

	public void setDemoMsg(String m1) {
		showDemoMessage(m1 + nl + nl + nl);
	}

	public String readUserDemo() {
		String demoString = "";

		return demoString;
	}

	public void setTest(String[] testNames, AbacusTest test) {
		if (testDialog == null) {
			createTestDialog(testNames, test);
		}
	}

	public void setTestLabel(String msg) {
		testDialog.showTestLabel(msg);
	}

	public void setTestQuestion(String msg) {
		testDialog.showTestQuestion(msg);
	}

	public void setTestPercentValue(String msg) {
		testDialog.showTestPercentValue(msg);
	}

	public void setGrade(int numberCorrect, int numberQuestions, int grade, String gradeString) {
		testDialog.showTestGrade(numberCorrect, numberQuestions, grade, gradeString);
	}

	public void createTestDialog(String[] testNames, AbacusTest test) {
		if (testDialog == null) {
//			FontMetrics fm = getFontMetrics(this.getFont());
			FontMetrics fm = getFontMetrics(font);

			testDialog = new TestDialog(ComponentUtil.findFrame(this),
				this, test, AbacusInterface.TITLE + ": Test",
				testNames, largeIcon, testValue);
			testDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2,
				425, 13 * fm.getHeight());
			testDialog.setBackground(Color.white);
			testDialog.setVisible(true);
		}
	}

	public void killTestDialog() {
		if (testDialog != null) {
			testDialog.dispose();
			testDialog = null;
		}
	}

	public void createTeachDialog() {
		if (teachDialog == null) {
//			FontMetrics fm = getFontMetrics(this.getFont());
			FontMetrics fm = getFontMetrics(font);

			teachDialog = new TeachDialog(ComponentUtil.findFrame(this),
				this, abacus, AbacusInterface.TITLE + ": Teach",
				AbacusInterface.TEACH_STRING0, 0, largeIcon);
			teachDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2,
				600, (4 + ((getApplication()) ? 4 : 6)) *
				fm.getHeight());
			teachDialog.setBackground(Color.white);
			teachDialog.setVisible(true);
		}
	}

	public void showTeachMessage(String msg, int line) {
		teachDialog.showTeachMessage(msg, line);
	}

	public void killTeachDialog() {
		if (teachDialog != null) {
			teachDialog.dispose();
			teachDialog = null;
		}
	}

	public void setTeachMsg(String m, int line) {
		showTeachMessage(m, line);
	}

	public void showMessage(String msg) {
		if (aDialog == null || !aDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findFrame(this),
				AbacusInterface.TITLE, msg, "OK",
				mediumIcon, false);
			aDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2, 350, 120);
			aDialog.setVisible(true);
		}
	}

	public void descriptionHelp() {
		if (aDialog == null || !aDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findFrame(this),
				AbacusInterface.TITLE + ": Description",
"This is an implementation of the classic Chinese abacus " +
"(Saun-pan) which has its origins in the 12th century." + nl +
" " + nl +
"The device  has two decks.  Each deck, separated by a  partition," +
" normally  has  13  rails  on which are  mounted  beads." + nl +
"Each rail on the top deck contains 1 or 2  beads,  and  each " +
"rod  on the bottom deck contains 4 or 5 beads.  Each bead on" + nl +
"the upper deck has a value of five, while each bead  on  the " +
"lower  deck has value of one.  Beads are considered counted," + nl +
"when moved towards the partition separating the decks, i.e. " +
"to add a value of one, a bead in the bottom deck is moved up," + nl +
"and to add a value of 5, a bead in the top deck is moved " +
"down." + nl +
" " + nl +
"The basic operations of the abacus are addition and subtraction. " +
"  Multiplication can be done by mentally multiplying the" + nl +
"digits  and adding up the intermediate results  on the abacus. " +
" Division would be similar  where the intermediate results" + nl +
"are subtracted.  There are techniques like using your thumb " +
"with forefinger  which does not apply with mouse entry.  Also" + nl +
"with multiplication,  one can  carry out calculations  on " +
"different parts  of the abacus, here it is nice to  have a long" + nl +
"abacus." + nl +
" " + nl +
"The pre-WWII Japanese abacus (Soroban)  (or Korean Supan) is " +
"similar to the Chinese abacus but has only one bead per rail" + nl +
"on the top deck.  The later Japanese abacus was further " +
"simplified to have only 4 beads per rail on the bottom deck." + nl +
" " + nl +
"The Roman hand-abacus predates the  Chinese  abacus  and  is " +
"very similar to the later Japanese abacus, but seems to have" + nl +
"fallen out of use with the Fall  of  the  Roman  Empire  (at " +
"least 3 are in existence).  The Roman abaci are brass plates" + nl +
"where the beads move in slots.  In addition to the normal  7 " +
"columns  of  beads, they generally have 2 special columns on" + nl +
"the right side.  In two examples:  the first  special column " +
"was for 12ths  (12 uncia (ounces) = 1 as) and had one  extra" + nl +
"bead in the bottom deck.  Also the last column was a combination " +
"of halves, quarters, and twelfths of an ounce and had no" + nl +
"beads in the top deck and 4 at the bottom (beads did not have " +
"to come to the top  to be  counted  but at one of 3  marked" + nl +
"points, where the top bead was for halves, the next bead for " +
"quarters,  and the last two beads for twelfths).  In another" + nl +
"surviving example: the 2 special columns were switched and the " +
"combination column was broken into 3 separate slots." + nl +
" " + nl +
"The Russian abacus was invented in the 17th  century,  here " +
"the beads are moved from right to left.  It has colored beads" + nl +
"in the middle for ease of use.  Quarters represent 1/4 Rubles " +
"and are only  present  historically on the Russian abacus" + nl +
"(Schoty).  Some of the older  Schoty  have a extra place for " +
"the  1/4  Kopek  (quarter  percent)  as well as the 1/4 Ruble" + nl +
"(quarter)." + nl +
" " + nl +
"The Danish abacus was used in the early 20th century in " +
"elementary schools as a teaching aid." + nl +
" " + nl +
"The Mesoamerican Nepohualtzintzin is a Japanese abacus base 20. " +
"  The Mesoamericans had base 20 with the exception of the" + nl +
"3rd decimal  place where instead of 20 * 20 = 400 the third place " +
"marked 360  and the 4th place was 20 * 360, etc..  They" + nl +
"independently  created their own zero  (only Babylon (base 60) " +
"and  India (base 10) have done this)  but the anomaly took" + nl +
"away its true power." + nl +
" " + nl +
"An easy way of figuring out time in seconds given hours, " +
"minutes, and seconds,  can be done on the abacus with special" + nl +
"Japanese abacus with special anomaly \"watch\" settings." + nl +
" " + nl +
"The Chinese Solid-and-Broken-Bar System is a base 12 numbering " +
"system and not really an abacus.  When the abacus is setup" + nl +
"in this way though (topFactor 3, topNumber 3, bottomNumber 2, " +
"base 12, displayBase 12), it is easy to relate the two." + nl +
" " + nl +
"The signed bead is an invention of the  author  and  is  not " +
"present  on  any historical abacus (to his knowledge) and is" + nl +
"used to represent negatives.  \"New & Improved\" abacus " +
" models have two auxiliary  decks stacked above the principal decks" + nl +
"that enable multiplication, division, square-root, and " +
"cube-root computations to be performed with equal ease as addition" + nl +
"and subtraction.",
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void featuresHelp() {
		if (aDialog == null || !aDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findFrame(this),
				AbacusInterface.TITLE + ": Features",
"Click \"mouse-left\" button on a bead you want to  move.   The " +
"beads  will  shift  themselves  to vacate the area of the" + nl +
"column that was clicked." + nl +
" " + nl +
"Click \"mouse-right\" button, or press \"C\" or \"c\" keys, to " +
"clear the abacus." + nl +
" " + nl +
"Press \"O\" or \"o\" keys to toggle the demo mode." + nl +
" " + nl +
"Press \"$\" key to toggle the teach mode." + nl +
" " + nl +
"In teach mode, \"+\" key toggles starting side to sum,  " +
"\"*\" key toggles for starting side for multiplicand." + nl +
" " + nl +
"Press \"~\" or \"`\" keys to complement the beads on the rails." + nl +
" " + nl +
"Press \"I\" or \"i\" keys to increment the number of rails.  " +
"Press \"D\" or \"d\" keys to decrement the number of rails." + nl +
" " + nl +
"Press \"F\" or \"f\" keys to switch between Chinese, Japanese, " +
"Korean, Roman, Russian, and Danish formats.  There is an" + nl +
"extra \"Generic\" format, this allows one to break some rules " +
"binding the other formats (for example, if one wanted more" + nl +
"beads on top deck than on bottom deck you would use this, in " +
"addition to resource option changes)." + nl +
" " + nl +
"Press \"E\" or \"e\" keys to switch between it, uk, and fr " +
"museum formats." + nl +
" " + nl +
"Press \"M\" or \"m\" keys to toggle the availability of Roman " +
"Numerals." + nl +
" " + nl +
"Press \"S\" or \"s\" keys to toggle the availability of sign " +
"bead." + nl +
" " + nl +
"Press \"U\" or \"u\" keys to toggle the availability of quarter " +
"beads.  (Mutually exclusive to twelfth beads).  Intended for" + nl +
"the Russian Abacus." + nl +
" " + nl +
"Press \"P\" or \"p\" keys to toggle the availability of quarter " +
"percent beads.   (Dependent on quarter beads (or twelfth" + nl +
"beads)).  Intended for the older Russian Abacus." + nl +
" " + nl +
"Press \"T\" or \"t\" keys to toggle the availability of twelfth " +
"beads.  (Mutually exclusive to quarter beads).  Intended for" + nl +
"the Roman Abacus." + nl +
" " + nl +
"Press \"B\" or \"b\" keys to toggle the availability of subdecks." +
"  (Dependent on twelfth beads (or quarter beads) and Roman" + nl +
"format).  Intended for the Roman Abacus.  , the lowest value of" + nl +
"the rightmost beads are a twelfth of the column second from right." + nl +
" " + nl +
"Press \"H\" or \"h\" keys to toggle the availability of subdecks." +
"  (Dependent on twelfth beads (or quarter beads) and Roman" + nl +
"format).  Intended for the Roman Abacus.  , the lowest value of" + nl +
"the rightmost beads are an eighth of the column second from right." + nl +
" " + nl +
"Press \"L\" or \"l\" keys to toggle the availability of anomaly " +
"bars.  Intended to used with Japanese Abacus and base 20 for" + nl +
"the Mesoamerican Abacus. (Mutually exclusive to watch bars)." + nl +
" " + nl +
"Press \"W\" or \"w\" keys to toggle the availability of watch " +
"bars.  Intended to represent seconds where hours and minutes" + nl +
"can be set.  (Mutually exclusive to anomaly bars)." + nl +
" " + nl +
"Press \">\" or \".\" keys to speed up the movement of beads." + nl +
"Press \"<\" or \",\" keys to slow down the movement of beads." + nl +
" " + nl +
"Press \"@\" key to toggle the sound." + nl +
" " + nl +
"Press \"Esc\" key to hide program." + nl +
" " + nl +
"Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program if an application." + nl +
" " + nl +
"The abacus may be resized.  Beads will reshape depending  on " +
"the room they have.  Demo Mode:  In this mode, the abacus is" + nl +
"controlled by the  program.   When  started  with  the  demo " +
"option,  a  second window is presented that should be placed" + nl +
"directly below the abacus-window. Descriptive text, and user " +
"prompts  are  displayed in this window.  Pressing 'q' during" + nl +
"the demo will quit it.  Clicking the left mouse-button  with " +
"the  pointer  in the window will restart the demo (beginning" + nl +
"of current lesson).",
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void referencesHelp() {
		if (aDialog == null || !aDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findFrame(this),
				AbacusInterface.TITLE + ": References",
"Luis Fernandes  http://www.ee.ryerson.ca/~elf/abacus/" + nl +
"Lee Kai-chen, How to Learn Lee's Abacus, 1958, 58 pages." + nl +
"Abacus Guide Book, 57 pages." + nl +
"Georges Ifrah, The Universal history of Numbers, Wiley Press " +
"2000, pp 209-211, 288-294." + nl +
"Review of above: http://www.ams.org/notices/200201/rev-dauben.pdf" + nl +
"David Eugene Smith, History of Mathematics Volume II, " +
"Dover Publications, Inc 1958, pp 156-195." + nl,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void aboutHelp() {
		if (aDialog == null || !aDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findFrame(this),
				AbacusInterface.TITLE + ": About",
"Java AbacusApplet Version 7.6.8" + nl +
"Send bugs (reports or fixes) to the author: " +
"David Bagley <bagleyd@tux.org>" + nl +
"The latest version is at: " +
"http://www.tux.org/~bagleyd/abacus.html" + nl,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void processKey(char key) {
		abacus.processKey(key);
	}

	public void setRailChangeListenerValue(int n) {
		if (railSlider != null)
			railSlider.setValue(n);
	}

	public String getValue() {
		//replace with spaces then trim!!
		return valueTextField.getText().replace(',', ' ').trim();
	}

	public Icon getIcon(String string, int sx, int sy) {
		return new Icon(((getApplication()) ?
			Toolkit.getDefaultToolkit().
			getImage(AbacusApplet.class.getResource(string)) :
			getImage(getCodeBase(), string)),
			sx, sy);
	}

	public AudioClip getAudioClip(String string) {
		return (getApplication()) ? Applet.newAudioClip(
			AbacusApplet.class.getResource(string)) :
			getAudioClip(getCodeBase(), string);
	}

	public void playBumpAudio() {
		if (bumpAudioClip == null)
			bumpAudioClip = getAudioClip("bump.au");
		bumpAudioClip.play();
	}

	public void playMoveAudio() {
		if (moveAudioClip == null)
			moveAudioClip = getAudioClip("move.au");
		moveAudioClip.play();
	}

	public void setSize(int width, int height) {
		if (abacus == null)
			return;
		//if (getApplication())
		{
			if (leeValue) {
				int newWidth = leftAuxRailsValue * width /
					totalAuxRailsValue;
				int newHeight = height / 4;

				leftAuxAbacus.setBounds(0, 0,
					newWidth, newHeight);
				rightAuxAbacus.setBounds(0, 0,
					width - newWidth, newHeight);
				abacus.setBounds(0, 0,
					width, height - newHeight);
				validate();
			} else {
				abacus.setBounds(0, 0, width, height);
			}
			//repaint(); // causes flashing
		}
		/* else {
			int topOffset = topPanel.getSize().height;
			int ht = height - topOffset;

			//if (controlValue)
			//	topOffset += menuBar.getSize().height;
			if (leeValue) {
				int newWidth = leftAuxRailsValue * width /
					totalAuxRailsValue;
				int newHeight = ht / 4;

				leftAuxAbacus.setBounds(0, 0,
					newWidth, newHeight);
				rightAuxAbacus.setBounds(0, 0,
					width - newWidth, newHeight);
				abacus.setBounds(0, topOffset,
					width, ht - newHeight);
			} else {
				abacus.setBounds(0, topOffset,
					width, ht);

			}
			super.setSize(width, height);
			validate();
		}*/
	}

	public void paint(Graphics g) {
		//if (getApplication())
		{
			int topOffset = topPanel.getSize().height;

			if (controlValue)
				topOffset += menuBar.getSize().height;
			setSize(getSize().width, getSize().height - topOffset);
		}
		super.paint(g);
	}

	public void writeDemoCookie(StringBuffer sb) {
		stringBufferSave = sb;

		try {
			if (!getApplication()) {
				netscape.javascript.JSObject win =
					netscape.javascript.JSObject.getWindow(
					this);
				win.call("writeDemoCookie", null);
			}
		} catch (Exception e) {
			//e.printStackTrace();
			showMessage("no Cookies or JavaScript");
		}
	}

	public void readDemoCookie() {
		try {
			if (!getApplication()) {
				netscape.javascript.JSObject win =
					netscape.javascript.JSObject.getWindow(
					this);
				win.call("readDemoCookie", null);
			}
		} catch (Exception e) {
			//e.printStackTrace();
			showMessage("no Cookies or JavaScript");
		}
	}

	public void shuffleDown() {
		try {
			if (getApplication()) {
				ComponentUtil.findFrame(this).toBack();
			} else {
				netscape.javascript.JSObject win =
					netscape.javascript.JSObject.getWindow(
					this);
				win.call("shuffleDown", null);
			}
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}

	public void quit() {
		try {
			if (getApplication()) {
				System.exit(0);
			} else {
				netscape.javascript.JSObject win =
					netscape.javascript.JSObject.getWindow(
					this);
				win.call("quit", null);
			}
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}

	class RailChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent event) {
			int oldValue = railsValue, min;

			railsValue = railSlider.getValue();
			min = ((signValue) ? 1 : 0) +
				((bottomPieceValue == 0) ? 0 : 1) +
				((bottomPiecePercentValue == 0) ? 0 :
				1 + shiftPercentValue) +
				((demoValue) ? AbacusInterface.MIN_DEMO_RAILS :
				AbacusInterface.MIN_RAILS);
			if (railsValue < min) {
				/* Prohibit possible endless recursion */
				if (oldValue >= min)
					railsValue = oldValue;
				else
					railsValue = 2 * shiftPercentValue + 1;
				railSlider.setValue(railsValue);
				return;
			}
			if (railsValue != oldValue) {
				abacus.shiftRails(oldValue, railsValue,
					false, false, false);
			}
		}
	}


	class ComboBoxActionListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int oldValue = modeValue;
			String modeString = (String) ((JComboBox/*<?>*/)event.
				getSource()).getSelectedItem();

			modeValue = AbacusInterface.Modes.Generic.ordinal();
			for (AbacusInterface.Modes mode : EnumSet.range(AbacusInterface.Modes.Chinese, AbacusInterface.Modes.Danish)) {
				if (modeString.indexOf(mode.toString()) >= 0) {
					modeValue = mode.ordinal();
					mediumIcon = getIcon(AbacusInterface.MEDIUM_ICONS[modeValue], 32, 32);
					largeIcon = getIcon(AbacusInterface.LARGE_ICONS[modeValue], 48, 48);
					break;
				}
			}
			if (modeValue != AbacusInterface.Modes.Generic.ordinal() &&
						oldValue != modeValue) {
				abacus.setMode(modeValue);
				abacus.formatRails(0);
				//make sure digits are not lost when
				//chinese -> japanese
			}
		}
	}

	class AbacusBaseChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent event) {
			int oldValue = baseValue;

			baseValue = baseSlider.getValue();
			if (oldValue == baseValue)
				return;
			if (bottomNumberValue == oldValue && topSpacesValue == 0) {
				topFactorValue = abacus.convertBaseToBottom(baseValue);
				bottomNumberValue = baseValue;
			} else if (topNumberValue * topFactorValue == oldValue) {
				topFactorValue = abacus.convertBaseToBottom(baseValue);
				bottomNumberValue = topFactorValue;
				topNumberValue = baseValue / topFactorValue;
			} else if ((topNumberValue + 1) * topFactorValue == oldValue) {
				topFactorValue = abacus.convertBaseToBottom(baseValue);
				bottomNumberValue = topFactorValue - 1;
				topNumberValue = baseValue / topFactorValue - 1;
			} else {
				System.err.println("Base adjustment, strange format: topNumber " +
					topNumberValue +
					", topFactor " + topFactorValue +
					", newBase " + baseValue +
					", oldBase " + oldValue);
			}
			abacus.setBaseAbacus(baseValue);
			if (leeValue && leftAuxAbacus != null) {
				leftAuxAbacus.setBaseAbacus(baseValue);
			}
			if (leeValue && rightAuxAbacus != null) {
				rightAuxAbacus.setBaseAbacus(baseValue);
			}
		}
	}

	class DisplayBaseChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent event) {
			int oldValue = displayBaseValue;

			displayBaseValue = displaySlider.getValue();
			if (oldValue != displayBaseValue)
				return;
			abacus.setDisplayBaseAbacus(displayBaseValue);
			if (leeValue && leftAuxAbacus != null) {
				leftAuxAbacus.setDisplayBaseAbacus(displayBaseValue);
			}
			if (leeValue && rightAuxAbacus != null) {
				rightAuxAbacus.setDisplayBaseAbacus(displayBaseValue);
			}
		}
	}

	private void forceDemoParams() {
		if (modeValue == AbacusInterface.Modes.Generic.ordinal()) {
			topNumberValue = AbacusInterface.DEFAULT_TOP_NUMBER;
			bottomNumberValue = AbacusInterface.DEFAULT_BOTTOM_NUMBER;
			topFactorValue = AbacusInterface.DEFAULT_TOP_FACTOR;
			bottomOrientValue = AbacusInterface.DEFAULT_BOTTOM_ORIENT;
			topSpacesValue = AbacusInterface.DEFAULT_TOP_SPACES;
			modeValue = AbacusInterface.Modes.Chinese.ordinal();
			baseValue = AbacusInterface.DEFAULT_BASE;
			displayBaseValue = AbacusInterface.DEFAULT_BASE;
			abacus.setValuesModeAbacus(
				topOrientValue, bottomOrientValue,
				topNumberValue, bottomNumberValue,
				topFactorValue, bottomFactorValue,
				topSpacesValue, bottomSpacesValue,
				baseValue, displayBaseValue, modeValue);
			modeComboBox.setSelectedItem(Integer.valueOf(modeValue));
			baseSlider.setValue(baseValue);
			displaySlider.setValue(displayBaseValue);
			mediumIcon = getIcon(AbacusInterface.MEDIUM_ICONS[modeValue], 32, 32);
			largeIcon = getIcon(AbacusInterface.LARGE_ICONS[modeValue], 48, 48);
			if (leeValue && leftAuxAbacus != null) {
				leftAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
			}
			if (leeValue && rightAuxAbacus != null) {
				rightAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
			}
		} else if (baseValue != AbacusInterface.DEFAULT_BASE  ||
				displayBaseValue != AbacusInterface.DEFAULT_BASE) {
			baseValue = AbacusInterface.DEFAULT_BASE;
			displayBaseValue = AbacusInterface.DEFAULT_BASE;
			baseSlider.setValue(baseValue);
			displaySlider.setValue(displayBaseValue);
			abacus.setValuesBaseAbacus(baseValue, displayBaseValue);
			if (leeValue && leftAuxAbacus != null) {
				leftAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
			}
			if (leeValue && rightAuxAbacus != null) {
				rightAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
			}
		}
	}

	private void forceTeachParams() {
		if (displayBaseValue != baseValue || !signValue ||
				anomalyValue != 0 || anomalySqValue != 0) {
			displayBaseValue = baseValue;
			signValue = false;
			anomalyValue = 0;
			anomalySqValue = 0;
			if (displaySlider != null)
				displaySlider.setValue(displayBaseValue);
			abacus.setValuesTeachAbacus(displayBaseValue,
				signValue, anomalyValue, anomalySqValue);
			if (leeValue && leftAuxAbacus != null) {
				leftAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
				leftAuxAbacus.setValuesTeachAbacus(displayBaseValue,
					signValue, anomalyValue, anomalySqValue);
			}
			if (leeValue && rightAuxAbacus != null) {
				rightAuxAbacus.setValuesBaseAbacus(baseValue, displayBaseValue);
				rightAuxAbacus.setValuesTeachAbacus(displayBaseValue,
					signValue, anomalyValue, anomalySqValue);
			}
		}
	}

	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();

		if ("Detach".equals(command)) {
			popoutApplet();
			return;
		} else if ("Attach".equals(command)) {
			popinApplet();
			return;
		}
		for (int opt = 0; opt < commandString.length; opt++)
			if (command.equals(commandString[opt]))
				processKey(commandChar[opt]);
	}

	public void callbackAbacusDemo(int aux, int deck, int rail, int number) {
		if (leeValue && aux == AbacusInterface.LEFT_AUX)
			leftAuxAbacus.directMove(deck, rail, number);
		else if (leeValue && aux == AbacusInterface.RIGHT_AUX)
			rightAuxAbacus.directMove(deck, rail, number);
		else if (aux == AbacusInterface.PRIMARY)
			abacus.directMove(deck, rail, number);
	}

	public void callbackAbacus(AbacusCanvas ab, String value) {
		if (leeValue && leftAuxAbacus.equals(ab)) {
			leftAuxValueTextField.setText(value);
		} else if (leeValue && rightAuxAbacus.equals(ab)) {
			rightAuxValueTextField.setText(value);
		} else {
			valueTextField.setText(value);
		}
	}

	public void callbackAbacus(int aux, int deck, int rail, int number) {
		System.out.println(aux + " " + deck + " " + rail +
			" " + number + " 4");
		System.out.println("Lesson" + nl + nl + nl +
			"Press Space-bar");
	}

	StringBuffer cleanString(StringBuffer dirty) {
		int i, j = 0;

		for (i = 0; i < dirty.length(); i++) {
			if (dirty.charAt(i) == '[' ||
					dirty.charAt(i) == ']') {
				dirty.setLength(j);
				return dirty;
			} else if (dirty.charAt(i) != ' ' &&
					dirty.charAt(i) != '\t') {
				dirty.setCharAt(j, dirty.charAt(i));
				j++;
			}
		}
		return dirty;
	}

	public void clearAllBeads(int aux) {
		if (aux == 0) {
			valueTextField.setText("0.0");
		} else if (leeValue && aux == AbacusInterface.LEFT_AUX) {
			leftAuxAbacus.clearAllBeads();
		} else if (leeValue && aux == AbacusInterface.RIGHT_AUX) {
			rightAuxAbacus.clearAllBeads();
		}
	}

	public void callbackAbacus(AbacusCanvas ab, int reason) {
		switch(reason) {
		/*case AbacusInterface.ACTION_SCRIPT:
			break;*/
		case AbacusInterface.ACTION_MOVE:
			break;
		case AbacusInterface.ACTION_CLEAR:
			if (leeValue) {
				leftAuxAbacus.clearAllBeads();
				rightAuxAbacus.clearAllBeads();
				//leftAuxValueTextField.setText("0.0");
				//rightAuxValueTextField.setText("0.0");
			}
			valueTextField.setText("0.0");
			break;
		case AbacusInterface.ACTION_INCREMENT:
			if (!abacus.equals(ab))
				return;
			railsValue++;
			if (railSlider != null)
				railSlider.setValue(railsValue);
			break;
		case AbacusInterface.ACTION_DECREMENT:
			if (!abacus.equals(ab))
				return;
			railsValue--;
			if (railSlider != null)
				railSlider.setValue(railsValue);
			break;
		case AbacusInterface.ACTION_DEMO:
			if (!abacus.equals(ab))
				return;
			if (getTest() || getTeach())
				return;
			forceDemoParams();
			demoValue = !demoValue;
			break;
		case AbacusInterface.ACTION_TEACH:
			if (!abacus.equals(ab))
				return;
			if (getTest() || demoValue)
				return;
			forceTeachParams();
			if (getTeach())
				killTeachDialog();
			else
				createTeachDialog();
			break;
		case AbacusInterface.ACTION_FORMAT:
			if (!abacus.equals(ab))
				return;
			modeValue = modeValue + 1;
			if (modeValue < 0 || modeValue >= AbacusInterface.Modes.values().length - 1)
				modeValue = AbacusInterface.Modes.Chinese.ordinal();
			modeComboBox.setSelectedIndex(modeValue);
			mediumIcon = getIcon(AbacusInterface.MEDIUM_ICONS[modeValue], 32, 32);
			largeIcon = getIcon(AbacusInterface.LARGE_ICONS[modeValue], 48, 48);
			break;
		case AbacusInterface.ACTION_SIGN:
			if (!abacus.equals(ab))
				return;
			signValue = !signValue;
			break;
		case AbacusInterface.ACTION_QUARTER:
			if (!abacus.equals(ab))
				return;
			if (bottomPieceValue == 0)
				bottomPieceValue = 4;
			else
				bottomPieceValue = 0;
			break;
		case AbacusInterface.ACTION_QUARTER_PERCENT:
			if (!abacus.equals(ab))
				return;
			if (bottomPiecePercentValue == 0)
				bottomPiecePercentValue = 4;
			else
				bottomPiecePercentValue = 0;
			break;
		case AbacusInterface.ACTION_TWELFTH:
			if (!abacus.equals(ab))
				return;
			if (bottomPieceValue == 0)
				bottomPieceValue = 6;
			else
				bottomPieceValue = 0;
			break;
		case AbacusInterface.ACTION_SUBDECK:
			if (!abacus.equals(ab))
				return;
			if (submodeValue == AbacusInterface.IT) {
				submodeValue = AbacusInterface.UK;
			} else if (submodeValue == AbacusInterface.UK) {
				submodeValue = AbacusInterface.FR;
			} else {
				submodeValue = AbacusInterface.IT;
			}
			break;
		case AbacusInterface.ACTION_PLACE:
			break;
		default:
		}
	}

	public void init() {
		/*JPanel info0Panel = new JPanel();
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();*/
		JPanel startPanel = new JPanel();
		JPanel info1Panel = new JPanel();
		JPanel info2Panel = new JPanel();
		JPanel valuePanel = new JPanel();
		JPanel leftAuxValuePanel = new JPanel();
		JPanel rightAuxValuePanel = new JPanel();
		JPanel sliderPanel = new JPanel();
		JPanel comboBoxPanel = new JPanel();
		JPanel canvasPanel = new JPanel();
		JPanel leftAuxCanvasPanel = new JPanel();
		JPanel rightAuxCanvasPanel = new JPanel();

		/*// Determine what the default GraphicsDevice can support.
		GraphicsEnvironment ge =
			GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();

		boolean isShapedWindowSupported =
			gd.isWindowTranslucencySupported(PERPIXEL_TRANSPARENT);*/

		generator = new Random(System.currentTimeMillis());
		getOpts();
		mediumIcon = getIcon(AbacusInterface.MEDIUM_ICONS[modeValue], 32, 32);
		largeIcon = getIcon(AbacusInterface.LARGE_ICONS[modeValue], 48, 48);
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.white);
		if (controlValue) {
			menuBar = new JMenuBar();
			JMenu menu;
			JMenuItem menuItem;

			// Allows menus to be on top of canvas
			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			menu = new JMenu("File");
			// Comment out next line to DEBUG
			if (!getApplication()) {
				menu.add(detachMenuItem = new JMenuItem("Detach"));
				detachMenuItem.setMnemonic(KeyEvent.VK_H);
				detachMenuItem.addActionListener(this);
				menu.addSeparator();
			}
			menu.add(menuItem = new JMenuItem("Exit"));
			menuItem.setMnemonic(KeyEvent.VK_X);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Controls");
			menu.add(menuItem = new JMenuItem("Clear"));
			menuItem.setMnemonic(KeyEvent.VK_C);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Complement ~"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Increment"));
			menuItem.setMnemonic(KeyEvent.VK_I);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Decrement"));
			menuItem.setMnemonic(KeyEvent.VK_D);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Format"));
			menuItem.setMnemonic(KeyEvent.VK_F);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Museum"));
			menuItem.setMnemonic(KeyEvent.VK_M);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Roman Nvmerals"));
			menuItem.setMnemonic(KeyEvent.VK_V);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Group"));
			menuItem.setMnemonic(KeyEvent.VK_G);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Sign"));
			menuItem.setMnemonic(KeyEvent.VK_S);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Quarter"));
			menuItem.setMnemonic(KeyEvent.VK_U);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Quarter Percent"));
			menuItem.setMnemonic(KeyEvent.VK_P);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Twelfth"));
			menuItem.setMnemonic(KeyEvent.VK_T);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Subdeck"));
			menuItem.setMnemonic(KeyEvent.VK_B);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Eighth"));
			menuItem.setMnemonic(KeyEvent.VK_E);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Anomaly"));
			menuItem.setMnemonic(KeyEvent.VK_L);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Watch"));
			menuItem.setMnemonic(KeyEvent.VK_W);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Right To Left Add +"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Right To Left Mult *"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Speed >"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Slow <"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Sound @"));
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Learn");
			menu.add(menuItem = new JMenuItem("Demo"));
			menuItem.setMnemonic(KeyEvent.VK_O);
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Teach $"));
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Help");
			menu.add(menuItem = new JMenuItem("Description ?"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("Features !"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("References ^"));
			menuItem.addActionListener(this);
			menu.add(menuItem = new JMenuItem("About"));
			menuItem.setMnemonic(KeyEvent.VK_A);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			//setJMenuBar(menuBar); // needs to be in a panel
			mainPanel.add(menuBar, BorderLayout.PAGE_START);
			/*leftPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			leftPanel.add(new JLabel("Move bead"));
			rightPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			rightPanel.add(new JLabel("Clear"));
			leftPanel.add(getIcon("mouse-l.gif", 16, 16));
			rightPanel.add(getIcon("mouse-r.gif", 16, 16));
			info0Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));
			info0Panel.add(leftPanel);
			info0Panel.add(rightPanel);*/
			railSlider = new Slider("Abacus Size",
				railsValue, AbacusInterface.MIN_RAILS,
				((railsValue > MAX_RAILS) ? railsValue : MAX_RAILS),
				126);
			baseSlider = new Slider("Abacus Base",
				baseValue,
				AbacusInterface.MIN_BASE, AbacusInterface.MAX_BASE,
				126);
			displaySlider = new Slider("Display Base",
				displayBaseValue,
				AbacusInterface.MIN_BASE, AbacusInterface.MAX_BASE,
				126);
			sliderPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			sliderPanel.add(railSlider);
			sliderPanel.add(baseSlider);
			sliderPanel.add(displaySlider);
			comboBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT,
				5, 0));
			for (AbacusInterface.Modes mode : AbacusInterface.Modes.values()) {
				modeComboBox.addItem(mode.toString() + " " +
					AbacusInterface.formatStrings[mode.ordinal()]);
			}
			modeComboBox.setSelectedItem(
				AbacusInterface.Modes.values()[modeValue].toString() +
					" " + AbacusInterface.formatStrings[modeValue]);
			comboBoxPanel.add(modeComboBox);
		}
		if (leeValue) {
			leftAuxValuePanel.setLayout(new FlowLayout(FlowLayout.LEFT,
				0, 0));
			rightAuxValuePanel.setLayout(new FlowLayout(FlowLayout.LEFT,
				0, 0));
			leftAuxValueTextField.setColumns(valueTextLength *
				leftAuxRailsValue / totalAuxRailsValue + 1);
			rightAuxValueTextField.setColumns(valueTextLength *
				rightAuxRailsValue / totalAuxRailsValue + 1);
			leftAuxValuePanel.add(leftAuxValueTextField);
			rightAuxValuePanel.add(rightAuxValueTextField);
			if (testValue == 0) {
				info1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
				info1Panel.add(leftAuxValuePanel);
				info1Panel.add(rightAuxValuePanel);
			}
		}
		if (!controlValue)
			info1Panel.setBackground(bgColor);
		if (testValue == 0) {
			valuePanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			valueTextField.setColumns(valueTextLength);
			//valueTextField.setEditable(false);
			valuePanel.add(valueTextField);
			info2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			if (!controlValue)
				info2Panel.setBackground(bgColor);
			info2Panel.add(valuePanel);
		}
		topPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.CENTER));
		/*topPanel.add(info0Panel);*/
		if (controlValue) {
			topPanel.add(sliderPanel);
			topPanel.add(comboBoxPanel);
		} else
			topPanel.setBackground(bgColor);
		if (testValue == 0 && leeValue) {
			topPanel.add(info1Panel);
		}
		if (testValue == 0)
			topPanel.add(info2Panel);
		abacus = new Abacus(this,
			fgColor, bgColor, borderColor,
			primaryBeadColor, secondaryBeadColor, highlightBeadColor,
			primaryRailColor, secondaryRailColor, highlightRailColor,
			delayValue, railsValue, verticalValue,
			colorSchemeValue, slotValue,
			diamondValue, railIndexValue,
			topOrientValue, bottomOrientValue,
			topNumberValue, bottomNumberValue,
			topFactorValue, bottomFactorValue,
			topSpacesValue, bottomSpacesValue,
			topPieceValue, bottomPieceValue,
			topPiecePercentValue, bottomPiecePercentValue,
			shiftPercentValue, subdeckValue, subbeadValue,
			signValue, decimalPositionValue,
			groupValue, groupSizeValue,
			decimalCommaValue, baseValue, subbaseValue,
			anomalyValue, shiftAnomalyValue,
			anomalySqValue, shiftAnomalySqValue,
			displayBaseValue, (pressOffsetValue) ? 1 : 0,
			romanNumeralsValue, latinValue,
			ancientRomanValue, modernRomanValue,
			modeValue, submodeValue);
		if (leeValue) {
		int auxModeValue = AbacusInterface.Modes.Japanese.ordinal();
		leftAuxAbacus = new Abacus(this,
			fgColor, bgColor, borderColor,
			leftAuxBeadColor, secondaryBeadColor, highlightBeadColor,
			primaryRailColor, secondaryRailColor, highlightRailColor,
			delayValue, leftAuxRailsValue, verticalValue,
			colorSchemeValue, slotValue,
			diamondValue, railIndexValue,
			topOrientValue, bottomOrientValue,
			topNumberValue, bottomNumberValue,
			topFactorValue, bottomFactorValue,
			topSpacesValue, bottomSpacesValue,
			topPieceValue, bottomPieceValue,
			topPiecePercentValue, bottomPiecePercentValue,
			shiftPercentValue, subdeckValue, subbeadValue,
			signValue, decimalPositionValue,
			groupValue, groupSizeValue,
			decimalCommaValue, baseValue, subbaseValue,
			anomalyValue, shiftAnomalyValue,
			anomalySqValue, shiftAnomalySqValue,
			displayBaseValue, (pressOffsetValue) ? 1 : 0,
			romanNumeralsValue, latinValue,
			ancientRomanValue, modernRomanValue,
			auxModeValue, submodeValue);
		rightAuxAbacus = new Abacus(this,
			fgColor, bgColor, borderColor,
			rightAuxBeadColor, secondaryBeadColor, highlightBeadColor,
			primaryRailColor, secondaryRailColor, highlightRailColor,
			delayValue, rightAuxRailsValue, verticalValue,
			colorSchemeValue, slotValue,
			diamondValue,railIndexValue,
			topOrientValue, bottomOrientValue,
			topNumberValue, bottomNumberValue,
			topFactorValue, bottomFactorValue,
			topSpacesValue, bottomSpacesValue,
			topPieceValue, bottomPieceValue,
			topPiecePercentValue, bottomPiecePercentValue,
			shiftPercentValue, subdeckValue, subbeadValue,
			signValue, decimalPositionValue,
			groupValue, groupSizeValue,
			decimalCommaValue, baseValue, subbaseValue,
			anomalyValue, shiftAnomalyValue,
			anomalySqValue, shiftAnomalySqValue,
			displayBaseValue, (pressOffsetValue) ? 1 : 0,
			romanNumeralsValue, latinValue,
			ancientRomanValue, modernRomanValue,
			auxModeValue, submodeValue);
		}
		startPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		if (!controlValue)
			startPanel.setBackground(bgColor);
		startPanel.add(topPanel);
		secondaryPanel.setLayout(new BorderLayout());
		secondaryPanel.add(startPanel, BorderLayout.PAGE_START);
		canvasPanel.setBackground(bgColor);
		if (leeValue) {
			leftAuxCanvasPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			rightAuxCanvasPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			canvasPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			leftAuxCanvasPanel.add(leftAuxAbacus);
			rightAuxCanvasPanel.add(rightAuxAbacus);
			canvasPanel.add(abacus);
			JPanel leePanel = new JPanel();
			JPanel extraPanel = new JPanel();
			leePanel.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.anchor = GridBagConstraints.FIRST_LINE_START;
			c.weightx = 0.5;
			c.gridx = 0;
			c.gridy = 0;
			leePanel.add(leftAuxCanvasPanel, c);
			c.anchor = GridBagConstraints.FIRST_LINE_END;
			c.gridx = 1;
			c.gridy = 0;
			leePanel.add(rightAuxCanvasPanel, c);
			c.weightx = 0.0;
			c.gridwidth = 2;
			c.anchor = GridBagConstraints.PAGE_START;
			c.gridx = 0;
			c.gridy = 1;
			leePanel.add(canvasPanel, c);
			extraPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			extraPanel.add(leePanel);
			secondaryPanel.add(extraPanel, BorderLayout.CENTER);
			extraPanel.setBackground(bgColor);
		} else {
			canvasPanel.setBackground(bgColor);
			canvasPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			canvasPanel.add(abacus);
			secondaryPanel.add(canvasPanel, BorderLayout.CENTER);
		}
		mainPanel.add(secondaryPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		if (controlValue) {
			railSlider.addChangeListener(new RailChangeListener());
			modeComboBox.addActionListener(new ComboBoxActionListener());
			baseSlider.addChangeListener(
				new AbacusBaseChangeListener());
			displaySlider.addChangeListener(
				new DisplayBaseChangeListener());
		}
		if (leeValue) {
			leftAuxValueTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					StringBuffer mathBuffer = new
						StringBuffer(leftAuxValueTextField.getText());

					leftAuxAbacusCalc.calculateAndDisplay(leftAuxAbacus,
						cleanString(mathBuffer).toString(),
						AbacusInterface.LEFT_AUX);
				}
			});
			leftAuxCanvasPanel.addMouseListener(new MouseAdapter() {
				public void mouseEntered(MouseEvent event) {
					leftAuxAbacus.requestFocus();
				}
				public void mouseExited(MouseEvent event) {
					leftAuxAbacus.requestFocus();
				}
				public void mousePressed(MouseEvent event) {
					leftAuxAbacus.requestFocus();
				}
			});
			rightAuxValueTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					StringBuffer mathBuffer = new
						StringBuffer(rightAuxValueTextField.getText());

					rightAuxAbacusCalc.calculateAndDisplay(rightAuxAbacus,
						cleanString(mathBuffer).toString(),
						AbacusInterface.RIGHT_AUX);
				}
			});
			rightAuxCanvasPanel.addMouseListener(new MouseAdapter() {
				public void mouseEntered(MouseEvent event) {
					rightAuxAbacus.requestFocus();
				}
				public void mouseExited(MouseEvent event) {
					rightAuxAbacus.requestFocus();
				}
				public void mousePressed(MouseEvent event) {
					rightAuxAbacus.requestFocus();
				}
			});
		}
		valueTextField.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				StringBuffer mathBuffer = new
					StringBuffer(valueTextField.getText());

				abacusCalc.calculateAndDisplay(abacus,
					cleanString(mathBuffer).toString(),
					AbacusInterface.PRIMARY);
			}
		});
		canvasPanel.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent event) {
				abacus.requestFocus();
			}
			public void mouseExited(MouseEvent event) {
				abacus.requestFocus();
			}
			public void mousePressed(MouseEvent event) {
				abacus.requestFocus();
			}
		});
		submodeValue = generator.nextInt(AbacusInterface.MAX_MUSEUMS);
		/* abacus.setSize(getSize().width,
			getSize().height - topPanel.getSize().height); */
		if (leeValue) {
			int newWidth = leftAuxRailsValue * widthValue /
				totalAuxRailsValue;
			int newHeight = heightValue / 4;

			leftAuxAbacus.setSize(newWidth, newHeight);
			rightAuxAbacus.setSize(widthValue - newWidth,
				newHeight);
			abacus.setSize(widthValue, heightValue - newHeight);
		} else {
			abacus.setSize(widthValue, heightValue);
		}
		if (!getApplication() && testValue != 0) {
			forceDemoParams();
			abacus.checkBeads();
			abacus.resetBeads();
			AbacusTest abacusTest = new AbacusTest(this, abacus);
			abacusTest.getTest();
		}
	}

	public static void main(String args[]) {
		String laf = UIManager.getCrossPlatformLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(laf);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
		URL url = AbacusApplet.class.getResource(
			(nl.compareTo("\n") == 0) ?
			AbacusInterface.LARGE_ICON :
			AbacusInterface.TINY_ICON);

		argumentParser = new ArgumentParser(args);
		if (argumentParser.hasOption("?")) {
			System.out.println("usage:" + nl +
"\t[-windowWidth={int}] [-windowHeight={int}]" + nl +
"\t[-primaryBeadColor={int}] [-leftAuxBeadColor={int}]" + nl +
"\t[-rightAuxBeadColor={int}] [-secondaryBeadColor={int}]" + nl +
"\t[-highlightBeadColor={int}] [-primaryRailColor={int}]" + nl +
"\t[-secondaryRailColor={int}] [-highlightRailColor={int}]" + nl +
"\t[-borderColor={int}] [-fgColor={int}] [-bgColor={int}]" + nl +
"\t[-sound={bool}] [-delay={int}] [-script={bool}]" + nl +
"\t[-rightToLeftAdd={bool}] [-rightToLeftMult={bool}]" + nl +
"\t[-control={bool}] [-lee={bool}] [-rails={int}]" + nl +
"\t[-leftAuxRails={int}] [-rightAuxRails={int}] [-vertical={bool}]" + nl +
"\t[-colorScheme={int}] [-slot={bool}] [-diamond={bool}]" + nl +
"\t[-railIndex={int}] [-topOrient={bool}] [-bottomOrient={bool}]" + nl +
"\t[-topNumber={int}] [-bottomNumber={int}] [-topFactor={int}]" + nl +
"\t[-bottomFactor={int}] [-topSpaces={int}] [-bottomSpaces={int}]" + nl +
"\t[-topPiece={int}] [-bottomPiece={int}] [-topPiecePercent={int}]" + nl +
"\t[-bottomPiecePercent={int}] [-shiftPercent={int}]" + nl +
"\t[-subdeck={int}] [-subbead={int}] [-sign={bool}]" + nl +
"\t[-decimalPosition={int}] [-groupSize={int}] [-group={bool}]" + nl +
"\t[-decimalComma={bool}] [-base={int}] [-eighth={bool}]" + nl +
"\t[-anomaly={int}] [-shiftAnomaly={bool}] [-anomalySq={int}]" + nl +
"\t[-shiftAnomalySq={bool}] [-displayBase={int}]" + nl +
"\t[-pressOffset={bool}] [-romanNumerals={bool}]" + nl +
"\t[-latin={bool}] [-ancientRoman={bool}] [-modernRoman={bool}]" + nl +
"\t[-format={chinese|japanese|korean|roman|russian|danish|generic}]" + nl +
"\t[-museum={it|uk|fr}]");
			return;
		}
		AbacusApplet abacusApplet = new AbacusApplet();
		JFrame frame = new JFrame(AbacusInterface.TITLE);
		abacusApplet.init();
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.setIconImage(Toolkit.getDefaultToolkit().getImage(url));
		frame.add(abacusApplet);
		frame.setBounds(0, 0,
			abacusApplet.getSize().width, abacusApplet.getSize().height);
		frame.pack();
		frame.setVisible(true);
		if (abacusApplet.testValue != 0) {
			abacusApplet.forceDemoParams();
			abacusApplet.abacus.checkBeads();
			abacusApplet.abacus.resetBeads();
			AbacusTest abacusTest = new AbacusTest(abacusApplet, abacusApplet.abacus);
			abacusTest.getTest();
		}
	}
}
