/*
 * @(#)AbacusCanvas.java
 *
 * Copyright 1994 - 2011  David A. Bagley, bagleyd@tux.org
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.applet.*;
import java.awt.*;
import java.util.Random;
import java.util.StringTokenizer;

/**
 * The <code>AbacusCanvas</code> class does all the graphics for
 * <code>AbacusApplet</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd@tux.org
 * @author	http:/www.tux.org/~bagleyd/abacus.html
 */

public class AbacusCanvas extends Canvas {
	private static final long serialVersionUID = 42L;
	public static final String nl = System.getProperty("line.separator");
	static final boolean debug = false;
	static final int PLACE_SETTING = 2;
	static final int TOP = 1;
	static final int BOTTOM = 0;
	static final int UP = 1;
	static final int DOWN = 0;
	static final int MAX_DECKS = 2;
	static final int MAX_SLICES = 10;
	static final int QUARTERS = 4;
	static final int QUARTER_PERCENTS = 4;
	static final int TWELFTHS = 12;
	static final int EIGHTHS = 8;
	/* I guess the next one could be even more if you have a weird config */
	static final int CARRY = 1;
	static final int NUM_DEGREES = 360;

	static final String DATAFILE = "abacus.dat";
	static final String SYMBOL = ":";

	Color       foreground, background;
	Color       borderColor, symbolColor;
	Color[]     beadColor = new Color[3];
	Color[]     railColor = new Color[3];
	Color       currentForeground;
	AbacusDeck[] decks = null;
	AbacusSubdeck[] subdecks = null;
	AbacusMath  abacusMath = new AbacusMath();
	int         currentDeck = AbacusInterface.ACTION_IGNORE;
	int         currentRail, currentPosition;
	int         currentSpace = -1;
	int         rails;	/* number of columns of beads */
	int         base, displayBase;	/* 10 usually */
	int         shiftPercent, shiftAnomaly, shiftAnomalySq;
	int         groupSize, anomaly, anomalySq, subdeck, subbead;
	int         delay, numSlices, mode, submode, colorScheme;
	int         decimalPosition, decimalShift, numDigits;
	int         width, railWidth;
	int         midBeginX, midHeight, midBarY;
	StringBuffer digits;
	String      multiplier, divisor;
	boolean     demo = false, aux = false;
	int         topNumber, bottomNumber, topFactor, bottomFactor;
	int         topSpaces, bottomSpaces; /* spaces between beads */
	boolean     topOrient, bottomOrient;
	boolean     precedenceBegin, precedenceEnd, signBead = false;
	int         subbase = TWELFTHS;
	int         railIndex = 0;
	int         pressOffsetX = 1, pressOffsetY = 0;
	Point       coreSize;
	Point       pos;
	Point       frameSize, beadSize;
	Point       delta, offset;

	Applet      applet = null;
	boolean     firstPaint = false, resizePaint = false;
	boolean     framePaint = false, resetPaint = false;
	boolean     selectPaint = false, releasePaint = false;
	boolean     mouseDown = false;
	boolean     enoughRails = true, startOperation, negativeResult;
	boolean     sign = false, minusSign = false;
	boolean     romanNumerals = false, latin = false;
	boolean     ancientRoman = false, modernRoman = false;
	boolean     group = false, decimalComma = false;
	boolean     carryAnomaly = false, carryAnomalySq = false;
	boolean     vertical = false, slot = false, diamond = false;
	Random      generator;
	AbacusDemo  abacusDemo = null;
	String      stringSave = null, expression, operateOn, origExpression;
	String      lhOperand = "";
	String      divResult = "", prevResult = "";
	char        operand;
	Color[]     beadShade = new Color[12];
	String      museum = "--";
	String      format = "Generic";
	StringTokenizer token = null;
	Image[][][]   bufferedBeadImage = null;
	Graphics[][][] bufferedBeadGraphics = null;

	protected String paramString() {
		return super.paramString() +
		",foreground=" + foreground +
		",background=" + background +
		",borderColor" + borderColor +
		",primaryBeadColor" + beadColor[0] +
		",secondaryBeadColor" + beadColor[1] +
		",highlightBeadColor" + beadColor[2] +
		",primaryRailColor" + railColor[0] +
		",secondaryRailColor" + railColor[1] +
		",highlightRailColor" + railColor[2] +
		",delay=" + delay +
		",rails=" + rails +
		",vertical=" + vertical +
		",colorScheme=" + colorScheme +
		",slot=" + slot +
		",diamond=" + diamond +
		",railIndex=" + railIndex +
		",topOrient=" + topOrient +
		",bottomOrient=" + bottomOrient +
		",topNumber=" + topNumber +
		",bottomNumber=" + bottomNumber +
		",topFactor=" + topFactor +
		",bottomFactor=" + bottomFactor +
		",topSpaces=" + topSpaces +
		",bottomSpaces=" + bottomSpaces +
		",topPiece=" + decks[TOP].piece +
		",bottomPiece=" + decks[BOTTOM].piece +
		",topPiecePercent=" + decks[TOP].piecePercent +
		",bottomPiecePercent=" + decks[BOTTOM].piecePercent +
		",shiftPercent=" + shiftPercent +
		",subdeck=" + subdeck +
		",subbead=" + subbead +
		",sign=" + sign +
		",decimalPosition=" + decimalPosition +
		",group=" + group +
		",groupSize=" + groupSize +
		",decimalComma=" + decimalComma +
		",base=" + base +
		",subbase=" + subbase +
		",anomaly=" + anomaly +
		",shiftAnomaly=" + shiftAnomaly +
		",anomalySq=" + anomalySq +
		",shiftAnomalySq=" + shiftAnomalySq +
		",displayBase=" + displayBase +
		",pressOffsetY=" + pressOffsetY +
		",romanNumberals=" + romanNumerals +
		",latin=" + latin +
		",ancientRoman=" + ancientRoman +
		",modernRoman=" + modernRoman +
		",mode=" + mode +
		",submode=" + submode;
	}

	public void clearAbacus() {
		/* Should check if one really wants to destroy calculations. */
		if (mouseDown)
			return;
		resetPaint = true;
		repaint();
	}

	public void clearWithQueryAbacus() {
		if (!isEmptyCounter() && !aux) {
			clearAbacus();
		}
	}

	public int getMode() {
		return this.mode;
	}

	public void setMode(int mode) {
		this.mode = mode;
	}

	public int getBase() {
		return this.base;
	}

	public int getDisplayBase() {
		return this.displayBase;
	}

	public int getDecimalPosition() {
		return this.decimalPosition;
	}

	public int getShiftPercent() {
		return this.shiftPercent;
	}

	public boolean getSign() {
		return this.sign;
	}

	public int getAnomaly() {
		return this.anomaly;
	}

	public int getAnomalySq() {
		return this.anomalySq;
	}

	public int getShiftAnomaly() {
		return this.shiftAnomaly;
	}

	public int getShiftAnomalySq() {
		return this.shiftAnomalySq;
	}

	public int getTopFactor() {
		return this.decks[AbacusCanvas.TOP].factor;
	}

	public int getBottomFactor() {
		return this.decks[AbacusCanvas.BOTTOM].factor;
	}

	public int getTopNumber() {
		return this.decks[AbacusCanvas.TOP].number;
	}

	public int getBottomNumber() {
		return this.decks[AbacusCanvas.BOTTOM].number;
	}

	public Abacus getAbacus(int aux) {
		return ((AbacusApplet)applet).getAbacus(aux);
	}

	public void setValuesBaseAbacus(int base, int displayBase) {
		this.base = base;
		this.displayBase = displayBase;
		initializeAbacus();
		resizePaint = true;
		repaint();
	}

	public void setValuesModeAbacus(
			boolean topOrient, boolean bottomOrient,
			int topNumber, int bottomNumber,
			int topFactor, int bottomFactor,
			int topSpaces, int bottomSpaces,
			int base, int displayBase, int mode) {
		this.topOrient = topOrient;
		this.bottomOrient = bottomOrient;
		this.topNumber = topNumber;
		this.bottomNumber = bottomNumber;
		this.topFactor = topFactor;
		this.bottomFactor = bottomFactor;
		this.topSpaces = topSpaces;
		this.bottomSpaces = bottomSpaces;
		this.base = base;
		this.displayBase = displayBase;
		this.mode = mode;
		initializeAbacus();
		resizePaint = true;
		repaint();
	}

	public void setValuesTeachAbacus(int displayBase, boolean sign,
			int anomaly, int anomalySq) {
		this.displayBase = displayBase;
		this.sign = sign;
		this.anomaly = anomaly;
		this.anomalySq = anomalySq;
		initializeAbacus();
		resizePaint = true;
		repaint();
	}

	boolean abacusMoveFast(int deck, int rail, int number) {
		//no delay - used for chinese -> japanese conversion
		if (!enoughRails)
			return false;
		if (checkMove(deck, rail, number, false)) {
			moveBeadsByValue(deck, rail + decimalPosition, number,
				false);
			return true;
		} /*else {
			if (number > 0)
				if (deck > 0) {
					abacusMoveFast(TOP, rail, -1);
					abacusMoveFast(BOTTOM, rail + 1, 1);
				} else {
					//move the bead on upper deck
					abacusMoveFast(TOP, rail, 1);
					//take away beads.
					abacusMoveFast(BOTTOM, rail,
						number - 5);
				}
			else
				if (deck > 0) {
					abacusMoveFast(TOP, rail, 2 + number);
					abacusMoveFast(BOTTOM, rail + 1, -1);
				} else {
					//move the bead on upper deck
					abacusMoveFast(TOP, rail, -1);
					//take away beads.
					abacusMoveFast(BOTTOM, rail, 5 + number);
				}
		}*/
		return false;
	}

	void abacusMove(int aux, int deck, int rail, int number) {
		/*this.aux = aux;
		this.deck = deck;
		this.rail = rail;
		this.number = number;*/
		((AbacusApplet)applet).callbackAbacusDemo(aux, deck, rail, number);
	}

	void directMove(int deck, int rail, int number) {
		if (checkMove(deck, rail, number, true)) {
			moveBeadsByValue(deck, rail + decimalPosition, number,
				false);
		}
	}

	/* This is fast for small i. */
	int rootInt(int i, int n) {
		int j = 0, k;
		int absI = (i >= 0) ? i : -i;
		int prod;

		if (n < 0 || i == 0 || (n % 2 == 0 && i < 0))
			return 0;
		if (n == 1)
			return i;
		absI = (i >= 0) ? i : -i;
		do {
			prod = 1;
			j++;
			for (k = 0; k < n; k++)
				prod *= j;
		} while (prod <= absI);
		return (i == absI) ? (j - 1) : (1 - j);
	}

	int convertBaseToBottom(int base) {
		int j;

		for (j = rootInt(base, 2); j > 1; j--) {
			if (base % j == 0) {
				return (base / j);
			}
		}
		return base;
	}

	/* This is setup for Roman abacus of 3 subdecks
	 * deck == 0 => 1/12 * pieceFactor
	 * deck == 1 => 1/4 * pieceFactor
	 * deck == 2 => 1/2 * pieceFactor
	 * For other subdecks its more whimsical. */
	int convertRomanFactor(int deck) {
		int b = decks[BOTTOM].piece;

		if (decks[TOP].piece != 0)
			b *= decks[TOP].piece;
		if (subbase == EIGHTHS) {
			if (deck <= 1)
				return 1;
			return deck * b / (subdeck + 2);
		}
		if (deck == 0)
			return 1;
		return deck * b / (subdeck + 1);
	}

	int checkBottomSpace() {
		return decks[BOTTOM].spaces + convertBaseToBottom(base) - 1;
	}

	public boolean checkPiece() {
		return (this.decks[BOTTOM].piece != 0);
	}

	public boolean checkPiecePercent() {
		return (this.decks[BOTTOM].piece != 0 &&
			this.decks[BOTTOM].piecePercent != 0);
	}

	public boolean checkSubdeck(int position) {
		return (decks[BOTTOM].piece != 0 &&
			decks[BOTTOM].piecePercent == 0 &&
			slot && subdeck != 0 &&
			decks[BOTTOM].room >= subbead +
			subdeck * AbacusInterface.SUBDECK_SPACE &&
			decimalPosition == position);
	}

	public boolean checkAnomaly() {
		return (anomaly != 0);
	}

	public boolean checkAnomalySq() {
		return (anomalySq != 0);
	}

	int numberPieces(int deck) {
		int pieces = 0;

		if (deck == BOTTOM) {
			pieces = decks[BOTTOM].piece;
			if (decks[TOP].number == 0 &&
					decks[TOP].piece != 0)
				pieces *= decks[TOP].piece;
			if (decks[BOTTOM].number ==
					decks[TOP].factor - 1)
				pieces -= 1;
		} else {
			if (decks[TOP].number != 0 && decks[TOP].piece != 0) {
				pieces = decks[TOP].piece;
				if ((decks[TOP].number + 1) *
						decks[TOP].factor == base)
					pieces -= 1;
			}
		}
		return pieces;
	}

	int numberPiecePercents(int deck) {
		int piecePercents = 0;

		if (deck == BOTTOM) {
			piecePercents = decks[BOTTOM].piecePercent;
				if (decks[TOP].number == 0 &&
						decks[TOP].piecePercent != 0)
					piecePercents *= decks[TOP].piecePercent;
				if (decks[BOTTOM].number ==
						decks[TOP].factor - 1)
					piecePercents -= 1;
		} else {
			if (decks[TOP].number != 0 &&
					decks[TOP].piecePercent != 0) {
				piecePercents = decks[TOP].piecePercent;
				if ((decks[TOP].number + 1) *
						decks[TOP].factor == base)
					piecePercents -= 1;
			}
		}
		return piecePercents;
	}

	int valuePiece() {
		int sum = 0, localBase = 1;
		int factor = 1, rail = decimalPosition - 1;

		if (decks[BOTTOM].piece != 0) {
			int deck;

			for (deck = BOTTOM; deck <= TOP; deck++) {
				int piecePosition = decks[deck].position[rail];

				localBase = decks[BOTTOM].piece;
				if (deck == TOP) {
					if (decks[TOP].piece == 0) {
						break;
					}
					factor = decks[BOTTOM].piece;
					localBase *= decks[TOP].piece;
				}
				if (decks[deck].orientation) {
					sum += (numberPieces(deck) - piecePosition) * factor;
				} else {
					sum += piecePosition * factor;
				}
			}
		}
		return sum % localBase;
	}

	int valuePiecePercent() {
		int sum = 0, localBase = 1;
		int factor = 1, rail = decimalPosition - shiftPercent - 1 -
			((decks[BOTTOM].piece == 0) ? 0 : 1);

		if (decks[BOTTOM].piece != 0 &&
				decks[BOTTOM].piecePercent != 0) {
			int deck;

			for (deck = BOTTOM; deck <= TOP; deck++) {
				int piecePercentPosition = decks[deck].position[rail];

				localBase = decks[BOTTOM].piecePercent;
				if (deck == TOP) {
					if (decks[TOP].piecePercent == 0) {
						break;
					}
					factor = decks[BOTTOM].piecePercent;
					localBase *= decks[TOP].piecePercent;
				}
				if (decks[deck].orientation) {
					sum += (numberPiecePercents(deck) - piecePercentPosition) * factor;
				} else {
					sum += piecePercentPosition * factor;
				}
			}
		}
		return sum % localBase;
	}

	int valueSubdeck() {
		int sum = 0;

		if (checkSubdeck(3)) {
			int deck;

			for (deck = 0; deck < subdeck; deck++) {
				if (decks[BOTTOM].orientation) {
					sum += (subdecks[deck].number - subdecks[deck].position) * convertRomanFactor(deck);
				} else {
					sum += subdecks[deck].position * convertRomanFactor(deck);
				}
			}
		}
		return sum;
	}

	void setSpace(int deck) {
		int room, number;

		room = decks[deck].number + decks[deck].spaces;
		if (decks[BOTTOM].piece != 0) {
			number = numberPieces(deck);
			if (room <= number)
				room = number + 1;
			if (decks[BOTTOM].piecePercent != 0) {
				number = numberPiecePercents(deck);
				if (room <= number)
					room = number + 1;
			}
			if (deck == BOTTOM && checkSubdeck(3)) {
				number = subbead + subdeck *
					AbacusInterface.SUBDECK_SPACE - 1;
				if (room <= number)
					room = number + 1;
			}
		}
		if (decks[deck].number != 0) {
			if (room > decks[deck].number +
					decks[deck].spaces)
				decks[deck].spaces = room -
					decks[deck].number;
			decks[deck].room = decks[deck].number +
				decks[deck].spaces;
		}
	}

	int numberSubbeads(int localSubdecks) {
		if (subdeck != 0)
			return subbead / subdeck +
				((subbead % subdeck - localSubdecks <= 0) ? 0 : 1);
		return 0;
	}

	int numberSubbeadsOffset(int localSubdecks) {
		int deck, nOffset = 0;
		int space = 1;

		if (localSubdecks < 0)
			return subbead +
				subdeck * AbacusInterface.SUBDECK_SPACE;
		for (deck = 0; deck < subdeck - 1 - localSubdecks; deck++) {
			nOffset += numberSubbeads(subdeck - 1 - deck) +
				space;
		}
		return nOffset;
	}

	int positionSubdeck(int j) {
		int b1, b2, subj, d;

		subj = j;
		b1 = b2 = 0;
		for (d = subdeck - 1; d >= 0; d--) {
			b1 = b2;
			b2 = numberSubbeadsOffset(d - 1);
			if (subj <= b2) {
				subj = subj - b1;
				break;
			}
		}
		return subj;
	}

	int subdeckPosition(int j) {
		int subj, d;

		subj = j;
		for (d = subdeck - 1; d >= 0; d--) {
			if (subj <= numberSubbeadsOffset(d - 1)) {
				break;
			}
		}
		return d;
	}

/*
	int numberSubdeck(int subbeads) {
		int deck, offset = 0;
		int space = 1;

		for (deck = 0; deck < subdeck; deck++) {
			offset += numberSubbeads(deck) + space;
			if (offset >= subbeads)
				return deck;
		}
		return subdeck;
	}
*/

	/* Pure colors can get brighter */
	Color brighter(Color color) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		final double FACTOR = 0.8;

		/* From 2D group:
		 * 1. black.brighter() should return grey
		 * 2. applying brighter to blue will always return blue, brighter
		 * 3. non pure color (non zero rgb) will eventually return white
		 */
		int i = (int) ((1 - FACTOR) * 255);

		if (r >= 0 && r < i)
			r = i;
		if (g >= 0 && g < i)
			g = i;
		if (b >= 0 && b < i)
			b = i;
		return new Color(Math.min((int) (r / FACTOR), 255),
			Math.min((int) (g / FACTOR), 255),
			Math.min((int) (b / FACTOR), 255));
	}

	void drawCircle(Graphics g, int diameter, int x, int y) {
		int xCenter = x - diameter / 2, yCenter = y - diameter / 2;

		if (diameter <= 0)
			return;
		g.drawOval(xCenter, yCenter, diameter + 1, diameter + 1);
		g.drawOval(xCenter, yCenter, diameter, diameter);
		g.drawOval(xCenter + 1, yCenter, diameter, diameter);
		g.drawOval(xCenter, yCenter + 1, diameter, diameter);
		g.drawOval(xCenter + 1, yCenter + 1, diameter, diameter);
	}

	void fillCircle(Graphics g, int diameter, int x, int y) {
		if (diameter <= 0)
			return;
		g.fillOval(x - diameter / 2, y - diameter / 2,
			diameter + 1, diameter + 1);
	}

	/* Not used as it takes too much time */
	void fill3DCircle(Graphics g, int diameter, int x, int y,
			boolean raised, boolean second) {
		int xCenter = x - diameter / 2, yCenter = y - diameter / 2;
		if (diameter <= 0)
			return;

		Color c = g.getColor();
		Color darker = c.darker();

		if (!raised) {
			g.setColor(darker);
		}
		g.fillOval(xCenter, yCenter, diameter, diameter);
		if (raised) {
			if (!second) {
				g.setColor(brighter(c));
				g.drawArc(xCenter, yCenter, diameter, diameter,
					90, 90);
			}
			g.setColor(c.darker());
			g.drawArc(xCenter, yCenter, diameter, diameter,
				270, 90);
		} else {
			g.setColor(brighter(c));
			g.drawArc(xCenter, yCenter, diameter, diameter,
				270, 90);
			if (!second) {
				g.setColor(c.darker());
				g.drawArc(xCenter, yCenter, diameter, diameter,
					90, 90);
			}
		}
		g.setColor(c);
	}

	void fill3DRect(Graphics g, int x, int y, int width, int height,
		boolean raised, boolean updown) {

		Color c = g.getColor();
		Color brighter = brighter(c);
		Color darker = c.darker();

		if (!raised) {
			g.setColor(darker);
		}
		if (updown) {
			g.fillRect(x + 1, y, width - 2, height);
			g.setColor(raised ? brighter : darker);
			g.drawLine(x, y, x + width - 1, y);
			g.setColor(raised ? darker : brighter);
			g.drawLine(x, y + height - 1, x + width - 1, y + height - 1);
		} else {
			g.fillRect(x, y + 1, width, height - 2);
			g.setColor(raised ? brighter : darker);
			g.drawLine(x, y, x, y + height - 1);
			g.setColor(raised ? darker : brighter);
			g.drawLine(x + width - 1, y, x + width - 1, y + height - 1);
		}
		g.setColor(c);
	}

	char decimalChar() {
		return ((decimalComma) ? ',' : '.');
	}

	char groupChar() {
		return ((decimalComma) ? '.' : ',');
	}

	void setSubmodeFromMuseum() {
		if (museum.equalsIgnoreCase("it")) {
			submode = AbacusInterface.IT;
		} else if (museum.equalsIgnoreCase("uk")) {
			submode = AbacusInterface.UK;
		} else if (museum.equalsIgnoreCase("fr")) {
			submode = AbacusInterface.FR;
		} else {
			submode = generator.nextInt(AbacusInterface.MAX_MUSEUMS);
		}
	}

	boolean submodeSlotsSeparate(int localSubmode) {
		return (localSubmode == AbacusInterface.UK);
	}

	void checkBeads() {
		if (railIndex != 0)
			railIndex = 1;
		if (shiftPercent <= 0) {
			shiftPercent = AbacusInterface.DEFAULT_SHIFT_PERCENT;
		}
		if (decimalPosition < 0) {
			decimalPosition = 0;
		}
		if (decimalPosition >= rails) {
			decimalPosition = rails - 1;
		}
		if (shiftPercent >= rails) {
			shiftPercent = 2;
		}
		if (decks[BOTTOM].piece > AbacusInterface.MAX_BASE) {
			System.out.println(
				"Bottom Piece must be less than or equal to " +
				AbacusInterface.MAX_BASE);
			decks[BOTTOM].piece = 0;
		} else if (decks[BOTTOM].piece != 0 &&
				decks[BOTTOM].piece < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Bottom Piece must be greater than or equal to " +
				AbacusInterface.MIN_BASE + ", or 0");
			decks[BOTTOM].piece = 0;
		}
		if (decks[TOP].piece != 0 &&
				decks[TOP].piece < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Top Piece must be greater than or equal to " +
				AbacusInterface.MIN_BASE + ", or 0");
			decks[TOP].piece = 0;
		}
		if (decks[BOTTOM].piecePercent > AbacusInterface.MAX_BASE) {
			System.out.println(
				"Bottom Piece Percent must be less than or equal to " +
				AbacusInterface.MAX_BASE);
			decks[BOTTOM].piecePercent = 0;
		} else if (decks[BOTTOM].piecePercent != 0 &&
				decks[BOTTOM].piecePercent < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Bottom Piece Percent must be greater than or equal to " +
				AbacusInterface.MIN_BASE + ", or 0");
			decks[BOTTOM].piecePercent = 0;
		}
		if (decks[TOP].piecePercent != 0 &&
				decks[TOP].piecePercent < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Top Piece Percent must be greater than or equal to " +
				AbacusInterface.MIN_BASE + ", or 0");
			decks[TOP].piecePercent = 0;
		}
		if (base > AbacusInterface.MAX_BASE) {
			System.out.println(
				"Base must be less than or equal to " +
				AbacusInterface.MAX_BASE);
			base = AbacusInterface.DEFAULT_BASE;
		} else if (base < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Base must be greater than or equal to " +
				AbacusInterface.MIN_BASE);
			base = AbacusInterface.DEFAULT_BASE;
		} else if (base != AbacusInterface.DEFAULT_BASE && demo) {
			System.out.println(
				"Base must be equal to " +
				AbacusInterface.DEFAULT_BASE + ", for demo");
			base = AbacusInterface.DEFAULT_BASE;
		}
		if (decks[BOTTOM].piece != 0 &&
				(checkBottomSpace() < decks[BOTTOM].piece)) {
			System.out.println(
				"Bottom Spaces must be large enough with base when piece set");
			decks[BOTTOM].spaces -=
				(checkBottomSpace() - decks[BOTTOM].piece);
		}
		if (decks[BOTTOM].piecePercent > 1 &&
				(checkBottomSpace() < decks[BOTTOM].piecePercent)) {
			System.out.println(
				"Bottom Spaces must be large enough with base when piece percent set");
			decks[BOTTOM].spaces -=
				(checkBottomSpace() - decks[BOTTOM].piecePercent);
		}
		if (anomaly < 0) {
			System.out.println(
				"Anomaly must be greater than or equal to 0");
			anomaly = 0;
		}
		if (anomaly >= base) {
			System.out.println(
				"Anomaly must be less than " + base);
			anomaly = 0;
		}
		if (shiftAnomaly <= 0) {
			System.out.println(
				"Shift Anomaly must be greater than 0");
			shiftAnomaly = AbacusInterface.DEFAULT_SHIFT_ANOMALY;
		}
		if (anomalySq < 0) {
			System.out.println(
				"Anomaly Squared must be greater than or equal to 0");
			anomalySq = 0;
		}
		if (anomalySq >= base) {
			System.out.println(
				"Anomaly Squared must be less than " + base);
			anomalySq = 0;
		}
		if (shiftAnomalySq <= 0) {
			System.out.println(
				"Shift Anomaly Squared must be greater than 0");
			shiftAnomalySq = AbacusInterface.DEFAULT_SHIFT_ANOMALY;
		}
		if (displayBase > AbacusInterface.MAX_BASE) {
			System.out.println(
				"Display base must be less than or equal to " +
				AbacusInterface.MAX_BASE);
			displayBase = AbacusInterface.DEFAULT_BASE;
		} else if (displayBase < AbacusInterface.MIN_BASE) {
			System.out.println(
				"Display base must be greater than or equal to " +
				AbacusInterface.MIN_BASE);
			displayBase = AbacusInterface.DEFAULT_BASE;
		} else if (displayBase != AbacusInterface.DEFAULT_BASE && demo) {
			System.out.println(
				"Display base must be equal to " +
				AbacusInterface.DEFAULT_BASE + ", for demo");
			displayBase = AbacusInterface.DEFAULT_BASE;
		}
		if (mode == AbacusInterface.Modes.Generic.ordinal() && demo) {
			System.out.println(
				"Format must not be \"Generic\", for demo");
		}
		if (mode == AbacusInterface.Modes.Danish.ordinal()) {
			colorScheme = AbacusInterface.COLOR_HALF;
			decks[BOTTOM].factor = 1;
			decks[TOP].factor = base;
			decks[BOTTOM].number = base;
			decks[TOP].number = 0;
			decks[BOTTOM].orientation =
				AbacusInterface.DEFAULT_BOTTOM_ORIENT;
			decks[TOP].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			decks[BOTTOM].spaces =
				4 * AbacusInterface.DEFAULT_BOTTOM_SPACES;
			decks[TOP].spaces = 0;
			vertical = true;
			slot = false;
			diamond = false;
			railIndex = 1;
			setSpace(BOTTOM);
			setSpace(TOP);
		} else if (mode == AbacusInterface.Modes.Russian.ordinal()) {
			colorScheme = AbacusInterface.COLOR_MIDDLE |
				AbacusInterface.COLOR_FIRST;
			decks[BOTTOM].factor = 1;
			decks[TOP].factor = base;
			decks[BOTTOM].number = base;
			decks[TOP].number = 0;
			decks[BOTTOM].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			decks[TOP].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			decks[BOTTOM].spaces =
				AbacusInterface.DEFAULT_BOTTOM_SPACES;
			decks[TOP].spaces = 0;
			vertical = true;
			slot = false;
			diamond = false;
			railIndex = 1;
			setSpace(BOTTOM);
			setSpace(TOP);
		} else if (mode == AbacusInterface.Modes.Japanese.ordinal() ||
				mode == AbacusInterface.Modes.Roman.ordinal()) {
			colorScheme = 0;
			decks[BOTTOM].factor = 1;
			decks[TOP].factor = convertBaseToBottom(base);
			decks[BOTTOM].number = decks[TOP].factor - 1;
			decks[TOP].number = base /
				decks[TOP].factor - 1;
			decks[BOTTOM].orientation =
				AbacusInterface.DEFAULT_BOTTOM_ORIENT;
			decks[TOP].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			if (subdeck < 0)
				subdeck = AbacusInterface.DEFAULT_SUBDECKS;
			vertical = false;
			if (mode == AbacusInterface.Modes.Japanese.ordinal()) {
				slot = false;
				diamond = true;
				decks[BOTTOM].spaces =
					AbacusInterface.DEFAULT_BOTTOM_SPACES - 1;
				decks[TOP].spaces =
					AbacusInterface.DEFAULT_TOP_SPACES - 1;
			} else if (mode == AbacusInterface.Modes.Roman.ordinal()) {
				slot = true;
				diamond = false;
				decks[BOTTOM].spaces =
					AbacusInterface.DEFAULT_BOTTOM_SPACES + 1;
				decks[TOP].spaces =
					AbacusInterface.DEFAULT_TOP_SPACES + 1;
			}
			railIndex = 0;
			setSpace(BOTTOM);
			setSpace(TOP);
		} else if (mode == AbacusInterface.Modes.Korean.ordinal()) {
			colorScheme = 0;
			decks[BOTTOM].factor = 1;
			decks[TOP].factor = convertBaseToBottom(base);
			decks[BOTTOM].number = decks[TOP].factor;
			decks[TOP].number = base /
				decks[TOP].factor - 1;
			decks[BOTTOM].orientation =
				AbacusInterface.DEFAULT_BOTTOM_ORIENT;
			decks[TOP].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			decks[BOTTOM].spaces =
				AbacusInterface.DEFAULT_BOTTOM_SPACES - 1;
			decks[TOP].spaces =
				AbacusInterface.DEFAULT_TOP_SPACES - 1;
			vertical = false;
			slot = false;
			diamond = true;
			railIndex = 0;
			setSpace(BOTTOM);
			setSpace(TOP);
		} else if (mode == AbacusInterface.Modes.Chinese.ordinal()) {
			colorScheme = 0;
			decks[BOTTOM].factor = 1;
			decks[TOP].factor = convertBaseToBottom(base);
			decks[BOTTOM].number = decks[TOP].factor;
			decks[TOP].number = base /
				decks[TOP].factor;
			decks[BOTTOM].orientation =
				AbacusInterface.DEFAULT_BOTTOM_ORIENT;
			decks[TOP].orientation =
				AbacusInterface.DEFAULT_TOP_ORIENT;
			decks[BOTTOM].spaces =
				AbacusInterface.DEFAULT_BOTTOM_SPACES;
			decks[TOP].spaces = AbacusInterface.DEFAULT_TOP_SPACES;
			vertical = false;
			slot = false;
			diamond = false;
			railIndex = 0;
			setSpace(BOTTOM);
			setSpace(TOP);
		}
		if (demo && !aux) { /* Trying to keep these at a minimum... */
			if (rails < AbacusInterface.MIN_DEMO_RAILS) {
				System.out.println(
					"Number of rails must be at least " +
					AbacusInterface.MIN_DEMO_RAILS +
					", for demo");
				for (int i = 0; i < AbacusInterface.MIN_DEMO_RAILS - rails; i++)
					((AbacusApplet)applet).callbackAbacus(this,
						AbacusInterface.ACTION_INCREMENT);
				rails = AbacusInterface.MIN_DEMO_RAILS;
			}
			if (rails - decimalPosition <
					AbacusInterface.MIN_DEMO_RAILS) {
				if (decks[BOTTOM].piecePercent != 0) {
					decks[BOTTOM].piecePercent = 0;
					((AbacusApplet)applet).callbackAbacus(this,
						AbacusInterface.ACTION_QUARTER_PERCENT);
				}
				if (decks[BOTTOM].piece != 0) {
					decks[BOTTOM].piece = 0;
					((AbacusApplet)applet).callbackAbacus(this,
						AbacusInterface.ACTION_QUARTER);
				}
				decimalPosition = 0;
			}
		} else {
			if (rails < AbacusInterface.MIN_RAILS) {
				System.out.println(
					"Number of rails must be at least " +
					AbacusInterface.MIN_RAILS);
			}
			if (decks[TOP].factor < 1 ||
					decks[TOP].factor > base) {
				System.out.println(
					"Factor of Top Beads out of bounds, use 1.." +
					base);
				decks[TOP].factor = 5;
			}
			if (decks[BOTTOM].factor < 1 ||
					decks[BOTTOM].factor >
					base) {
				System.out.println(
					"Factor of Bottom Beads out of bounds, use 1.." +
					base);
				decks[BOTTOM].factor = 1;
			}
		}
		if (decks[TOP].number < 0 ||
				decks[TOP].number > base) {
			System.out.println(
				"Number of Top Beads out of bounds, use 1.." +
				base);
			decks[TOP].number = 2;
		}
		if (decks[BOTTOM].number < 0 ||
				decks[BOTTOM].number > base) {
			System.out.println(
				"Number of Bottom Beads out of bounds, use 1.." +
				base);
			decks[BOTTOM].number = 5;
		}
		if (decks[TOP].spaces < 0) {
			System.out.println(
				"Number of Top Spaces must be at least 0");
			decks[TOP].spaces = 2;
		}
		if (decks[BOTTOM].spaces < 0) {
			System.out.println(
				"Number of Bottom Spaces must be at least 0");
			decks[BOTTOM].spaces = 2;
		}
		if (decks[TOP].spaces == 0 && decks[BOTTOM].spaces == 0) {
			System.out.println(
				"Number of Top plus Bottom Spaces must be at least 1");
			decks[BOTTOM].spaces = 2;
		}
		if (groupSize < 2) {
			System.out.println(
				"Group Size must be at least 2");
			groupSize = AbacusInterface.DEFAULT_GROUP_SIZE;
		}
		if (delay < 0) {
			System.out.println("Delay must be at least 0");
			delay = -delay;
		}
	}

	boolean checkMove(int deck, int rail , int number, boolean print) {
		int deck_number = 0, deck_position = 0;

		if (deck < 0 || deck > 2) {
			if (print)
				System.out.println(
					"Corrupted deck input value " + deck +
					" out of bounds, use 0..2, ignoring");
			return false;
		}
		if (rail < -decimalPosition ||
				rail >= rails - decimalPosition) {
			if (print)
				System.out.println(
					"Number of rails too small for input, needs rail " +
					rail);
			enoughRails = false;
			return false;
		}
		enoughRails = true;
		if (deck == PLACE_SETTING) {
			/* moving decimal point */
			if (number + decimalPosition >= rails ||
					number + decimalPosition < 0) {
				System.out.println(
					"Corrupted number, input value " +
					number + " out of bounds, use " +
					(-decimalPosition) + ".." +
					(rails - decimalPosition));

				return false;
			}
			return true;
		}
		deck_number = decks[deck].number;
		deck_position = decks[deck].position[rail + decimalPosition];
		if (decks[deck].orientation &&
				(number < -deck_number + deck_position ||
				number > deck_position)) {
			if (print)
				System.out.println(
					"Corrupted number for input value " +
					number + " out of bounds, use " +
					(-deck_number + deck_position) + ".." +
					deck_position);
			return false;
		}
		if (!decks[deck].orientation &&
				(number < -deck_position ||
				number > deck_number - deck_position)) {
			if (print)
				System.out.println(
					"Corrupted number for input value " +
					(-deck_position) + ".." +
					(deck_number - deck_position));
			return false;
		}
		return true;
	}

	void resetSubdecks() {
		int deck, ndecks;

		ndecks = (subdeck <= 0) ? AbacusInterface.DEFAULT_SUBDECKS :
			subdeck;
		subdecks = new AbacusSubdeck[ndecks];
		for (deck = 0; deck < ndecks; deck++) {
			subdecks[deck] = new AbacusSubdeck(
				numberSubbeads(deck),
				AbacusInterface.SUBDECK_SPACE);
			subdecks[deck].room = subdecks[deck].number +
				subdecks[deck].spaces;
			subdecks[deck].position = (decks[BOTTOM].orientation) ?
				subdecks[deck].number : 0;
		}
	}

	void resetBeads() {
		int deck, rail;

		currentDeck = AbacusInterface.ACTION_IGNORE;
		numDigits = rails + CARRY + 1;
		for (deck = BOTTOM; deck <= TOP; deck++) {
			decks[deck].room = decks[deck].number +
				decks[deck].spaces;
			decks[deck].position = new int[rails];
			for (rail = 0; rail < rails; rail++)
				decks[deck].position[rail] =
					(decks[deck].orientation) ?
					decks[deck].number : 0;
		}
		if (sign) {
			rail = rails - 1;
			decks[BOTTOM].position[rail] =
				(decks[BOTTOM].orientation) ? 1 : 0;
		}
		if (decks[BOTTOM].piece != 0) {
			rail = decimalPosition - 1;
			decks[BOTTOM].position[rail] =
				(decks[BOTTOM].orientation) ?
				numberPieces(BOTTOM) : 0;
			if (decks[TOP].number != 0 &&
					decks[TOP].piece != 0) {
				decks[TOP].position[rail] =
					(decks[TOP].orientation) ?
					numberPieces(TOP) : 0;
			}
		}
		if (decks[BOTTOM].piece != 0 &&
				decks[BOTTOM].piecePercent != 0) {
			rail = decimalPosition - shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1);
			decks[BOTTOM].position[rail] =
				(decks[BOTTOM].orientation) ?
				numberPiecePercents(BOTTOM) : 0;
			if (decks[TOP].number != 0 &&
					decks[TOP].piecePercent != 0) {
				decks[TOP].position[rail] =
					(decks[TOP].orientation) ?
					numberPiecePercents(TOP) : 0;
			}
		}
		resetSubdecks();
		digits = new StringBuffer(numDigits);
		digits.setLength(numDigits);
		for (rail = 0; rail < numDigits - 1; rail++)
			digits = digits.insert(rail, '0');
	}

	boolean isEmptyCounter() {
		int n = 0;

		while (n < numDigits - 1) {
			if (digits.charAt(n) != '0') {
				return false;
			}
			n++;
		}
		return true;
	}

	boolean emptyCounter() {
		int n = 0;
		boolean good = true;

		while (n < numDigits - 1) {
			if (digits.charAt(n) != '0') {
				good = false;
				digits.setCharAt(n, '0');
			}
			n++;
		}
		return good;
	}

	void setCounter(int deck, int rail, int number) {
		int n = 0, s = 0, i;
		int m = 0, o = 0, half;
		int rom = AbacusMath.sizeofRoman(base, romanNumerals,
			ancientRoman);
		String st;
		StringBuffer buffer = new StringBuffer(12 + 3 * numDigits / 2 +
			rom);
		boolean anomalyActive;

		/* n digits above decimal *
		 * m digits below decimal */
		while (n < numDigits - 2 - decimalPosition &&
				digits.charAt(n) == '0')
			n++;
		while (m < decimalPosition - 1 &&
				digits.charAt(numDigits - CARRY - m) == '0')
			m++;
		while (o < numDigits - 1 && digits.charAt(o) == '0')
			o++;
		half = numDigits - decimalPosition - n - 1;
		s = (sign &&
			(((decks[BOTTOM].position[rails - 1] == 1 &&
			!decks[BOTTOM].orientation) ||
			(decks[BOTTOM].position[rails - 1] == 0 &&
			decks[BOTTOM].orientation)) ||
			(decks[TOP].number != 0 &&
			decks[TOP].piecePercent != 0 &&
			((decks[TOP].position[rails - 1] == 1 &&
			!decks[TOP].orientation) ||
			(decks[TOP].position[rails - 1] == 0 &&
			decks[TOP].orientation)))) &&
			o < numDigits - 1) ? 1 : 0;
		buffer.setLength(12 + numDigits + rom);
		if (s == 1)
			buffer.setCharAt(0, '-');
		for (i = 0; i < half; i++)
			buffer.setCharAt(s + i, digits.charAt(n + i));
		buffer.setCharAt(s + half, decimalChar());
		if (decks[BOTTOM].piece != 0) {
			StringBuffer stringBuf =
				new StringBuffer(numDigits + 2);
			StringBuffer midBuf =
				new StringBuffer(numDigits + 2);
			StringBuffer finalBuf =
				new StringBuffer(numDigits + 2);
			int pieces = decks[BOTTOM].piece;
			int precision;

			precision = ((decks[BOTTOM].piecePercent == 0) ?
				1 : 2) * shiftPercent + 2;
			if ((precision <= 2 * shiftPercent + 2) &&
				checkSubdeck(3)) {
				precision = 2 * shiftPercent + 2;
			}
			if (precision <= decimalPosition)
				precision = decimalPosition +
					((decks[BOTTOM].piecePercent == 0) ? 1 : 0);
			if (decks[TOP].piece != 0)
				pieces *= decks[TOP].piece;
			stringBuf.setLength(numDigits + 2);
			midBuf.setLength(numDigits + 2);
			finalBuf.setLength(numDigits + 2);
			abacusMath.dividePieces(midBuf, base, pieces,
				abacusMath.char2Int(digits.charAt(n + half)),
				precision, decimalChar());
			if (decks[BOTTOM].piecePercent != 0) {
				int piecePercents =
					decks[BOTTOM].piecePercent;

				if (decks[TOP].piecePercent != 0)
					piecePercents *=
						decks[TOP].piecePercent;
				/* get piecePercent part */
				abacusMath.dividePieces(stringBuf, base, piecePercents,
					abacusMath.char2Int(digits.charAt(n + half + shiftPercent + 1)),
					precision, decimalChar());
				/* shift to proper piece percent place */
				abacusMath.shiftDecimal(finalBuf, stringBuf.toString(),
					shiftPercent, 0, decimalChar());
				/* shift out piece part */
				/*shiftDecimal(midBuf, buffer, -1, 0, decimalChar()); */
				abacusMath.addStrings(stringBuf, finalBuf.toString(),
					midBuf.toString(), base, decimalChar());
				for (i = 0; i < decimalPosition - m; i++)
					buffer.setCharAt(s + half + 1 + i,
						digits.charAt(n + half + 1 + i));
				if (m == decimalPosition - 1)
					m--;
				buffer.setLength(s + half + decimalPosition - m);
				/* shift out piecePercent part */
				abacusMath.shiftDecimal(midBuf, buffer.toString(),
					-1, shiftPercent, decimalChar());
				/* get pieces + piece percent + normal */
				abacusMath.addStrings(finalBuf, stringBuf.toString(),
					midBuf.toString(), base, decimalChar());
			} else if (checkSubdeck(3)) {
				int pieceFractions = decks[BOTTOM].piece;

				if (decks[TOP].piece != 0)
					pieceFractions *= decks[TOP].piece;
				/* get pieceFraction part */
				abacusMath.dividePieces(finalBuf, base,
					pieceFractions * subbase,
					abacusMath.char2Int(digits.charAt(n + half + 1)),
					precision, decimalChar());
				/* get piece + pieceFraction */
				abacusMath.addStrings(stringBuf, finalBuf.toString(),
					midBuf.toString(), base, decimalChar());
				/* No fixing because no digits after Roman fraction */
				/* get pieces + piece percent + normal */
				abacusMath.addStrings(finalBuf, stringBuf.toString(),
					buffer.toString(), base, decimalChar());

			} else {
				for (i = 0; i < decimalPosition - m; i++)
					buffer.setCharAt(s + half + 1 + i,
						digits.charAt(n + half + 1 + i));
				if (m == decimalPosition - 1)
					m--;
				buffer.setLength(s + half + decimalPosition - m);
				abacusMath.addStrings(finalBuf, buffer.toString(),
					midBuf.toString(), base, decimalChar());
			}
			buffer = new StringBuffer(finalBuf.toString().trim());
		}
		if (decks[BOTTOM].piece == 0 &&
				decks[BOTTOM].piecePercent == 0) {
			int dOffset = decimalPosition - m + 1;

			if (dOffset < 0)
				dOffset = 0;
			for (i = 0; i < dOffset; i++)
				buffer.setCharAt(s + half + 1 + i,
					digits.charAt(n + half + i));
			buffer = new StringBuffer(buffer.toString().trim());
		}
		while (buffer.length() > 3 &&
				buffer.charAt(buffer.length() - 1) == '0' &&
				buffer.charAt(buffer.length() - 2) != decimalChar()) {
			buffer.setLength(buffer.length() - 1);
		}
		if (((AbacusApplet)applet).getScript()) {
			((AbacusApplet)applet).callbackAbacus(
				/*AbacusInterface.ACTION_SCRIPT*/
				AbacusInterface.PRIMARY, deck,
				rail - decimalPosition, number);
		}
		anomalyActive = checkAnomaly();
		if (base != displayBase || anomalyActive) {
			StringBuffer buff = new StringBuffer(1024);

			buff.setLength(1024);
			if (anomalyActive) {
				abacusMath.convertString(buff, buffer.toString(), base,
					displayBase, decimalPosition,
					anomaly, shiftAnomaly, carryAnomaly,
					anomalySq, shiftAnomalySq,
					carryAnomalySq, decimalChar());
			} else {
				abacusMath.convertString(buff, buffer.toString(), base,
					displayBase, decimalPosition,
					0, shiftAnomaly, false,
					0, shiftAnomalySq, false,
					decimalChar());
			}
			buffer = new StringBuffer(buff.toString().trim());
		}
		if (romanNumerals) {
			int pieces = decks[BOTTOM].piece;

			if (decks[TOP].piece != 0)
				pieces *= decks[TOP].piece;
			StringBuffer romanString = new StringBuffer(rom);

			abacusMath.string2Roman(romanString, buffer.toString(),
				displayBase, pieces,
				valuePiece(), valueSubdeck(), subbase,
				decimalChar(), ancientRoman, latin);
			buffer.append("          ");
			buffer.append(romanString);
		}
		if (group) {
			st = abacusMath.string2Group(buffer.toString(),
				groupSize, decimalChar(), groupChar());
		} else {
			st = new String(buffer);
		}
		if (st.indexOf('.') >= 0 && st.substring(st.indexOf('.') +
				1).trim().length() == 0)
			//trim last "." if nothing behind it
			st = st.substring(0, st.indexOf('.'));
		//System.out.println(displayBase + "#displayBase#");
		//System.out.println(st + "##st");
		((AbacusApplet)applet).callbackAbacus(this, st);
	}

	void fillPolygon(Graphics g, Color fillColor, Color lineColor,
			Point[] list, int n) {
		int[] tempListX = new int[n];
		int[] tempListY = new int[n];

		for (int i = 0; i < n; i++) {
			tempListX[i] = list[i].x;
			tempListY[i] = list[i].y;
		}
		g.setColor(fillColor);
		g.fillPolygon(tempListX, tempListY, n);
		g.setColor(lineColor);
		g.drawPolygon(tempListX, tempListY, n);
	}

	void vDrawLine(Graphics g, int x1, int y1, int x2, int y2) {
		if (vertical) {
			g.drawLine(y1, x1, y2, x2);
		} else {
			g.drawLine(x1, y1, x2, y2);
		}
	}

	void vDrawRect(Graphics g, int i, int j, int l, int h) {
		if (vertical) {
			g.drawRect(j, i, h, l);
		} else {
			g.drawRect(i, j, l, h);
		}
	}

	void vfillRect(Graphics g, int i, int j, int l, int h) {
		if (vertical) {
			g.fillRect(j, i, h, l);
		} else {
			g.fillRect(i, j, l, h);
		}
	}

	void vDrawCircle(Graphics g, int d, int i, int j) {
		if (vertical) {
			drawCircle(g, d, j, i);
		} else {
			drawCircle(g, d, i, j);
		}
	}

	void vFillCircle(Graphics g, int d, int i, int j) {
		if (vertical) {
			fillCircle(g, d, j, i);
		} else {
			fillCircle(g, d, i, j);
		}
	}

	void vFillPolygon(Graphics g, Color fillColor, Color lineColor,
			Point[] list, int n) {
		if (vertical) {
			int i, t;

			for (i = 0; i < n; i++) {
				t = list[i].x;
				list[i].x = list[i].y;
				list[i].y = t;
			}
			fillPolygon(g, fillColor, lineColor, list, n);
		} else {
			fillPolygon(g, fillColor, lineColor, list, n);
		}
	}

/* Note On Draws:
 * There is some overlap i.e. it draws some extra pixels.
 * This is to synchronize Java, X and Windows.
 */

	void drawDecimalSeparator(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 1 - railWidth / 2 + offset.x +
			rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x, y, railWidth + 4, midHeight);
	}

	void drawGroupSeparator(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 1 - railWidth / 2 + offset.x +
			rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x + 1, y + 1,
			railWidth + 2, midHeight - 2);
		vfillRect(g, x, y + (midHeight - 1) / 2,
			railWidth + 4, (midHeight - 1) % 2 + 1);
		vfillRect(g, x + (railWidth + 1) / 2 + 1, y,
			(railWidth + 1) % 2 + 1, midHeight);
	}

	void drawAllGroupSeparators(Graphics g, boolean show) {
		int separator;

		for (separator = 1; separator <= ((rails - ((sign) ? 1 : 0) -
				decimalPosition - 1) / groupSize);
				separator++)
			drawGroupSeparator(g, rails -
				decimalPosition -
				groupSize * separator, show);
	}

	void drawRomanI(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x, y, 2, midHeight);
	}

	void drawRomanX(Graphics g, int rail, boolean show) {
		int x, y, ht;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		ht = midHeight - 1;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vDrawLine(g, x - 3, y, x + 3, y + ht);
		vDrawLine(g, x + 3, y, x - 3, y + ht);
		vDrawLine(g, x + 2, y + ht, x - 2, y);
		vDrawLine(g, x - 2, y + ht, x + 2, y);
		vDrawLine(g, x + 3, y + ht, x - 3, y);
		vDrawLine(g, x - 3, y + ht, x + 3, y);
		vDrawLine(g, x - 2, y, x + 2, y + ht);
		vDrawLine(g, x + 2, y, x - 2, y + ht);
	}

	void drawRomanC(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x - 2, y, 5, 1);
		vfillRect(g, x - 2, y + midHeight - 1, 5, 1);
		vfillRect(g, x - 3, y + 1, 2, midHeight - 2);
		vDrawLine(g, x + 3, y + 1, x + 2, y);
		vDrawLine(g, x + 3, y + midHeight - 2,
			x + 2, y + midHeight - 1);
	}

	void drawRomanM(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (modernRoman) {
			vfillRect(g, x - 3, y, 2, midHeight);
			vfillRect(g, x + 3, y, 2, midHeight);
			vDrawLine(g, x - 2, y, x, y + 2);
			vDrawLine(g, x + 3, y, x + 1, y + 2);
			vDrawLine(g, x - 3, y, x - 1, y + 1);
			vDrawLine(g, x + 4, y, x + 2, y + 1);
		} else {
			vfillRect(g, x, y, 2, midHeight);
			vfillRect(g, x - 5, y + 3, 2, midHeight - 4);
			vfillRect(g, x - 4, y + 2, 3, 1);
			vfillRect(g, x - 4, y + midHeight - 1, 3, 1);
			vfillRect(g, x + 5, y + 3, 2, midHeight - 4);
			vfillRect(g, x + 3, y + 2, 3, 1);
			vfillRect(g, x + 3, y + midHeight - 1, 3, 1);
		}
	}

	void drawRomanx(Graphics g, int rail, boolean show) {
		int x, y, ht;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		ht = midHeight - 1;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (modernRoman) {
			vDrawLine(g, x - 1, y + 2, x + 2, y + ht);
			vDrawLine(g, x + 2, y + 2, x - 1, y + ht);
			vDrawLine(g, x - 2, y + 2, x + 3, y + ht);
			vDrawLine(g, x + 3, y + 2, x - 2, y + ht);
			vfillRect(g, x - 3, y, 8, 1);
		} else {
			vfillRect(g, x, y, 2, midHeight);
			vfillRect(g, x - 3, y + 3, 1, midHeight - 4);
			vDrawLine(g, x - 2, y + 2, x - 3, y + 3);
			vDrawLine(g, x - 2, y + midHeight - 1,
				x - 3, y + midHeight - 2);
			vfillRect(g, x + 4, y + 3, 1, midHeight - 4);
			vDrawLine(g, x + 3, y + 2, x + 4, y + 3);
			vDrawLine(g, x + 3, y + midHeight - 1,
				x + 4, y + midHeight - 2);
			vfillRect(g, x - 5, y + 2, 1, midHeight - 3);
			vDrawLine(g, x - 3, y, x - 5, y + 2);
			vDrawLine(g, x - 4, y + midHeight - 1,
				x - 5, y + midHeight - 2);
			vfillRect(g, x + 6, y + 2, 1, midHeight - 3);
			vDrawLine(g, x + 4, y, x + 6, y + 2);
			vDrawLine(g, x + 5, y + midHeight - 1,
				x + 6, y + midHeight - 2);
		}
	}

	void drawRomanc(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (modernRoman) {
			vfillRect(g, x - 2, y + 2, 6, 1);
			vfillRect(g, x - 2, y + midHeight - 1, 6, 1);
			vfillRect(g, x - 3, y + 3, 2, 2);
			vfillRect(g, x - 3, y, 8, 1);
		} else {
			vfillRect(g, x, y, 2, midHeight);
			vDrawLine(g, x - 2, y + 3, x - 3, y + 4);
			vDrawLine(g, x - 3, y + 4, x - 2, y + 3);
			vDrawLine(g, x - 2, y + midHeight - 1,
				x - 3, y + midHeight - 2);
			vDrawLine(g, x + 3, y + 3, x + 4, y + 4);
			vDrawLine(g, x + 4, y + 4, x + 3, y + 3);
			vDrawLine(g, x + 3, y + midHeight - 1,
				x + 4, y + midHeight - 2);
			vfillRect(g, x - 3, y + 1, 2, 1);
			vDrawLine(g, x - 4, y + 2,
				x - 5, y + 3);
			vfillRect(g, x - 5, y + 3, 1, 2);
			vDrawLine(g, x - 4, y + midHeight - 1,
				x - 5, y + midHeight - 2);
			vfillRect(g, x + 3, y + 1, 2, 1);
			vDrawLine(g, x + 5, y + 2,
				x + 6, y + 3);
			vfillRect(g, x + 6, y + 3, 1, 2);
			vDrawLine(g, x + 5, y + midHeight - 1,
				x + 6, y + midHeight - 2);
			vfillRect(g, x - 7, y + 2, 1, midHeight - 3);
			vDrawLine(g, x - 5, y, x - 7, y + 2);
			vDrawLine(g, x - 6, y + midHeight - 1,
				x - 7, y + midHeight - 2);
			vfillRect(g, x + 8, y + 2, 1, midHeight - 3);
			vDrawLine(g, x + 6, y, x + 8, y + 2);
			vDrawLine(g, x + 7, y + midHeight - 1,
				x + 8, y + midHeight - 2);
		}
	}

	void drawRomanm(Graphics g, int rail, boolean show) {
		int x, y, ht;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		ht = midHeight - 1;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (modernRoman) {
			vfillRect(g, x - 3, y + 2, 2, midHeight - 2);
			vfillRect(g, x + 3, y + 2, 2, midHeight - 2);
			vDrawLine(g, x - 2, y + 2, x, y + 4);
			vDrawLine(g, x + 3, y + 2, x + 1, y + 4);
			vDrawLine(g, x - 3, y + 2, x - 1, y + 3);
			vDrawLine(g, x + 4, y + 2, x + 2, y + 3);
			vfillRect(g, x - 3, y, 8, 1);
		} else {
			vDrawLine(g, x - 1, y + 2, x + 2, y + ht);
			vDrawLine(g, x + 2, y + 2, x - 1, y + ht);
			vDrawLine(g, x - 2, y + 2, x + 3, y + ht);
			vDrawLine(g, x + 3, y + 2, x - 2, y + ht);
			vDrawLine(g, x - 1, y + ht, x + 2, y + 2);
			vDrawLine(g, x + 2, y + ht, x - 1, y + 2);
			vDrawLine(g, x - 2, y + ht, x + 3, y + 2);
			vDrawLine(g, x + 3, y + ht, x - 2, y + 2);
			vfillRect(g, x - 4, y, 1, midHeight);
			vfillRect(g, x + 5, y, 1, midHeight);
			vfillRect(g, x - 3, y, 8, 1);
		}
	}

	void drawRomanHalf(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 4 + offset.x + rail * pos.x - (pos.x - 1) / 2 -
			beadSize.x / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y +
			numberSubbeadsOffset(subdeck - 1) * pos.y + pos.y / 2;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (submode == AbacusInterface.IT) {
			vfillRect(g, x - 1, y, 4, 1);
			vfillRect(g, x - 1, y + midHeight - 1, 4, 1);
			vDrawLine(g, x - 2, y + 1, x - 1, y);
			vDrawLine(g, x - 2, y + midHeight - 2,
				x - 1, y + midHeight - 1);
			vDrawLine(g, x + 3, y + 1, x + 2, y);
			vDrawLine(g, x + 3, y + midHeight - 2,
				x + 2, y + midHeight - 1);
			vDrawLine(g, x - 2, y + 1,
				x + 3, y + midHeight - 2);
		} else if (submode == AbacusInterface.UK) {
			vfillRect(g, x, y, 2, 1);
			vfillRect(g, x - 2, y + midHeight - 1, 6, 1);
			vDrawLine(g, x - 1, y + 1, x, y);
			vDrawLine(g, x + 2, y + 1, x + 1, y);
			vDrawLine(g, x - 1, y + 1, x + 1, y + 3);
			vDrawLine(g, x + 1, y + midHeight - 3,
				x - 1, y + midHeight - 1);
		} else if (submode == AbacusInterface.FR) {
			vfillRect(g, x - 1, y + 2, 3, 1);
			vfillRect(g, x - 1, y + midHeight - 1, 5, 1);
			vDrawLine(g, x + 1, y, x - 1, y + 2);
			vDrawLine(g, x + 1, y + midHeight - 3,
				x - 1, y + midHeight - 1);
		}
	}

	void drawRomanQuarter(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 4 + offset.x + rail * pos.x - (pos.x - 1) / 2 -
			beadSize.x / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y +
			numberSubbeadsOffset(subdeck - 2) * pos.y + pos.y / 2;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (submode == AbacusInterface.IT) {
			vfillRect(g, x - 1, y, 5, 1);
			vfillRect(g, x - 1, y + midHeight - 1, 5, 1);
			vfillRect(g, x + 3, y + 1, 2, midHeight - 2);
			vDrawLine(g, x - 1, y, x - 2, y + 1);
			vDrawLine(g, x - 1, y + midHeight - 1,
				x - 2, y + midHeight - 2);
		} else if (submode == AbacusInterface.UK) {
			vfillRect(g, x + 1, y + 2, 1, 2);
			vDrawLine(g, x - 1, y, x + 1, y + 2);
			vDrawLine(g, x - 1, y + midHeight - 1,
				x + 1, y + midHeight - 3);
		} else if (submode == AbacusInterface.FR) {
			vfillRect(g, x, y, 2, 1);
			vfillRect(g, x + 1, y + 1, 1, 3);
			vDrawLine(g, x - 1, y + 1, x + 1, y);
			vDrawLine(g, x - 1, y + midHeight - 1,
				x + 1, y + midHeight - 3);
		}
	}

	void drawRomanTwelfth(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 4 + offset.x + rail * pos.x - (pos.x - 1) / 2 -
			beadSize.x / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y +
			numberSubbeadsOffset(subdeck - 3) * pos.y + pos.y / 2;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (submode == AbacusInterface.IT) {
			vfillRect(g, x - 3, y, 6, 1);
			vfillRect(g, x - 2, y + midHeight - 1, 6, 1);
			vDrawLine(g, x + 2, y, x - 2, y + midHeight - 1);
			vDrawLine(g, x - 3, y + midHeight - 1, x + 3, y);
			vDrawLine(g, x + 3, y, x - 3, y + midHeight - 1);
		} else if (submode == AbacusInterface.UK) {
			vfillRect(g, x - 2, y + midHeight - 1, 5, 1);
			vfillRect(g, x - 1, y, 3, 1);
			vfillRect(g, x + 2, y + 1, 1, 2);
			vDrawLine(g, x - 2, y + 1, x - 1, y);
			vDrawLine(g, x + 2, y + 1, x + 1, y);
			vDrawLine(g, x + 2, y + 2, x - 2, y + midHeight - 1);
		} else if (submode == AbacusInterface.FR) {
			vfillRect(g, x - 1, y + midHeight - 1, 6, 1);
			vfillRect(g, x, y, 3, 1);
			vDrawLine(g, x - 1, y + 1, x, y);
			vDrawLine(g, x + 3, y + 1, x + 2, y);
			vDrawLine(g, x + 3, y + 1, x - 1, y + midHeight - 1);
		}
	}

	void drawNegative(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 1 - railWidth / 2 + offset.x +
			rail * pos.x - (pos.x - 1) / 2;

		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x, y + 2, railWidth + 4, 2);
	}

	void drawPiece(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 1 - railWidth / 2 + offset.x +
			rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		vfillRect(g, x, y + 2, railWidth + 4, 2);
		vfillRect(g, x + 2, y, railWidth, midHeight);
	}

	void drawRomanPiece(Graphics g, int rail, boolean show) {
		int x, y, woffset = 0, hoffset = 1;

		g.setColor((show) ? symbolColor : background);
		x = 2 + offset.x + rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (!slot) {
			woffset = ((railWidth & 1) == 1) ? 1 : 0;
			hoffset = ((railWidth & 1) == 1) ? 0 : 1;
		}
		vfillRect(g, x - 1 + woffset , y, 3 + hoffset, 1);
		vfillRect(g,
			x - 1 + woffset, y + midHeight - 1, 3 + hoffset, 1);
		vfillRect(g, x - 2 + woffset, y + 1, 2, midHeight - 2);
		vfillRect(g,
			x + 1 + woffset + hoffset, y + 1, 2, midHeight - 2);
	}

	void drawAnomaly(Graphics g, int rail, boolean show) {
		int x, y;

		g.setColor((show) ? symbolColor : background);
		x = 1 - railWidth / 2 + offset.x +
			rail * pos.x - (pos.x - 1) / 2;
		y = decks[TOP].room * pos.y + delta.y - 1 + offset.y;
		if (vertical) {
			y = frameSize.y - midHeight - y;
		}
		if (railWidth - 4 > 0) {
			vDrawLine(g, x, y, x + 1, y + 1);
			vDrawLine(g, x + railWidth + 3, y + midHeight - 1,
				x + railWidth + 2, y + midHeight - 2);
			vDrawLine(g, x, y + midHeight - 1,
				x + 1, y + midHeight - 2);
			vDrawLine(g, x + railWidth + 3, y,
				x + railWidth + 2, y + 1);
			vfillRect(g, x + 1, y + 1, 2, 1);
			vfillRect(g, x + railWidth + 1, y + midHeight - 2,
				2, 1);
			vfillRect(g, x + 1, y + midHeight - 2, 2, 1);
			vfillRect(g, x + railWidth + 1, y + 1, 2, 1);
			vfillRect(g, x + 3, y + 2, railWidth - 2, 2);
		} else {
			vDrawLine(g, x, y, x + 2, y + 2);
			vDrawLine(g, x + railWidth + 3, y + midHeight - 1,
				x + railWidth + 1, y + midHeight - 3);
			vDrawLine(g, x, y + midHeight - 1,
				x + 2, y + midHeight - 3);
			vDrawLine(g, x + railWidth + 3, y,
				x + railWidth + 1, y + 2);
			vfillRect(g, x + 2, y + 2, railWidth, 2);
		}
	}

	public void drawFrame(boolean show) {
		Graphics g = getGraphics();

		if (g == null)
			return;
		try {
			int deck, x, y, yOffset, yPrime;

			if (show) {
				x = rails * pos.x + delta.x - 1;
				g.setColor(currentForeground);
				/* Top/Left */
				vfillRect(g, 0, 0,
					offset.x + 1, frameSize.y);
				/* Bottom/Right */
				vfillRect(g, x + offset.x, 0,
					frameSize.x - (x + offset.x),
					frameSize.y);
				for (deck = UP; deck >= DOWN; deck--) {
				  g.setColor(currentForeground);
				  yOffset = (deck == UP) ?
					0 : decks[TOP].height;
				  y = decks[deck].room * pos.y + delta.y - 1;
				  yPrime = y + yOffset + offset.y;
				  if (vertical) {
					yPrime = frameSize.y - midHeight -
						yPrime;
					yOffset = frameSize.y - offset.y -
						yOffset;
				  }
				  if (deck == UP) {
				      boolean anomalyActive = checkAnomaly();

				      /* Right/Top */
				      vfillRect(g, offset.x + 1, yOffset - 1,
					x - 1, offset.y + 1);
				      if (!slot) {
					/* Middle */
					vfillRect(g, offset.x + 1, yPrime,
						x - 1, midHeight);
				      }
				      if (slot && !anomalyActive) {
					int pRails = rails - decimalPosition -
					  ((sign) ? 1 : 0);

					drawRomanI(g, rails -
						decimalPosition, show);
					if (pRails - 1 > 0)
					drawRomanX(g, rails -
						decimalPosition - 1, show);
					if (pRails - 2 > 0)
					drawRomanC(g, rails -
						decimalPosition - 2, show);
					if (pRails - 3 > 0)
					drawRomanM(g, rails -
						decimalPosition - 3, show);
					if (pRails - 4 > 0)
					drawRomanx(g, rails -
						decimalPosition - 4, show);
					if (pRails - 5 > 0)
					drawRomanc(g, rails -
						decimalPosition - 5, show);
					if (pRails - 6 > 0)
					drawRomanm(g, rails -
						decimalPosition - 6, show);
					if (checkSubdeck(3)) {
						drawRomanHalf(g, rails -
						  decimalPosition + 3,
						  show);
						if (subdeck > 1)
						  drawRomanQuarter(g, rails -
						    decimalPosition + 3,
						    show);
						if (subdeck > 2)
						  drawRomanTwelfth(g, rails -
						    decimalPosition + 3,
						    show);
					}
				      } else {
					 drawDecimalSeparator(g,
					   rails - decimalPosition, show);
					if (!anomalyActive)
					   drawAllGroupSeparators(g, show);
				      }
				      if (sign)
					drawNegative(g, 1, show);
				      if (slot) {
					if (decks[BOTTOM].piece != 0)
					    drawRomanPiece(g, rails -
						decimalPosition + 1, show);
					if (decks[BOTTOM].piecePercent != 0)
					    drawRomanPiece(g, rails -
						decimalPosition + 1 +
						((decks[BOTTOM].piece == 0) ? 0 : 1) +
						shiftPercent, show);
				      } else {
					if (decks[BOTTOM].piece != 0)
					    drawPiece(g, rails -
						decimalPosition + 1, show);
					if (decks[BOTTOM].piecePercent != 0)
					    drawPiece(g, rails -
						decimalPosition + 1 +
						((decks[BOTTOM].piece == 0) ? 0 : 1) +
						shiftPercent, show);
				      }
				      if (anomalyActive)
					drawAnomaly(g, rails -
						decimalPosition -
						shiftAnomaly, show);
				      if (anomalyActive && anomalySq != 0)
					drawAnomaly(g, rails -
						decimalPosition -
						shiftAnomaly -
						shiftAnomalySq, show);
				} else {
				    if (vertical) {
					/* Bottom */
					g.fillRect(0, offset.x + 1,
						yPrime + 3, x - 1);
				    } else {
					/* Bottom */
					g.fillRect(offset.x + 1,
						yPrime + midHeight - 3,
						x - 1,
						frameSize.y -
						(yPrime + midHeight - 3));
				    }
				  }
				}
			} else {
				g.setColor(background);
				g.fillRect(0, 0,
					coreSize.x - 1, coreSize.y - 1);
			}
		} finally {
			g.dispose();
		}
	}

	void fillRectClipX(Graphics g, int dx, int dy, int sx, int sy,
			int ox, int wox, int wsx) {
		int nox = ox, nsx = sx;

		if (ox + sx < wox || ox > wox + wsx || wsx <= 0)
			return;
		if (nox < wox) {
			nox = wox;
			nsx = sx - wox + ox;
		}
		if (nox + nsx > wox + wsx) {
			nsx = wsx + wox - nox;
		}
		g.fillRect(dx + nox, dy, nsx, sy);
	}

	void fillRectClipY(Graphics g, int dx, int dy, int sx, int sy,
			int oy, int woy, int wsy) {
		int noy = oy, nsy = sy;

		if (oy + sy < woy || oy > woy + wsy || wsy <= 0)
			return;
		if (noy < woy) {
			noy = woy;
			nsy = sy - woy + oy;
		}
		if (noy + nsy > woy + wsy) {
			nsy = wsy + woy - noy;
		}
		g.fillRect(dx, dy + noy, sx, nsy);
	}

	void vfillRectClip(Graphics g, int dx, int dy, int sx, int sy,
			int o, int wo, int ws) {
		if (vertical) {
			fillRectClipX(g, dy, dx, sy, sx, o, wo, ws);
		} else {
			fillRectClipY(g, dx, dy, sx, sy, o, wo, ws);
		}
	}

	void copyImage(Graphics g, Image i, int dX, int dY, int sX, int sY,
			int width, int height) {
		g.drawImage(i, dX, dY, dX + width, dY + height,
			sX, sY, sX + width, sY + height, this);
	}

	void drawRail(Graphics g, int deck, int rail, int j,
			boolean highlight, int offsetX, int size) {
		int dx, dy, yOffset, subj = 0, d = -1, room;

		yOffset = (deck == UP) ? 0 :
			decks[TOP].height + midHeight - 3;
		dx = (rails - rail - 1) * pos.x + delta.x + offset.x;
		dy = (j - 1) * pos.y + delta.y +
			yOffset + offset.y - 1;
		if (vertical)
			dy = frameSize.y - beadSize.y - dy - 1;
		dy -= pressOffsetY;
		room = decks[deck].room;
		if (checkSubdeck(3) && rail == decimalPosition - 2) {
			if (submodeSlotsSeparate(submode)) {
				d = subdeckPosition(j);
				subj = positionSubdeck(j);
			} else {
				room = 0;
				for (d = 0; d < subdeck; d++)
					room += subdecks[d].room;
				d = -1;
			}
		}
		g.setColor(background);
		vfillRectClip(g, dx, dy,
			beadSize.x + 2, beadSize.y + 1 + 2 * pressOffsetY,
			0, offsetX, size);
		if (slot && (j == 1 || (d != -1 && subj == 1))) {
			g.setColor((highlight) ? railColor[2] : borderColor);
			vfillRectClip(g,
				dx + beadSize.x / 2 - railWidth / 2,
				dy,
				railWidth,
				5 * beadSize.y / 8 + 4,
				3 * beadSize.y / 8, offsetX, size);
			g.setColor(background);
			/* round off the top of rail */
			vfillRectClip(g,
				dx + beadSize.x / 2 - railWidth / 2 - 1,
				dy,
				2, 1, 3 * beadSize.y / 8, offsetX, size);
			vfillRectClip(g,
				dx + beadSize.x / 2 + (railWidth - 1) / 2,
				dy,
				2, 1, 3 * beadSize.y / 8, offsetX, size);
		} else if (slot && (j == room ||
				(d != -1 && subj == subdecks[d].room))) {
			g.setColor((highlight) ? railColor[2] : borderColor);
			vfillRectClip(g,
				dx + beadSize.x / 2 - railWidth / 2,
				dy,
				railWidth,
				5 * beadSize.y / 8 + 3,
				0, offsetX, size);
			g.setColor(background);
			/* round off the bottom of rail */
			vfillRectClip(g,
				dx + beadSize.x / 2 - railWidth / 2 - 1,
				dy,
				2, 1, 2 + 5 * beadSize.y / 8, offsetX, size);
			vfillRectClip(g,
				dx + beadSize.x / 2 + (railWidth - 1) / 2,
				dy,
				2, 1, 2 + 5 * beadSize.y / 8, offsetX, size);
		} else {
			g.setColor((highlight) ? railColor[2] :
				((slot) ? borderColor : railColor[railIndex]));
			vfillRectClip(g,
				dx + beadSize.x / 2 - railWidth / 2,
				dy,
				railWidth, beadSize.y + 1 + 2 * pressOffsetY,
				0, offsetX, size);
		}
	}

	void drawRail(int deck, int rail, int j,
			boolean highlight, int offsetX, int size) {
		Graphics g = getGraphics();

		if (g == null)
			return;
		try {
			drawRail(g, deck, rail, j, highlight, offsetX, size);
		} finally {
			g.dispose();
		}
	}

	void drawBead(Graphics g, int deck, int rail, int bead,
			int j, boolean show, boolean moving, boolean highlight,
			int pressed, int offsetX, int offsetY) {
		int dx, dy, yOffset, special = 0, pieces, piecePercents;
		int color = 0;
		int d = -1, subj = 0, room, sx, sy;

		yOffset = (deck == UP) ? 0 :
			decks[TOP].height + midHeight - 3;
		dx = (rails - rail - 1) * pos.x +
			delta.x + offset.x + offsetX;
		dy = (j - 1) * pos.y + delta.y +
			yOffset + offset.y - 2 + pressOffsetY +
			offsetY;
		room = decks[deck].room;
		if (checkSubdeck(3) && rail == decimalPosition - 2) {
			if (submodeSlotsSeparate(submode)) {
				d = subdeckPosition(j);
				subj = positionSubdeck(j);
			} else {
				room = 0;
				for (d = 0; d < subdeck; d++)
					room += subdecks[d].room;
				d = -1;
			}
		}
		if (show) {
			int railI = railIndex;

			if (highlight)
				railI = 2;
			if ((rail == rails - 1) && sign) {
				special++;
			} else if (decks[BOTTOM].piece != 0 &&
				(rail == decimalPosition - 1)) {
			    pieces = numberPieces(BOTTOM);
			    if ((colorScheme & AbacusInterface.COLOR_MIDDLE) != 0) {
				if ((((bead == pieces / 2) && ((pieces & 1) == 0)) ||
					bead == pieces / 2 + 1) && pieces > 2)
				  special++;
			    } else if ((colorScheme & AbacusInterface.COLOR_HALF) != 0) {
				if ((pieces & 1) != 0) {
				    if (bead == pieces / 2 + 1)
					color++;
				} else if (bead > pieces / 2) {
				    if (decks[deck].orientation)
					color++;
				} else {
				    if (!decks[deck].orientation)
					color++;
				}
			    } else {
				special++;
			    }
			} else if (decks[BOTTOM].piecePercent != 0 &&
				(rail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece != 0) ? 1 : 0))) {
			    piecePercents = numberPiecePercents(BOTTOM);
			    if ((colorScheme & AbacusInterface.COLOR_MIDDLE) != 0) {
				if ((((bead == decks[BOTTOM].piecePercent / 2) &&
					((decks[BOTTOM].piecePercent & 1) == 0)) ||
					bead == decks[BOTTOM].piecePercent / 2 + 1) &&
					decks[BOTTOM].piecePercent > 2)
				    special++;
			    } else if ((colorScheme & AbacusInterface.COLOR_HALF) != 0) {
				if ((piecePercents & 1) != 0) {
				    if (bead == piecePercents / 2 + 1)
					color++;
				} else if (bead > piecePercents / 2) {
				    if (decks[deck].orientation)
					color++;
				} else {
				    if (!decks[deck].orientation)
					color++;
				}
			    } else {
				special++;
			    }
			} else if (checkSubdeck(3) &&
				(rail == decimalPosition - 2)) {
			    if (((subdeck - subdeckPosition(j)) % 2) == 0)
				special++;
			} else if (!((rail == rails - 1) && sign) &&
					(colorScheme & AbacusInterface.COLOR_MIDDLE) != 0) {
			  if ((((bead == decks[deck].number / 2) &&
					((decks[deck].number & 1) == 0)) ||
					bead == decks[deck].number / 2 + 1) &&
					decks[deck].number > 2) {
				special++;
			  }
			  if ((colorScheme & AbacusInterface.COLOR_FIRST) != 0 &&
					deck == BOTTOM &&
					rail - decimalPosition > 0 &&
					(rail - decimalPosition) %
					groupSize == 0) {
				if (bead == decks[deck].number &&
				    decks[deck].orientation)
					special++;
				else if (bead == 1 &&
				    !decks[deck].orientation)
					special++;
			  }
			} else if (!((rail == rails - 1) && sign) &&
					(colorScheme &AbacusInterface.COLOR_HALF) != 0) {
				if ((decks[deck].number & 1) != 0) {
					if (bead == decks[deck].number / 2 + 1)
						color++;
				} else if (bead > decks[deck].number / 2) {
					if (decks[deck].orientation)
						color++;
				} else {
					if (!decks[deck].orientation)
						color++;
				}
			}
			if (vertical)
				dy = frameSize.y - beadSize.y -
					dy - 3 + 2 * pressOffsetY;
			dx += pressed * pressOffsetX;
			dy += pressed * pressOffsetY;
			if (!moving && pressed == 0) {
			    /* Draw the rail around bead */
			    g.setColor(background);
			    vfillRect(g, dx, dy + beadSize.y + pressOffsetY,
				beadSize.x, 1);
			    if (pressOffsetY != 0) {
			      if (!slot || (j != 1 && (d == -1 || subj != 1))) {
				g.setColor((slot) ? borderColor : railColor[railI]);
				vfillRect(g,
					dx + beadSize.x / 2 - railWidth / 2,
					dy - 1 + ((vertical) ? beadSize.y : 0),
					railWidth, 3);
			      }
			      if (!slot || (j != room && (d == -1 ||
					subj != subdecks[d].room))) {
				g.setColor((slot) ? borderColor : railColor[railI]);
				vfillRect(g,
					dx + beadSize.x / 2 - railWidth / 2,
					dy - 1 +
					((vertical) ? 0 : beadSize.y),
					railWidth, 3);
			      }
			    }
			} else {
			    /* Tweak */
			    g.setColor(background);
			    vfillRect(g, dx - pressOffsetX * pressed,
				dy - 2 * pressOffsetY * pressed + 1,
				1, beadSize.y + 1);
			    if (pressOffsetY != 0) {
			      vfillRect(g, dx - pressOffsetX * pressed,
				dy - pressOffsetY * pressed,
				beadSize.x + 1, 1);
			      if (!slot || (j != 1 && (d == -1 || subj != 1))) {
				g.setColor((slot) ? borderColor : railColor[railI]);
				vfillRect(g,
					dx + beadSize.x / 2 -
					railWidth / 2 - pressOffsetX * pressed,
					dy - pressOffsetY * pressed,
					railWidth, 1);
			      }
			    }
			}
			if (vertical) {
				sx = pos.y - pressOffsetY;
				sy = beadSize.x + 1;
				yOffset = dx;
				dx = dy + 1 - pressOffsetY;
				dy = yOffset;
			} else {
				sx = beadSize.x + 1;
				sy = pos.y - pressOffsetY;
				dy = dy + 1 - pressOffsetY;
			}
			if (highlight)
				color = 2;
			copyImage(g, bufferedBeadImage[color][pressed][special],
				dx, dy,
				0, 0,
				sx, sy);
		} else {
			drawRail(g, deck, rail, j, highlight,
				((pressOffsetY == 0) ?  0 : pressed),
				beadSize.y + 1 + 2 * pressOffsetY);
		}

	}

	void drawBead(int deck,  int rail, int bead,
			int j, boolean show, boolean moving, boolean highlight,
			int pressed, int offsetX, int offsetY) {
		Graphics g = getGraphics();

		if (g == null)
			return;
		try {
			drawBead(g, deck, rail, bead, j,
				show, moving, highlight,
				pressed, offsetX, offsetY);
		} finally {
			g.dispose();
		}
	}

	void drawBufferedBead(Graphics g, int color, int pressed,
			int special) {
		int shadeFill, shadeLine, shadeDot;
		int railWid = Math.min(beadSize.x - 5, railWidth);

		if (pressed == 1) {
			/*if (diamond) {
				shadeDot = 2;
				shadeLine = shadeFill = 1;
			} else {*/
			shadeFill = 2;
			shadeLine = 1;
			shadeDot = 1;
			/*}*/
		} else {
			shadeFill = 1;
			shadeLine = 2;
			shadeDot = 0;
		}
		if (special == 1) {
			shadeFill++;
			shadeLine++;
			shadeDot++;
		}
		shadeFill += 4 * color;
		shadeLine += 4 * color;
		shadeDot += 4 * color;
		g.setColor(background);
		vfillRect(g, 0, 0, beadSize.x + 1, pos.y);
		if (diamond) {
			Point[] tempList = new Point[5];

			tempList[0] = new Point(
				beadSize.x / 2 + (railWid - 1) / 2 + 2,
				beadSize.y);
			tempList[1] = new Point(
				beadSize.x / 2 - railWid / 2 - 2,
				beadSize.y);
			tempList[2] = new Point(0, beadSize.y / 2);
			tempList[3] = new Point(beadSize.x - 1,
				beadSize.y / 2);
			tempList[4] = new Point(
				beadSize.x / 2 + (railWid - 1) / 2 + 2,
				beadSize.y);
			vFillPolygon(g, beadShade[shadeFill],
				beadShade[shadeFill],
				tempList, 4);
			tempList[0] = new Point(
				beadSize.x / 2 - railWid / 2 - 2,
				0);
			tempList[1] = new Point(
				beadSize.x / 2 + (railWid - 1) / 2 + 2,
				0);
			tempList[2] = new Point(beadSize.x - 1,
				beadSize.y / 2);
			tempList[3] = new Point(0, beadSize.y / 2);
			tempList[4] = new Point(
				beadSize.x / 2 - railWid / 2 - 2,
				0);
			vFillPolygon(g, beadShade[shadeDot],
				beadShade[shadeDot],
				tempList, 4);
		} else {
		    if (beadSize.x >= beadSize.y + railWid + 2) {
		      {
			g.setColor(beadShade[shadeLine]);
			vDrawCircle(g, beadSize.y,
				beadSize.x / 2 -
				(beadSize.x - beadSize.y) / 2,
				beadSize.y / 2);
			vDrawCircle(g, beadSize.y,
				beadSize.x / 2 - 1 +
				(beadSize.x - beadSize.y) / 2,
				beadSize.y / 2);
		      }
			g.setColor(beadShade[shadeFill]);
			vFillCircle(g, beadSize.y,
				beadSize.x / 2 -
				(beadSize.x - beadSize.y) / 2,
				beadSize.y / 2);
			vFillCircle(g, beadSize.y,
				beadSize.x / 2 - 1 +
				(beadSize.x - beadSize.y) / 2,
				beadSize.y / 2);
		      {
			g.setColor(beadShade[shadeLine]);
			vDrawRect(g, beadSize.x / 2 -
				(beadSize.x - beadSize.y) / 2, 0,
				beadSize.x - beadSize.y, beadSize.y);
		      }
			g.setColor(beadShade[shadeFill]);
			vfillRect(g, beadSize.x / 2 -
				(beadSize.x - beadSize.y) / 2, 0,
				beadSize.x - beadSize.y + 1,
				beadSize.y);
			g.setColor(beadShade[shadeDot]);
			/*if (pressed == 0) {*/
				vFillCircle(g, beadSize.y / 6,
					-beadSize.y / 5 +
					beadSize.x / 2 -
					(beadSize.x - beadSize.y) / 2,
					-beadSize.y / 5 +
					(beadSize.y - 1) / 2);
			/* } else {
				vFillCircle(g, beadSize.y / 6,
					beadSize.y / 5 +
					beadSize.x / 2 +
					(beadSize.x - beadSize.y) / 2,
					beadSize.y / 5 +
					(beadSize.y - 1) / 2);
			}*/
		  } else {
			    /*(beadSize.x < beadSize.y + railWid + 2)*/
			    int beadDiameter = beadSize.x - railWid - 2;
			    int beadOffset = beadSize.y - beadDiameter;

			{
			    g.setColor(beadShade[shadeLine]);
			    vDrawCircle(g, beadDiameter,
				beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2);
			    vDrawCircle(g, beadDiameter,
				beadSize.x / 2 +
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2);
			    vDrawCircle(g, beadDiameter,
				beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2 + beadOffset);
			    vDrawCircle(g, beadDiameter,
				beadSize.x / 2 +
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2 + beadOffset);
			}
			    g.setColor(beadShade[shadeFill]);
			    vFillCircle(g, beadDiameter,
				beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2);
			    vFillCircle(g, beadDiameter,
				beadSize.x / 2 +
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2);
			    vFillCircle(g, beadDiameter,
				beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2 + beadOffset);
			    vFillCircle(g, beadDiameter,
				beadSize.x / 2 +
				(beadSize.x - beadDiameter) / 2,
				beadDiameter / 2 + beadOffset);
			{
			    g.setColor(beadShade[shadeLine]);
			    vDrawRect(g, beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2, 0,
				beadSize.x - beadDiameter,
				beadSize.y);
			    vDrawRect(g, beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2 -
				beadDiameter / 2,
				beadDiameter / 2,
				beadSize.x, beadOffset);
			}
			    g.setColor(beadShade[shadeFill]);
			    vfillRect(g, beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2, 0,
				beadSize.x - beadDiameter + 1,
				beadSize.y);
			    vfillRect(g, beadSize.x / 2 -
				(beadSize.x - beadDiameter) / 2 -
				beadDiameter / 2,
				beadDiameter / 2,
				beadSize.x + 1, beadOffset + 1);
			    g.setColor(beadShade[shadeDot]);
			    /*if (pressed == 0) {*/
				vFillCircle(g, beadDiameter / 6,
					-beadDiameter / 5 +
					beadSize.x / 2 -
					(beadSize.x -
					beadDiameter) / 2,
					-beadDiameter / 5 +
					(beadDiameter - 1) / 2);
			    /*} else {
					vFillCircle(g, beadDiameter / 6,
					  beadDiameter / 5 +
					  beadSize.x / 2 +
					  (beadSize.x -
					  beadDiameter) / 2,
					  beadDiameter / 5 +
					  (beadDiameter - 1) / 2);
			    }*/
			}
		}
	}

	void drawAllBufferedBeads() {
		int color, pressed, shade;

		for (color = 0; color < 3; color++) {
			for (pressed = 0; pressed < 2; pressed++) {
				for (shade = 0; shade < 2; shade++) {
					drawBufferedBead(bufferedBeadGraphics
						[color][pressed][shade],
						color, pressed, shade);
				}
			}
		}
	}

	int newPos(int dir, int inc) {
		return (((dir == UP) ? -1 : 1) * inc);
	}

	void animateSlide(int deck, int rail, int bead, int position,
			int j, int spaces, int dir, int delay) {
		int space, inc, aBead, numBeads;
		int gapJ;
		int posOff, beadOff;

		if (dir == UP)
			numBeads = bead - position;
		else
			numBeads = position - bead + 1;
		for (space = 0; space < spaces; space++) {
			gapJ = pos.y / numSlices;
			if (gapJ == 0)
				gapJ++;
			for (inc = 0; inc < pos.y + gapJ; inc += gapJ) {
				if (inc > pos.y) {
					gapJ = pos.y + gapJ - inc;
					inc = pos.y;
				}
				for (aBead = numBeads - 1; aBead >= 0; aBead--) {
					beadOff = newPos(dir, aBead);
					posOff = newPos(dir, (aBead + space));
					/* actual bead, bead position */
					drawBead(deck, rail, bead + beadOff, j,
						true, true, false, 0, 0,
						newPos(dir, inc) +
						posOff * pos.y);
					/* Erase old slivers */
					if ((vertical && dir == DOWN) ||
							(!vertical &&
							dir == UP)) {
						drawRail(deck, rail,
							j + posOff, false,
							pos.y - inc, gapJ);
					} else {
						drawRail(deck, rail,
							j + posOff, false,
							inc - gapJ +
							pressOffsetY, gapJ);
					}
				}
				try {
					Thread.sleep(delay);
				} catch (Exception e) {
					//e.printStackTrace();
				}
			}
		}
	}

	void addBead(int d, int p) {
		int position = numDigits - 2 - p;
		if (position < 0)
			System.out.println("addBead(" + d + "," + p + ")" +
				numDigits);
		int digit = abacusMath.char2Int(digits.charAt(position));
		int b = base;

		digit += d;
		if ((decks[BOTTOM].piece != 0 &&
				(p == decimalPosition - 1)) ||
				(checkSubdeck(3) &&
				(p == decimalPosition - 2))) {
			b = decks[BOTTOM].piece;
			if (decks[TOP].piece != 0)
				b *= decks[TOP].piece;
		} else if (decks[BOTTOM].piecePercent != 0 &&
				(p == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1))) {
			b = decks[BOTTOM].piecePercent;
			if (decks[TOP].piecePercent != 0)
				b *= decks[TOP].piecePercent;
		}
		digits.setCharAt(position, abacusMath.int2Char(digit % b));
		if (digit >= b) {
			if (checkSubdeck(3) &&
					(p + 1 == decimalPosition - 1))
				addBead(digit / b, p + 1);
			else if ((decks[BOTTOM].piece != 0 &&
					(p + 1 == decimalPosition - 1)) ||
					(decks[BOTTOM].piecePercent != 0 &&
					(p + 1 == decimalPosition -
					shiftPercent - 1 -
					((decks[BOTTOM].piece == 0) ? 0 : 1))))
				addBead(digit / b, p + 2);
			else {
				addBead(digit / b, p + 1);
				if (anomaly != 0) {
					if (p + 1 == shiftAnomaly +
							decimalPosition) {
						carryAnomaly = true;
					} else if (anomalySq != 0 &&
							p + 1 == shiftAnomaly +
							shiftAnomalySq +
							decimalPosition) {
						carryAnomalySq = true;
					}
				}
			}
		}
	}

	void subBead(int d, int p) {
		int position = numDigits - 2 - p;

		int digit = abacusMath.char2Int(digits.charAt(position));
		int b = base;

		digit -= d;
		if ((decks[BOTTOM].piece != 0 &&
				(p == decimalPosition - 1)) ||
				(checkSubdeck(3) &&
				(p == decimalPosition - 2))) {
			b = decks[BOTTOM].piece;
			if (decks[TOP].piece != 0)
				b *= decks[TOP].piece;
		} else if (decks[BOTTOM].piecePercent != 0 &&
				(p == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1))) {
			b = decks[BOTTOM].piecePercent;
			if (decks[TOP].piecePercent != 0)
				b *= decks[TOP].piecePercent;
		}
		digits.setCharAt(position,
			abacusMath.int2Char(((digit + b) % b)));
		if (digit < 0) {
			if (checkSubdeck(3) &&
					(p + 1 == decimalPosition - 1))
				subBead(1 + (-1 - digit) / b, p + 1);
			else if ((decks[BOTTOM].piece != 0 &&
					(p + 1 == decimalPosition - 1)) ||
					(decks[BOTTOM].piecePercent != 0 &&
					(p + 1 == decimalPosition -
					shiftPercent - 1 -
					((decks[BOTTOM].piece == 0) ? 0 : 1))))
				subBead(1 + (-1 - digit) / b, p + 2);
			else {
				subBead(1 + (-1 - digit) / b, p + 1);
				if (anomaly != 0) {
					if (p + 1 == shiftAnomaly +
							decimalPosition) {
						carryAnomaly = false;
					} else if (anomalySq != 0 &&
							p + 1 == shiftAnomaly +
							shiftAnomalySq +
							decimalPosition) {
						carryAnomalySq = false;
					}
				}
			}
		}
	}

	void moveUp(int deck, int rail, int j,
			int factor, int spaces,
			int fast, int delay) {
		int pos, rdeck, rj, rpos;

		if (j > AbacusInterface.MAX_BASE + spaces) {
			System.out.println("corruption (moveUp) " + j + " > " +
				(AbacusInterface.MAX_BASE + spaces) + ".");
			return;
		}
		if (checkSubdeck(3) && rail == decimalPosition - 2) {
			pos = subdecks[deck].position;
			rdeck = BOTTOM;
			rj = j + numberSubbeadsOffset(deck);
			rpos = pos + numberSubbeadsOffset(deck);
		} else {
			pos = decks[deck].position[rail];
			rdeck = deck;
			rj = j;
			rpos = pos;
		}
		if (j > pos + spaces) {
			int temp = rpos;

			if (fast == AbacusInterface.INSTANT || delay == 0) {
				int l;

				for (l = 0; l < spaces; l++) {
					int k;

					for (k = temp + spaces + 1; k <= rj; k++) {
						drawBead(rdeck, rail,
							k - spaces, k - l,
							false, false, false,
							0, 0, 0);
						drawBead(rdeck, rail,
							k - spaces, k - l - 1,
							true, false, false,
							0, 0, 0);
					}
					if (l + 1 != spaces) {
					    try {
						Thread.sleep(delay / fast);
					    } catch (Exception e) {
						//e.printStackTrace();
					    }
					}
				}
			} else {
				animateSlide(rdeck, rail, rj - spaces, rpos,
					rj, spaces, UP,
					delay / (numSlices * fast));
			}
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_MOVE);
			if (((AbacusApplet)applet).getToggleSound())
				((AbacusApplet)applet).playBumpAudio();
			if (checkSubdeck(3) && rail == decimalPosition - 2) {
				subdecks[deck].position = j - spaces;
			} else {
				decks[deck].position[rail] = j - spaces;
			}
			if (decks[rdeck].orientation) {
				subBead(factor * (rj - spaces - temp), rail);
				setCounter(deck, rail, -(rj - spaces - temp));
			} else {	/* decks[rdeck].orientation == DOWN */
				addBead(factor * (rj - spaces - temp), rail);
				setCounter(deck, rail, rj - spaces - temp);
			}
		}
	}

	void moveDown(int deck, int rail, int j, int factor, int spaces,
			int fast, int delay) {
		int pos, rdeck, rj, rpos;

		if (-j > AbacusInterface.MAX_BASE) {
			System.out.println("corruption (moveDown) " + -j +
				" > " + AbacusInterface.MAX_BASE + ".");
			return;
		}
		if (checkSubdeck(3) && rail == decimalPosition - 2) {
			pos = subdecks[deck].position;
			rdeck = BOTTOM;
			rj = j + numberSubbeadsOffset(deck);
			rpos = pos + numberSubbeadsOffset(deck);
		} else {
			pos = decks[deck].position[rail];
			rdeck = deck;
			rj = j;
			rpos = pos;
		}
		if (j <= pos) {
			int temp = rpos;

			if (fast == AbacusInterface.INSTANT || delay == 0) {
				int l;

				for (l = 0; l < spaces; l++) {
					int k;

					for (k = temp; k >= rj; k--) {
						drawBead(rdeck, rail, k, k + l,
							false, false, false,
							0, 0, 0);
						drawBead(rdeck, rail, k,
							k + l + 1,
							true, false, false,
							0, 0, 0);
					}
					if (l + 1 != spaces) {
					    try {
						Thread.sleep(delay / fast);
					    } catch (Exception e) {
						//e.printStackTrace();
					    }
					}
				}
			} else {
				animateSlide(rdeck, rail, rj, rpos,
					rj, spaces, DOWN,
					delay / (numSlices * fast));
			}
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_MOVE);
			if (((AbacusApplet)applet).getToggleSound())
				((AbacusApplet)applet).playBumpAudio();
			if (checkSubdeck(3) && rail == decimalPosition - 2) {
				subdecks[deck].position = j - 1;
			} else {
				decks[deck].position[rail] = j - 1;
			}
			if (decks[rdeck].orientation) {
				addBead(factor * (temp - rj + 1), rail);
				setCounter(deck, rail, temp - rj + 1);
			} else {	/* decks[rdeck].orientation == DOWN */
				subBead(factor * (temp - rj + 1), rail);
				setCounter(deck, rail, -(temp - rj + 1));
			}
		}
	}

	void moveBeadsUp(int deck, int rail, int j, boolean fast) {
		int factor = 1, pieces, piecePercents, spaces;

		if (debug)
			System.out.println("moveBeadsUp: deck " + deck +
				", rail " + rail + ", j " + j);
		if (sign && (rail == rails - 1)) {
			if (deck == BOTTOM) {
				factor = 0;
				spaces = decks[deck].room - 1;
				if (spaces > 0)
					moveUp(deck, rail, j, factor, spaces,
						AbacusInterface.NORMAL,
						(fast) ? 0 : delay *
						decks[BOTTOM].spaces /
						(decks[BOTTOM].room - 1));
			}
		} else if (decks[BOTTOM].piece != 0 &&
				(rail == decimalPosition - 1)) {
			pieces = numberPieces(deck);
			if (deck == TOP)
				factor *= decks[BOTTOM].piece;
			spaces = decks[deck].room - pieces;
			if (spaces > 0)
				moveUp(deck, rail, j, factor, spaces,
					AbacusInterface.NORMAL,
					(fast) ? 0 : delay *
					decks[deck].spaces /
					(decks[deck].room - pieces));
		} else if (decks[BOTTOM].piecePercent != 0 &&
				(rail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1))) {
			piecePercents = numberPiecePercents(deck);
			if (deck == TOP)
				factor *= decks[BOTTOM].piecePercent;
			spaces = decks[deck].room - piecePercents;
			if (spaces > 0)
				moveUp(deck, rail, j, factor, spaces,
					AbacusInterface.NORMAL,
					(fast) ? 0 : delay *
					decks[BOTTOM].spaces /
					(decks[BOTTOM].room - piecePercents));
		} else if (checkSubdeck(3) && (rail == decimalPosition - 2 ||
				rail == decimalPosition - 3)) {
			if (rail == decimalPosition - 2) {
				moveUp(deck, rail, j, convertRomanFactor(deck),
					AbacusInterface.SUBDECK_SPACE,
					AbacusInterface.NORMAL,
					(fast) ? 0 :
					delay * AbacusInterface.SUBDECK_SPACE);
			}
		} else if (j > decks[deck].position[rail] +
				decks[deck].spaces) {
			factor = decks[deck].factor;
			spaces = decks[deck].spaces;
			//System.out.println("moveUp(" + deck + "," + j + "," + factor + "," + rail + ")");
			moveUp(deck, rail, j, factor, spaces,
				AbacusInterface.NORMAL,
				(fast) ? 0 : delay);
		}
	}

	void moveBeadsDown(int deck, int rail, int j, boolean fast) {
		int factor = 1, pieces, piecePercents, spaces;

		if (debug)
			System.out.println("moveBeadsDown: deck " + deck +
				", rail " + rail + ", j " + j);
		if ((rail == rails - 1) && sign) {
			if (deck == BOTTOM) {
				factor = 0;
				spaces = decks[deck].room - 1;
				if (spaces > 0)
					moveDown(deck, rail, j, factor, spaces,
						AbacusInterface.NORMAL,
						(fast) ? 0 : delay *
						decks[BOTTOM].spaces /
						(decks[BOTTOM].room - 1));
			}
		} else if (decks[BOTTOM].piece != 0 &&
				(rail == decimalPosition - 1)) {
			pieces = numberPieces(deck);
			if (deck == TOP)
				factor *= decks[BOTTOM].piece;
			spaces = decks[deck].room - pieces;
			if (spaces > 0)
				moveDown(deck, rail, j, factor, spaces,
					AbacusInterface.NORMAL,
					(fast) ? 0 : delay *
					decks[BOTTOM].spaces /
					(decks[BOTTOM].room - pieces));
		} else if (decks[BOTTOM].piecePercent != 0 &&
				(rail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1))) {
			piecePercents = numberPiecePercents(deck);
			if (deck == TOP)
				factor *= decks[BOTTOM].piecePercent;
			spaces = decks[deck].room - piecePercents;
			if (spaces > 0)
				moveDown(deck, rail, j, factor, spaces,
					AbacusInterface.NORMAL,
					(fast) ? 0 : delay *
					decks[BOTTOM].spaces /
					(decks[BOTTOM].room - piecePercents));
		} else if (checkSubdeck(3) && (rail == decimalPosition - 2 ||
				rail == decimalPosition - 3)) {
			if (rail == decimalPosition - 2) {
				moveDown(deck, rail, j, convertRomanFactor(deck),
					AbacusInterface.SUBDECK_SPACE,
					AbacusInterface.NORMAL,
					(fast) ? 0 :
					delay * AbacusInterface.SUBDECK_SPACE);
			}
		} else if (j <= decks[deck].position[rail]) {
			factor = decks[deck].factor;
			spaces = decks[deck].spaces;
			//System.out.println("moveDown(" + deck + "," + j + "," + factor + "," + rail + ")");
			moveDown(deck, rail, j, factor, spaces,
				AbacusInterface.NORMAL,
				(fast) ? 0 : delay);
		}
	}

	boolean positionToBead(int positionX, int positionY) {
		int pieces, piecePercents;
		int x = positionX, y = positionY;

		if (vertical) {
			int temp = x;

			x = y;
			y = frameSize.y - 1 - temp;
		}
		x -= offset.x;
		y -= offset.y;
		if (y > decks[TOP].height) {
			y = y - decks[TOP].height - 3;
			currentDeck = BOTTOM;
		} else {
			currentDeck = TOP;
		}
		if (decks[currentDeck].number == 0) {
			return false;
		}
		currentRail = rails - 1 - (x - delta.x / 2) /
			pos.x;
		currentPosition = (y - delta.y / 2) / pos.y + 1;
		if (currentRail < 0)
			currentRail = 0;
		else if (currentRail >= rails)
			currentRail = rails - 1;
		if (currentPosition < 1)
			currentPosition = 1;
		else if (currentPosition > decks[currentDeck].room)
			currentPosition = decks[currentDeck].room;
		if (currentRail == rails - 1 && sign) {
			if (currentDeck == TOP)
				return false;
			return ((currentPosition == 1 &&
				decks[currentDeck].position
				[currentRail] == 1) ||
				(currentPosition == decks[currentDeck].room &&
				decks[currentDeck].position
				[currentRail] == 0));
		}
		if (decks[BOTTOM].piece != 0 &&
				(currentRail == decimalPosition - 1)) {
			pieces = numberPieces(currentDeck);
			if (currentDeck == TOP && pieces == 0)
				return false;
			return ((currentPosition >
				decks[currentDeck].position[currentRail] +
				decks[currentDeck].room - pieces) ||
				(currentPosition <=
				decks[currentDeck].position[currentRail]));
		}
		if (decks[BOTTOM].piecePercent != 0 &&
				(currentRail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1))) {
			piecePercents = numberPiecePercents(currentDeck);
			if (currentDeck == TOP && piecePercents == 0)
				return false;
			return ((currentPosition >
				decks[currentDeck].position[currentRail] +
				decks[currentDeck].room - piecePercents) ||
				(currentPosition <=
				decks[currentDeck].position[currentRail]));
		}
		if (checkSubdeck(3) && currentRail == decimalPosition - 3) {
			return false;
		} else if (checkSubdeck(3) &&
				currentRail == decimalPosition - 2) {
			int d, beads;

			if (currentDeck == TOP)
				return false;
			d = subdeckPosition(currentPosition);
			currentPosition = positionSubdeck(currentPosition);
			currentDeck = d;
			beads = numberSubbeads(d);
			return ((currentPosition >
				subdecks[currentDeck].position +
				subdecks[currentDeck].room - beads) ||
				(currentPosition <=
				subdecks[currentDeck].position));
			}
		return ((currentPosition >
			decks[currentDeck].position[currentRail] +
			decks[currentDeck].spaces) ||
			(currentPosition <=
			decks[currentDeck].position[currentRail]));
	}

	void moveBeadsByPos(int deck, int rail, int pos, boolean fast) {
		int bead;

		if (sign && (rail == rails - 1) && deck == TOP)
			return;
		if (decks[BOTTOM].piece != 0 &&
				(rail == decimalPosition - 1) &&
				deck == TOP && (decks[TOP].number == 0 ||
				decks[TOP].piece == 0))
			return;
		if (decks[BOTTOM].piecePercent != 0 &&
				(rail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ? 0 : 1)) &&
				deck == TOP && (decks[TOP].number == 0 ||
				decks[TOP].piecePercent == 0))
			return;
		if (checkSubdeck(3) && rail == decimalPosition - 3)
			return;
		if (checkSubdeck(3) && rail == decimalPosition - 2)
			bead = subdecks[deck].position;
		else
			bead = decks[deck].position[rail];
		if (pos <= bead) {
			moveBeadsDown(deck, rail, pos, fast);
		} else {
			moveBeadsUp(deck, rail, pos, fast);
		}
	}

	void shiftBar(int oldDecimalPosition) {
		int deck, rail;
		int[] pieces = new int[MAX_DECKS];
		int[] piecePercents = new int[MAX_DECKS];
		int pieceRail = decimalPosition - 1;
		int piecePercentRail = decimalPosition - shiftPercent -
			1 - ((decks[BOTTOM].piece == 0) ? 0 : 1);
		int oldPieceRail = oldDecimalPosition - 1;
		int oldPiecePercentRail = oldDecimalPosition - shiftPercent -
			1 - ((decks[BOTTOM].piece == 0) ? 0 : 1);
		char p = '0', pp = '0';

		pieces[TOP] = 0;
		pieces[BOTTOM] = 0;
		piecePercents[TOP] = 0;
		piecePercents[BOTTOM] = 0;
		if (decks[BOTTOM].piece != 0) {
			int digit = rails + CARRY - oldDecimalPosition;

			pieces[BOTTOM] =
				decks[BOTTOM].position[oldPieceRail];
			if (decks[TOP].piece != 0 ||
					decks[TOP].number == 0)
				pieces[TOP] =
					decks[TOP].position[oldPieceRail];
			p = digits.charAt(digit);
			digits.setCharAt(digit, '0');

		}
		if (decks[BOTTOM].piecePercent != 0) {
			int digit = rails + CARRY -
				oldDecimalPosition + shiftPercent +
				((decks[BOTTOM].piece == 0) ? 0 : 1);

			piecePercents[BOTTOM] =
				decks[BOTTOM].position[oldPiecePercentRail];
			if (decks[TOP].piecePercent != 0 ||
					decks[TOP].number == 0)
				piecePercents[TOP] =
					decks[TOP].position[oldPiecePercentRail];
			pp = digits.charAt(digit);
			digits.setCharAt(digit, '0');
		}
		/* shift around */
		if (oldDecimalPosition < decimalPosition) {
			for (rail = oldPieceRail; rail < pieceRail; rail++) {
				for (deck = BOTTOM; deck <= TOP; deck++) {
					decks[deck].position[rail] =
						decks[deck].position[rail + 1];
					if (decks[BOTTOM].piecePercent != 0)
						decks[deck].position[rail - shiftPercent - 1] =
							decks[deck].position[rail - shiftPercent];
				}
			}
			for (rail = rails + CARRY - oldDecimalPosition;
					rail > rails + CARRY -
					decimalPosition; rail--) {
				digits.setCharAt(rail, digits.charAt(rail - 1));
				if (decks[BOTTOM].piecePercent != 0)
					digits.setCharAt(rail + shiftPercent + 1,
						digits.charAt(rail + shiftPercent));
			}
		} else if (oldDecimalPosition > decimalPosition) {
			for (rail = oldPieceRail; rail > pieceRail; rail--) {
				for (deck = BOTTOM; deck <= TOP; deck++) {
					decks[deck].position[rail] =
						decks[deck].position[rail - 1];
					if (decks[BOTTOM].piecePercent != 0)
						decks[deck].position[rail - shiftPercent - 1] =
							decks[deck].position[rail - shiftPercent - 2];
				}
			}
			for (rail = rails + CARRY - oldDecimalPosition;
					rail < rails + CARRY -
					decimalPosition; rail++) {
				digits.setCharAt(rail, digits.charAt(rail + 1));
				if (decks[BOTTOM].piecePercent != 0)
					digits.setCharAt(rail + shiftPercent + 1,
						digits.charAt(rail + shiftPercent + 2));
			}
		}
		if (decks[BOTTOM].piece != 0) {
			decks[BOTTOM].position[pieceRail] = pieces[BOTTOM];
			if (decks[TOP].piece == 0 ||
					decks[TOP].number == 0)
				decks[TOP].position[pieceRail] = 0;
			else
				decks[TOP].position[pieceRail] = pieces[TOP];
			digits.setCharAt(rails + CARRY - decimalPosition, p);
		}
		if (decks[BOTTOM].piecePercent != 0) {
			decks[BOTTOM].position[piecePercentRail] =
				piecePercents[BOTTOM];
			if (decks[TOP].piecePercent == 0 &&
					decks[TOP].number == 0)
				decks[TOP].position[piecePercentRail] = 0;
			else
				decks[TOP].position[piecePercentRail] =
					piecePercents[TOP];
			digits.setCharAt(rails + CARRY -
				decimalPosition + shiftPercent +
				((decks[BOTTOM].piece == 0) ? 0 : 1), pp);
		}
	}

	void clearAllBeads() {
		int rail, deck;

		for (rail = 0; rail < rails; rail++) {
			for (deck = BOTTOM; deck <= TOP; deck++) {
				if (sign && (rail == rails - 1) &&
						deck == TOP) {
					continue;
				}
				if (decks[BOTTOM].piece != 0 &&
						(rail == decimalPosition - 1) &&
						deck == TOP &&
						(decks[TOP].number == 0 ||
						decks[TOP].piece == 0)) {
					continue;
				}
				if (decks[BOTTOM].piecePercent != 0 &&
						(rail == decimalPosition -
						shiftPercent - 1 -
						((decks[BOTTOM].piece == 0) ?
						0 : 1)) &&
						deck == TOP &&
						(decks[TOP].number == 0 ||
						decks[TOP].piecePercent == 0)) {
					continue;
				}
				if (checkSubdeck(3) && (rail ==
						decimalPosition - 2)) {
					if (deck == BOTTOM) {
					  int d;

					  for (d = 0; d < subdeck; d++)
					    if (decks[BOTTOM].orientation)
					      moveBeadsUp(d, rail,
						subdecks[d].room, true);
					    else
					      moveBeadsDown(d, rail, 1, true);
					}
					continue;
				}
				if (decks[deck].orientation)
					moveBeadsUp(deck, rail,
						decks[deck].room, true);
				else	/* decks[deck].orientation == DOWN */
					moveBeadsDown(deck, rail, 1, true);
			}
		}
	}

	void complementRails() {
		int rail, deck;

		for (rail = 0; rail < rails; rail++) {
			for (deck = BOTTOM; deck <= TOP; deck++) {
				if (sign && (rail == rails - 1) &&
						deck == TOP) {
					continue;
				}
				if (decks[BOTTOM].piece != 0 &&
						(rail == decimalPosition - 1)) {
					continue;
				}
				if (decks[BOTTOM].piecePercent != 0 &&
						(rail == decimalPosition -
						shiftPercent - 1 -
						((decks[BOTTOM].piece == 0) ?
						0 : 1))) {
					continue;
				}
				if (checkSubdeck(3) && (rail ==
						decimalPosition - 2)) {
					continue;
				}
				{
					int number = decks[deck].number;
					int bead = decks[deck].position[rail];
					boolean orient = decks[deck].orientation;
					int offset = 0;
					int factor, extra;

					if (deck == 1 && decks[0].number >= base - 1)
						continue;
					factor = (deck == 0) ? decks[1].factor :
						base / decks[deck].factor;
					extra = number - factor + 1;
					if (orient) {
						if (bead > decks[deck].number - extra + 1 || bead < extra)
							continue; /* not really well defined here so ignore */
						if (bead - extra < factor / 2)
							offset = decks[deck].spaces - 1 + 2 * extra;
						else
							offset = 2 * extra;
						moveBeadsByPos(deck, rail, factor - bead + offset, true);
					} else {
						if (bead > decks[deck].number - extra || bead < 0)
							continue; /* not really well defined here so ignore */
						if (bead < factor / 2)
							offset = decks[deck].spaces - 1;
						moveBeadsByPos(deck, rail, factor - bead + offset, true);
					}
				}
			}
		}
	}

	void drawBeadRail(int rail, boolean highlight) {
		int deck;

		if ((decks[BOTTOM].piece != 0 &&
				(rail == decimalPosition - 1)) ||
				(decks[BOTTOM].piecePercent != 0 &&
				(rail == decimalPosition -
				shiftPercent - 1 -
				((decks[BOTTOM].piece == 0) ?
				0 : 1))) || (checkSubdeck(3) &&
				(rail == decimalPosition - 2 ||
				rail == decimalPosition - 3))) {
			return;
		}
		for (deck = BOTTOM; deck <= TOP; deck++) {
			int j;

			for (j = 1; j <= decks[deck].position[rail]; j++)
				drawBead(deck, rail, j, j,
					true, false, highlight, 0, 0, 0);
			for (j = decks[deck].position[rail] + 1;
			   j < decks[deck].spaces +
			   decks[deck].position[rail] + 1; j++)
				drawBead(deck, rail, 0, j,
					false, false, highlight, 0, 0, 0);
			for (j = decks[deck].spaces +
			   decks[deck].position[rail] + 1;
			   j <= decks[deck].room; j++)
				drawBead(deck, rail,
					j - decks[deck].spaces, j,
					true, false, highlight, 0, 0, 0);
		}
	}

	void drawAllBeads() {
		int deck, rail, j, spaces;

		if (sign) {
			deck = BOTTOM;
			rail = rails - 1;
			drawBead(deck, rail, 1, 1,
				(decks[deck].position[rail] == 1),
				false, false, 0, 0, 0);
			for (j = 2; j < decks[deck].room; j++)
				drawBead(deck, rail, 0, j, false,
					false, false, 0, 0, 0);
			drawBead(deck, rail, 1, decks[deck].room,
				(decks[deck].position[rail] == 0),
				false, false, 0, 0, 0);
		}
		if (decks[BOTTOM].piece != 0) {
			int pieces = 0;

			for (deck = BOTTOM; deck <= TOP; deck++) {
				rail = decimalPosition - 1;
				pieces = numberPieces(deck);
				if (pieces == 0)
					continue;
				spaces = decks[deck].room - pieces;
				for (j = 1; j <= decks[deck].position[rail];
						j++) {
					drawBead(deck, rail, j, j,
						true, false, false, 0, 0, 0);
				}
				for (j = decks[deck].position[rail] + 1;
						j < spaces +
						decks[deck].position[rail] +
						1; j++) {
					drawBead(deck, rail, 0, j,
						false, false, false, 0, 0, 0);
				}
				for (j = spaces + decks[deck].position[rail] + 1;
						j <= decks[deck].room; j++) {
					drawBead(deck, rail, j - spaces, j,
						true, false, false, 0, 0, 0);
				}
			}
		}
		if (decks[BOTTOM].piece != 0 &&
				decks[BOTTOM].piecePercent != 0) {
			int piecePercents = 0;

			for (deck = BOTTOM; deck <= TOP; deck++) {
				rail = decimalPosition -
					shiftPercent - 1 -
					((decks[BOTTOM].piece == 0) ? 0 : 1);
				piecePercents = numberPiecePercents(deck);
				if (piecePercents == 0)
					continue;
				spaces = decks[deck].room - piecePercents;
				for (j = 1; j <= decks[deck].position[rail]; j++)
					drawBead(deck, rail, j, j,
						true, false, false, 0, 0, 0);
				for (j = decks[deck].position[rail] + 1;
						j < spaces +
						decks[deck].position[rail] + 1; j++)
					drawBead(deck, rail, 0, j,
						false, false, false, 0, 0, 0);
				for (j = spaces + decks[deck].position[rail] + 1;
				     j <= decks[deck].room; j++)
					drawBead(deck, rail, j - spaces, j,
						true, false, false, 0, 0, 0);
			}
		}
		if (checkSubdeck(3)) {
			int offset, d = BOTTOM;

			rail = decimalPosition - 2;
			for (deck = 0; deck < subdeck; deck++) {
				spaces = AbacusInterface.SUBDECK_SPACE;
				offset = numberSubbeadsOffset(deck);
				for (j = 1; j <= subdecks[deck].position;
						j++) {
					drawBead(d, rail, j + offset,
						j + offset,
						true, false, false, 0, 0, 0);
				}
				for (j = subdecks[deck].position + 1;
						j < spaces +
						subdecks[deck].position + 1;
						j++) {
					drawBead(d, rail, offset, j + offset,
						false, false, false, 0, 0, 0);
				}
				for (j = spaces + subdecks[deck].position + 1;
						j <= subdecks[deck].room;
						j++) {
					drawBead(d, rail, j + offset - spaces,
						j + offset,
						true, false, false, 0, 0, 0);
				}
			}
		}
		for (rail = 0; rail < rails - ((sign) ? 1 : 0);
				rail++) {
			drawBeadRail(rail, false);
		}
		setCounter(0, decimalPosition, 0);
	}

	void setDecimal(int newRail) {
		int rail = newRail;
		int j;

		if (((AbacusApplet)applet).getScript()) {
			((AbacusApplet)applet).callbackAbacus(
				/*AbacusInterface.ACTION_SCRIPT*/
				AbacusInterface.PRIMARY, 2,
				rail - decimalPosition, 0);
		}
		if (checkSubdeck(3)) {
			return;
		}
		if (rail <= shiftPercent + 1 &&
				decks[BOTTOM].piecePercent != 0) {
			decks[BOTTOM].piecePercent = 0;
			decks[TOP].piecePercent = 0;
			if (rail <= 0 && decks[BOTTOM].piece != 0)
				rail = 1;
			deleteSpecialRail(false, false, true);
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_QUARTER_PERCENT);
		} else if (rail <= 0 && decks[BOTTOM].piece != 0) {
			decks[BOTTOM].piece = 0;
			decks[TOP].piece = 0;
			deleteSpecialRail(false, true, false);
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_QUARTER);
		}
		if (sign && rail >= rails - 1)
			rail = rails - 2;
		/*if (decks[BOTTOM].piece != 0 ||
				decks[BOTTOM].piecePercent != 0) {
			drawFrame(false);
		}*/
		drawFrame(false);
		j = decimalPosition;
		decimalPosition = rail;
		drawFrame(true);
		if (decks[BOTTOM].piece != 0 ||
				decks[BOTTOM].piecePercent != 0) {
			shiftBar(j);
		}
		try {
			/* Fixes a drawing problem on lowest value beads, not sure why */
			Thread.sleep(100);
		} catch (Exception e) {
			//e.printStackTrace();
		}
		drawAllBeads();
		setCounter(0, decimalPosition, 0);
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_PLACE);
		if (((AbacusApplet)applet).getToggleSound())
			((AbacusApplet)applet).playMoveAudio();
	}

	void moveBeadsByValue(int deck, int rail, int number, boolean fast) {
		if (debug)
			System.out.println("moveBeadsByValue: deck " + deck +
				", rail " + rail + ", number " + number);
		if (deck != BOTTOM && deck != TOP) {
			setDecimal(number + decimalPosition);
			return;
		}
		if (sign && (rail == rails - 1)) {
			if (deck == TOP)
				return;
			if (number <= decks[deck].position[rail]) {
				moveBeadsDown(deck, rail, 1, fast);
			} else {
				moveBeadsUp(deck, rail,
					decks[BOTTOM].room, fast);
			}
			return;
		} else if ((decks[deck].orientation && number < 0) ||
				(!decks[deck].orientation && number > 0)) {
			int spaces = decks[deck].spaces;

			if (decks[BOTTOM].piece != 0 &&
					rail == decimalPosition - 1) {
				spaces = decks[deck].room - numberPieces(deck);
			}
			if (decks[BOTTOM].piecePercent != 0 &&
					rail == decimalPosition - 1 -
					shiftPercent -
					((decks[BOTTOM].piece == 0) ?
					0 : 1)) {
				spaces = decks[deck].room -
					numberPiecePercents(deck);
			}
			moveBeadsUp(deck, rail, spaces +
				decks[deck].position[rail] +
				((number >= 0) ? number : -number),
				fast);
		} else if ((!decks[deck].orientation && number < 0) ||
				(decks[deck].orientation && number > 0)) {
			moveBeadsDown(deck, rail,
				decks[deck].position[rail] + 1 -
				((number >= 0) ? number : -number),
				fast);
		}
	}

	boolean setBeadsForValue(String expression) {
		int i, val = -1, topUnits, bottomUnits;
		int percentPosition = decimalPosition - shiftPercent -
			((decks[BOTTOM].piece == 0) ? 0 : 1);
		int nPieces = 0; int nPiecePercents = 0;

		if (debug)
			System.out.println("setBeadsForValue: expression " +
				expression + ", minusSign " + minusSign);
		for (i = 0; i < expression.trim().length() - CARRY; i++) {
			char a = expression.charAt(i + CARRY);

			val = abacusMath.char2Int(a);
			if (decks[BOTTOM].piece != 0 &&
					rails - i != decimalPosition) {
				nPieces = decks[BOTTOM].piece;
				if (decks[TOP].piece != 0)
					nPieces *= decks[TOP].piece;
			} else if (decks[BOTTOM].piecePercent != 0 &&
					rails - i != percentPosition) {
				nPiecePercents = decks[BOTTOM].piecePercent;
				if (decks[TOP].piecePercent != 0)
					nPiecePercents *=
						decks[TOP].piecePercent;
			}
			if (decks[BOTTOM].piece != 0 &&
					rails - i == decimalPosition) {
				if (val >= nPieces)
					val -= nPieces;
				if (decks[TOP].number == 0) {
					bottomUnits = val % nPieces;
					moveBeadsByValue(BOTTOM,
						rails - i - 1,
						bottomUnits, true);
				} else {
					topUnits = val / decks[BOTTOM].piece;
					bottomUnits = val % decks[BOTTOM].piece;
					if (topUnits > decks[TOP].piece) {
						return false;
					}
					moveBeadsByValue(TOP,
						rails - i - 1,
						topUnits, true);
					moveBeadsByValue(BOTTOM,
						rails - i - 1,
						bottomUnits, true);
				}
			} else if (decks[BOTTOM].piecePercent != 0 &&
					rails - i == percentPosition) {
				if (val >= nPiecePercents)
					val -= nPiecePercents;
				if (decks[TOP].number == 0) {
					bottomUnits = val % nPiecePercents;
					moveBeadsByValue(BOTTOM,
						rails - i - 1,
						bottomUnits, true);
				} else {
					topUnits = val / decks[BOTTOM].piecePercent;
					bottomUnits = val % decks[BOTTOM].piecePercent;
					if (topUnits > decks[TOP].piecePercent) {
						return false;
					}
					moveBeadsByValue(TOP,
						rails - i - 1,
						topUnits, true);
					moveBeadsByValue(BOTTOM,
						rails - i - 1,
						bottomUnits, true);
				}
			} else {
				if (checkSubdeck(3) && rails - i <
						decimalPosition)
					continue;
				topUnits = val / decks[TOP].factor;
				bottomUnits = (val % decks[TOP].factor) /
					decks[BOTTOM].factor;
				if (topUnits > decks[TOP].number) {
					return false;
				}
				moveBeadsByValue(TOP, rails - i - 1,
					topUnits, true);
				moveBeadsByValue(BOTTOM, rails - i - 1,
					bottomUnits, true);
			}
		}
		if (sign) {
			moveBeadsByValue(BOTTOM, rails - 1,
				(decks[BOTTOM].orientation) ?
				((minusSign) ? 0 : decks[BOTTOM].room -
				decks[BOTTOM].spaces) :
				((minusSign) ? decks[BOTTOM].room -
				decks[BOTTOM].spaces : 0), true);
		}
		return true;
	}

	void
	clearRails() {
		clearAllBeads();
		if (demo) {
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_CLEAR);
		}
		if (!emptyCounter()) {
			System.out.println("corruption (clearRails).");
		}
		setCounter(0, decimalPosition, 0); /* needed when 0 */
	}

	void checkDecimal() {
		if (decimalPosition >= rails -
				((sign) ? 1 : 0) +
				((decks[BOTTOM].piece == 0) ? 0 : 1) +
				((decks[BOTTOM].piecePercent == 0) ? 0 : 1)) {
			drawFrame(false);
			decimalPosition = rails - 1 -
				((sign) ? 1 : 0) +
				((decks[BOTTOM].piece == 0) ? 0 : 1) +
				((decks[BOTTOM].piecePercent == 0) ? 0 : 1);
			drawFrame(true);
			setCounter(0, decimalPosition, 0);
		}
	}

	/* via increment/decrement or by sign and quarters */
	void shiftRails(int oldRails, int newRails,
			boolean oldSign, boolean piece, boolean piecePercent) {
		AbacusCanvas old = new AbacusCanvas();
		int deck, rail, offset;
		int[] piecePosition = new int[MAX_DECKS];
		int[] piecePercentPosition = new int[MAX_DECKS];
		int[] pieceNumber = new int[MAX_DECKS];
		int[] piecePercentNumber = new int[MAX_DECKS];
		int shift = newRails - oldRails;
		int decimalShift = 0;
		boolean signPosition = false;

		piecePosition[TOP] = piecePosition[BOTTOM] = 0;
		piecePercentPosition[TOP] = piecePercentPosition[BOTTOM] = 0;
		pieceNumber[BOTTOM] = numberPieces(BOTTOM);
		pieceNumber[TOP] = numberPieces(TOP);
		piecePercentNumber[BOTTOM] = numberPiecePercents(BOTTOM);
		piecePercentNumber[TOP] = numberPiecePercents(TOP);
		old.sign = sign;
		old.decks = new AbacusDeck[MAX_DECKS];
		old.decks[TOP] = new AbacusDeck(oldRails,
			decks[TOP].number, decks[TOP].orientation,
			decks[TOP].factor, decks[TOP].spaces,
			decks[TOP].piece, decks[TOP].piecePercent);
		old.decks[BOTTOM] = new AbacusDeck(oldRails,
			decks[BOTTOM].number, decks[BOTTOM].orientation,
			decks[BOTTOM].factor, decks[BOTTOM].spaces,
			decks[BOTTOM].piece, decks[BOTTOM].piecePercent);
		old.decks[TOP].piece = decks[TOP].piece;
		old.decks[BOTTOM].piece = decks[BOTTOM].piece;
		old.decks[TOP].piecePercent = decks[TOP].piecePercent;
		old.decks[BOTTOM].piecePercent = decks[BOTTOM].piecePercent;
		deck = BOTTOM;
		/* special items added already, 2 is a dummy value */
		if (oldSign)
			old.sign = !sign;
		if (piece) {
			old.decks[BOTTOM].piece =
				(decks[BOTTOM].piece == 0) ? 2 : 0;
		}
		if (piecePercent) {
			old.decks[BOTTOM].piecePercent =
				(decks[BOTTOM].piecePercent == 0) ? 2 : 0;
		}
		/* Save sign, this will be erased */
		if (old.sign && !oldSign)
			signPosition = ((decks[deck].orientation &&
				decks[deck].position[oldRails - 1] == 0) ||
				(!decks[deck].orientation &&
				decks[deck].position[oldRails - 1] != 0));
		old.rails = oldRails + ((shift > 0) ? shift : 0);
		old.decimalPosition = decimalPosition;
		for (deck = BOTTOM; deck <= TOP; deck++) {
			/* Alloc space to save the rails */
			old.decks[deck].position = new int[old.rails];
			/* initialization could be wrong if oriented from top */
			/* current pieces will be initialized later */
			if (decks[deck].orientation) {
				for (rail = 0; rail < old.rails; rail++) {
					old.decks[deck].position[rail] =
						decks[deck].number;
				}
				if (old.decks[BOTTOM].piece == 0 && piece)
					piecePosition[deck] = pieceNumber[deck];
				if (old.decks[BOTTOM].piecePercent == 0 &&
						piecePercent)
					piecePercentPosition[deck] =
						piecePercentNumber[deck];
			}
		}
		/* initialization from old */
		for (deck = BOTTOM; deck <= TOP; deck++) {
			offset = 0;
			if (old.decks[BOTTOM].piece != 0 && !piece)
				offset--;
			if (old.decks[BOTTOM].piecePercent != 0 && !piecePercent)
				offset--;
			for (rail = oldRails - ((old.sign) ? 1 : 0) - 1; rail >= 0; rail--) {
				if (old.decks[BOTTOM].piece != 0 &&
						rail == decimalPosition - 1) {
					if (old.decks[deck].piece != 0 && !piece)
						piecePosition[deck] =
							decks[deck].position[rail];
					offset++;
				} else if (old.decks[BOTTOM].piecePercent != 0 &&
						rail == decimalPosition -
						shiftPercent - 1 -
						((old.decks[BOTTOM].piece == 0) ? 0 : 1)) {
					if (old.decks[deck].piecePercent != 0 &&
							!piecePercent)
						piecePercentPosition[deck] =
							decks[deck].position[rail];
					offset++;
				} else {
					old.decks[deck].position[rail + offset] =
						decks[deck].position[rail];
				}
			}
		}
		rails = newRails;
		if (debug) {
			for (deck = TOP; deck >= BOTTOM; deck--) {
				for (rail = old.rails - 1; rail >= 0; rail--) {
					System.out.print(abacusMath.int2Char(
						old.decks[deck].position[rail]));
				}
				System.out.print(":" + old.rails + ",p" +
					piecePosition[deck] + ",pp" +
					piecePercentPosition[deck]);
				if (old.sign && !oldSign)
					System.out.println(",s" + signPosition);
				else
					System.out.println();
			}
			System.out.println("shiftRails decimalPosition w" +
				decimalPosition + ", rails w" + rails +
				", shift" + shift);
		}
		if (decimalPosition > rails - 1 - ((sign) ? 1 : 0)) {
			decimalShift = decimalPosition;
			decimalPosition = rails - 1 - ((sign) ? 1 : 0);
			decimalShift -= decimalPosition;
		} else if (decimalPosition < ((sign) ? 1 : 0)) {
			decimalPosition = 0;
		}
		offset = 0;
		if (piece && old.decks[BOTTOM].piece == 0) {
			decimalPosition++;
		}
		if (piece && old.decks[BOTTOM].piece != 0) {
			offset--;
			decimalPosition--;
		}
		if (piecePercent && old.decks[BOTTOM].piecePercent == 0) {
			decimalPosition++;
		}
		if (piecePercent && old.decks[BOTTOM].piecePercent != 0) {
			offset--;
			decimalPosition--;
		}
		if (debug)
			System.out.println("shiftRails decimalPosition w" +
				decimalPosition + ", rails w" + rails +
				", shift" + shift + ", offset" + offset);
		resetBeads();
		resizeAbacus();
		drawAllBufferedBeads();
		drawFrame(false);
		drawFrame(true);
		drawAllBeads();
		for (deck = BOTTOM; deck <= TOP; deck++) {
			int localOffset = offset;

			for (rail = 0; rail < rails + ((offset < 0) ? -offset : 0) -
					(((old.sign && !oldSign) ||
					(!old.sign && oldSign)) ? 1 : 0);
					rail++) {
				if (decks[BOTTOM].piece != 0 &&
						rail + localOffset == decimalPosition - 1) {
					if (deck != TOP || pieceNumber[TOP] != 0) {
						if (decks[deck].orientation) {
							moveBeadsByValue(deck, rail + localOffset,
								pieceNumber[deck] -
								piecePosition[deck], true);
						} else {
							moveBeadsByValue(deck, rail + localOffset,
								piecePosition[deck], true);
						}
					}
					localOffset++;
				} else if (decks[BOTTOM].piecePercent != 0 &&
						rail + localOffset == decimalPosition -
						shiftPercent - 1 -
						((decks[BOTTOM].piece == 0) ?
						0 : 1)) {
					if (deck != TOP || piecePercentNumber[TOP] != 0) {
						if (decks[deck].orientation) {
							moveBeadsByValue(deck, rail + localOffset,
								piecePercentNumber[deck] -
								piecePercentPosition[deck], true);
						} else {
							moveBeadsByValue(deck, rail + localOffset,
								piecePercentPosition[deck], true);
						}
					}
					localOffset++;
				}
				if (rail + localOffset < rails) {
					if (decks[deck].orientation)
						moveBeadsByValue(deck, rail + localOffset,
							decks[deck].number -
							old.decks[deck].position[rail + decimalShift],
							true);
					else
						moveBeadsByValue(deck, rail + localOffset,
							old.decks[deck].position[rail + decimalShift],
							true);
				}
			}
		}
		if ((old.sign && !oldSign) || (!old.sign && oldSign)) {
			deck = BOTTOM;
			rail = rails - 1;
			if (decks[deck].orientation) {
				moveBeadsByPos(deck, rail,
					(signPosition) ?
					1 : decks[deck].room,
					true);
			} else {
				moveBeadsByPos(deck, rail,
					(signPosition) ?
					decks[deck].room : 1,
					true);
			}
		}
		if (debug) {
			for (deck = TOP; deck >= BOTTOM; deck--) {
				for (rail = rails - 1; rail >= 0; rail--) {
					System.out.print(abacusMath.int2Char(
						decks[deck].position[rail]));
				}
				System.out.print(":" + rails + ",p" +
					piecePosition[deck] + ",pp" +
					piecePercentPosition[deck]);
				if (old.sign && !oldSign)
					System.out.println(",s" + signPosition);
				else
					System.out.println();
				if (deck == TOP) {
					for (rail = rails - 1; rail >= 0; rail--) {
						if (decimalPosition == rail)
							System.out.print(".");
						else if ((rail > decimalPosition) &&
								((decimalPosition - rail) % 3 == 0))
							System.out.print(",");
						else if ((decks[BOTTOM].piece != 0) &&
								(rail == decimalPosition - 1))
							System.out.print("+");
						else if ((decks[BOTTOM].piecePercent != 0) &&
								(rail == decimalPosition -
								shiftPercent - 1 -
								((decks[BOTTOM].piece == 0) ? 0 : 1)))
							System.out.print("+");
						else
							System.out.print(" ");
					}
					System.out.println("");
				}
			}
		}
	}

	boolean insertSpecialRail(boolean sign, boolean piece,
			boolean piecePercent) {
		int minRails = ((demo) ? AbacusInterface.MIN_DEMO_RAILS :
			AbacusInterface.MIN_RAILS);

		if (rails + 2 <= minRails + ((sign) ? 1 : 0) +
				((decks[BOTTOM].piece == 0) ? 0 : 1) +
				((decks[BOTTOM].piecePercent == 0) ?
				0 : shiftPercent + 1))
			return false;
		shiftRails(rails, rails + 1,
			sign, piece, piecePercent);
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_INCREMENT);
		return true;
	}

	boolean deleteSpecialRail(boolean sign, boolean piece,
			boolean piecePercent) {
		int minRails = ((demo) ? AbacusInterface.MIN_DEMO_RAILS :
			AbacusInterface.MIN_RAILS);

		if (rails <= minRails + ((sign) ? 1 : 0) +
				((decks[BOTTOM].piece == 0) ? 0 : 1) +
				((decks[BOTTOM].piecePercent == 0) ?
				0 : shiftPercent + 1) +
				(checkSubdeck(3) ? 2 : 0))
			return false;
		shiftRails(rails, rails - 1,
			sign, piece, piecePercent);
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_DECREMENT);
		checkDecimal();
		return true;
	}

	void incrementRails() {
		if (mouseDown)
			return;
		if (rails >= 64)
			return;
		shiftRails(rails, rails + 1,
			false, false, false);
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_INCREMENT);
	}

	void decrementRails() {
		int minRails = ((demo) ? AbacusInterface.MIN_DEMO_RAILS :
			AbacusInterface.MIN_RAILS);

		if (mouseDown)
			return;
		if (rails <= minRails + ((sign) ? 1 : 0) +
				((decks[BOTTOM].piece == 0) ? 0 : 1) +
				((decks[BOTTOM].piecePercent == 0) ?
				0 : shiftPercent + 1) +
				(checkSubdeck(3) ? 2 : 0))
			return;
		shiftRails(rails, rails - 1,
			false, false, false);
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_DECREMENT);
		checkDecimal();
	}

	void reformatRails() {
		String buffer; /* new char[length(digits) + 1] */

		buffer = digits.toString();
		resetBeads();
		resizeAbacus();
		drawAllBufferedBeads();
		drawFrame(false);
		drawFrame(true);
		drawAllBeads();
		if (decks[BOTTOM].piecePercent != 0 &&
				decimalPosition - shiftPercent -
				((decks[BOTTOM].piece == 0) ? 0 : 1) < 1)
			decks[BOTTOM].piecePercent = 0;
		if (decks[BOTTOM].piece != 0 &&
				decimalPosition < 1)
			decks[BOTTOM].piece = 0;
		if (!setBeadsForValue(buffer)) {
			resetBeads();
		}
	}

	void formatRails(int inc) {
		if (mode < AbacusInterface.Modes.Chinese.ordinal() ||
				mode > AbacusInterface.Modes.Generic.ordinal())
			mode = AbacusInterface.Modes.Chinese.ordinal();
		else
			mode = (mode + inc) % (AbacusInterface.Modes.values().length - 1);
		if (demo && mode == AbacusInterface.Modes.Generic.ordinal()) {
			mode = AbacusInterface.Modes.Chinese.ordinal();
		}
		if (sign) {
			if (decks[BOTTOM].orientation)
				minusSign =
				decks[BOTTOM].position[rails - 1] == 0;
			else
				minusSign =
					decks[BOTTOM].position[rails - 1] != 0;
		}
		/* clear special beads */
		if (checkSubdeck(3)) {
			int deck = 0;

			for (deck = 0; deck < subdeck; deck++) {
				moveBeadsDown(deck, decimalPosition - 2,
					subdecks[deck].room - 1, true);
			}
			/* This should not be necessary. */
			digits.setCharAt(rails - 1, '0');
		}
		checkBeads();
		reformatRails();
		if (inc != 0)
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_FORMAT);
	}

	void museumRails() {
		if (submode < AbacusInterface.IT ||
				submode > AbacusInterface.MAX_MUSEUMS)
			submode = generator.nextInt(AbacusInterface.MAX_MUSEUMS);
		else
			submode = (submode + 1) % AbacusInterface.MAX_MUSEUMS;
		drawFrame(false);
		drawFrame(true);
		drawAllBeads();
	}

	public void toggleRomanNumeralsDisplay() {
		romanNumerals = !romanNumerals;
		setCounter(0, decimalPosition, 0);
	}

	public void toggleGroupDisplay() {
		group = !group;
		setCounter(0, decimalPosition, 0);
	}

	void toggleSignRail() {
		sign = !sign;
		if ((sign && ((((AbacusApplet)applet).getTeach()) ||
				!insertSpecialRail(true, false, false))) ||
				(!sign &&
				!deleteSpecialRail(true, false, false))) {
			sign = !sign;
			return;
		}
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_SIGN);
	}

	void togglePieceRail(int topPieces, int bottomPieces) {
		int[] oldPiece = new int[MAX_DECKS];

		oldPiece[TOP] = decks[TOP].piece;
		oldPiece[BOTTOM] = decks[BOTTOM].piece;
		if (decks[BOTTOM].piece != 0 &&
				decks[BOTTOM].piecePercent != 0)
			return;
		if (decks[TOP].number == 0 && topPieces != 0) {
			if (decks[BOTTOM].room <= topPieces * bottomPieces) {
				decks[BOTTOM].room = topPieces * bottomPieces + 1;
				decks[BOTTOM].spaces = decks[BOTTOM].room -
					decks[BOTTOM].number;
			}
		} else {
			if (checkBottomSpace() < bottomPieces) {
				decks[BOTTOM].spaces -=
					(checkBottomSpace() - bottomPieces);
				decks[BOTTOM].room = decks[BOTTOM].number +
					decks[BOTTOM].spaces;
			}
		}
		decks[BOTTOM].piece = (decks[BOTTOM].piece == 0) ?
			bottomPieces : 0;
		decks[TOP].piece = (decks[TOP].piece != 0 ||
			decks[BOTTOM].piece == 0) ? 0 : topPieces;
		if (checkSubdeck(2)) {
			decks[BOTTOM].position[1] = 0;
		}
		if (decks[BOTTOM].piece == 0) {
			if (!deleteSpecialRail(false, true, false)) {
				decks[BOTTOM].piece = oldPiece[BOTTOM];
				decks[TOP].piece = oldPiece[TOP];
			}
		} else {
			if (!insertSpecialRail(false, true, false)) {
				decks[BOTTOM].piece = oldPiece[BOTTOM];
				decks[TOP].piece = oldPiece[TOP];
			}
		}
		if (decks[BOTTOM].piece != oldPiece[BOTTOM] ||
				decks[TOP].piece != oldPiece[TOP]) {
			if (bottomPieces == QUARTERS && topPieces == 0)
				((AbacusApplet)applet).callbackAbacus(this,
					AbacusInterface.ACTION_QUARTER);
			else if (bottomPieces == TWELFTHS / 2 && topPieces == 2)
				((AbacusApplet)applet).callbackAbacus(this,
					AbacusInterface.ACTION_TWELFTH);
		}
	}

	void togglePiecePercentRail(int topPiecePercents, int bottomPiecePercents) {
		int[] oldPiecePercent = new int[MAX_DECKS];

		oldPiecePercent[TOP] = decks[TOP].piecePercent;
		oldPiecePercent[BOTTOM] = decks[BOTTOM].piecePercent;
		if (decimalPosition < 1 + shiftPercent)
			return;
		if (decks[BOTTOM].piece == 0 &&
				decks[BOTTOM].piecePercent == 0)
			return;
		if (decks[TOP].number == 0 && topPiecePercents != 0) {
			if (decks[BOTTOM].room <= topPiecePercents * bottomPiecePercents) {
				decks[BOTTOM].room = topPiecePercents * bottomPiecePercents + 1;
				decks[BOTTOM].spaces = decks[BOTTOM].room -
					decks[BOTTOM].number;
			}
		} else {
			if (checkBottomSpace() < bottomPiecePercents) {
				decks[BOTTOM].spaces -=
					(checkBottomSpace() - bottomPiecePercents);
				decks[BOTTOM].room = decks[BOTTOM].number +
					decks[BOTTOM].spaces;
			}
		}
		decks[BOTTOM].piecePercent =
			(decks[BOTTOM].piecePercent == 0) ?
			bottomPiecePercents : 0;
		decks[TOP].piecePercent =
			(decks[TOP].piecePercent != 0 ||
			decks[BOTTOM].piecePercent == 0) ?
			0 : topPiecePercents;
		if (checkSubdeck(4)) {
			decks[BOTTOM].position[2] = 0;
		}
		if (decks[BOTTOM].piecePercent == 0) {
			if (!deleteSpecialRail(false, false, true)) {
				decks[BOTTOM].piecePercent = oldPiecePercent[BOTTOM];
				if (topPiecePercents != 0)
					decks[TOP].piecePercent = oldPiecePercent[TOP];
			}
		} else {
			if (!insertSpecialRail(false, false, true)) {
				decks[BOTTOM].piecePercent = oldPiecePercent[BOTTOM];
				if (topPiecePercents != 0)
					decks[TOP].piecePercent = oldPiecePercent[TOP];
			}
		}
		if ((decks[BOTTOM].piecePercent != oldPiecePercent[BOTTOM] ||
				decks[TOP].piecePercent !=
				oldPiecePercent[TOP]) &&
				bottomPiecePercents == QUARTER_PERCENTS &&
				topPiecePercents == 0)
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_QUARTER_PERCENT);
	}

	void toggleSubdeckRail(int ndecks, int nbeads) {
		int deck;

		if (subdeck == 0) {
			subdeck = ndecks;
			subbead = nbeads;
			for (deck = 0; deck < ndecks; deck++) {
				subdecks[deck].number =
					numberSubbeads(BOTTOM);
				subdecks[deck].room =
					subdecks[deck].number + 1;
				subdecks[deck].position =
					(decks[BOTTOM].orientation) ?
					subdecks[deck].number : 0;
			}
		} else {
			if (checkSubdeck(3)) {
				for (deck = 0; deck < subdeck; deck++) {
					moveBeadsDown(deck,
						decimalPosition - 2,
						subdecks[deck].room - 1, true);
				}
				/* This should not be necessary. */
				digits.setCharAt(rails - 1, '0');
			}
			subdeck = 0;
		}
		if (sign) {
			if (decks[BOTTOM].orientation)
				minusSign =
					decks[BOTTOM].position[rails - 1] == 0;
			else
				minusSign =
					decks[BOTTOM].position[rails - 1] != 0;
		}
		reformatRails();
	}

	void toggleAnomalyRails(int anomalyRail, int anomalySqRail) {
		if (anomaly == 0) {
			if (((AbacusApplet)applet).getTeach())
				return;
			anomaly = anomalyRail;
		} else {
			anomaly = 0;
		}
		anomalySq = anomalySqRail;
		if (sign) {
			if (decks[BOTTOM].orientation)
				minusSign =
					decks[BOTTOM].position[rails - 1] == 0;
			else
				minusSign =
					decks[BOTTOM].position[rails - 1] != 0;
		}
		reformatRails();
	}

	void toggleVerticalRails() {
		drawFrame(false);
		vertical = !vertical;
		resizeAbacus();
		drawAllBufferedBeads();
		drawFrame(true);
		drawAllBeads();
	}

	void showMessage(String msg) {
		((AbacusApplet)applet).showMessage(msg);
	}

	void resizeAbacus() {
		Point size = new Point(0, 0);
		int color, pressed, shade;
		int height;

		/* Determine size of client area */
		coreSize = new Point(getSize().width, getSize().height);
		frameSize = new Point(0, 0);
		delta = new Point(8, 1 + pressOffsetY);
		midHeight = 6;
		if (vertical) {
			frameSize.y = coreSize.x;
			frameSize.x = coreSize.y;
		} else {
			frameSize.x = coreSize.x;
			frameSize.y = coreSize.y;
		}
		pos = new Point(java.lang.Math.max((frameSize.x -
			delta.x) / rails, delta.x),
			java.lang.Math.max((frameSize.y - 2 *
			delta.y - midHeight) / (decks[TOP].room +
			decks[BOTTOM].room), delta.y));
		width = pos.x * rails + delta.x + 2;
		decks[TOP].height = pos.y *
			decks[TOP].room + delta.y + 2;
		decks[BOTTOM].height = pos.y *
			decks[BOTTOM].room + delta.y + 2;
		height = decks[TOP].height + decks[BOTTOM].height;
		offset = new Point(java.lang.Math.max((frameSize.x -
			width + 2) / 2, 0),
			java.lang.Math.max(((frameSize.y -
			height - midHeight + 5) / 2 + 1), 0));
		midBarY = decks[TOP].room * pos.y +
			delta.y - 1 + offset.y;
		if (slot)
			railWidth = pos.x / 8 + 1;
		else
			railWidth = pos.x / 24 + 1;
		beadSize = new Point(pos.x - delta.x, pos.y - delta.y);
		if (vertical) {
			size.x = pos.y - pressOffsetY;
			size.y = beadSize.x + pressOffsetX;
		} else {
			size.x = beadSize.x + pressOffsetX;
			size.y = pos.y - pressOffsetY;
		}
		bufferedBeadImage = new Image[3][2][2];
		bufferedBeadGraphics = new Graphics[3][2][2];
		for (color = 0; color < 3; color++)
			for (pressed = 0; pressed < 2; pressed++)
				for (shade = 0; shade < 2; shade++) {
					bufferedBeadImage[color][pressed][shade] =
						createImage(size.x, size.y);
					bufferedBeadGraphics[color][pressed][shade] =
						bufferedBeadImage[color][pressed][shade].getGraphics();
				}
	}

	public void initializeAbacus() {
		int deck, i;
		//decks[deck].position = null;

		numSlices = ((delay < 5 * MAX_SLICES) ? delay / 5 + 1 :
			MAX_SLICES);
		if (decks[BOTTOM].piece != 0)
			decimalPosition++;
		if (decks[BOTTOM].piecePercent != 0)
			decimalPosition++;
		abacusDemo = new AbacusDemo(applet, this);
		for (deck = 0; deck < MAX_DECKS; deck++)
			decks[deck].position = null;
		digits = null;
		//mode = AbacusInterface.setModeFromFormat();
		//setSubmodeFromMuseum();
		checkBeads();
		resetBeads();
		resizeAbacus();

		for (i = 0; i < 3; i++) {
			beadShade[4 * i] = beadColor[i].brighter();
			beadShade[1 + 4 * i] = beadColor[i];
			beadShade[2 + 4 * i] = beadColor[i].darker();
			beadShade[3 + 4 * i] = (beadShade[2 + 4 * i]).darker();
		}
		generator = new Random(System.currentTimeMillis());
	}

	void exposeAbacus() {
		drawFrame(false);
		drawFrame(true);
		drawAllBufferedBeads();
		drawAllBeads();
	}

	void selectAbacus(int x, int y) {
		int rail;

		if (demo) {
			resetPaint = true;
			repaint();
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_CLEAR);
			currentDeck = AbacusInterface.ACTION_IGNORE;
		} else if ((!vertical && (y >= midBarY) &&
				(y <= midBarY + midHeight))) {
			rail = rails -
				(x + 1 + railWidth / 2 -
				delta.x - offset.x) /
				pos.x - 1;
			setDecimal(rail);
		} else if ((vertical && (x <= frameSize.y - 1 - midBarY) &&
			(x >= frameSize.y - 1 - midBarY - midHeight))) {
			rail = rails -
				(y - 1 - railWidth / 2 -
				delta.x - offset.x) /
				pos.x - 1;
			setDecimal(rail);
		} else if (positionToBead(x, y)) {
			selectPaint = true;
			repaint();
		} else
			currentDeck = AbacusInterface.ACTION_IGNORE;
	}

	void releaseAbacus() {
		if (currentDeck < 0)
			return;
		currentSpace = 0;
		releasePaint = true;
		repaint();
	}

	public void complementAbacus() {
		complementRails();
	}

	public void incrementAbacus() {
		incrementRails();
	}

	public void decrementAbacus() {
		decrementRails();
	}

	public void speedUpAbacus() {
		delay -= 10;
		if (delay < 0)
			delay = 0;
	}

	public void slowDownAbacus() {
		delay += 10;
	}

	public void changeFormatAbacus() {
		formatRails(1);
	}

	public void changeMuseumAbacus() {
		museumRails();
	}

	public void toggleRomanNumeralsAbacus() {
		toggleRomanNumeralsDisplay();
	}

	void toggleGroupingAbacus() {
		toggleGroupDisplay();
	}

	void toggleNegativeSignAbacus() {
		toggleSignRail();
	}

	public void toggleQuartersAbacus() {
		togglePieceRail(0, QUARTERS);
	}

	public void toggleQuarterPercentsAbacus() {
		togglePiecePercentRail(0, QUARTER_PERCENTS);
	}

	public void toggleTwelfthsAbacus() {
		togglePieceRail(2, TWELFTHS / 2);
	}

	public void toggleSubdecksAbacus() {
		subbase = TWELFTHS;
		resetSubdecks();
		toggleSubdeckRail(AbacusInterface.DEFAULT_SUBDECKS,
			AbacusInterface.DEFAULT_SUBBEADS);
	}

	public void toggleEighthsAbacus() {
		subbase = EIGHTHS;
		resetSubdecks();
		toggleSubdeckRail(AbacusInterface.DEFAULT_SUBDECKS,
			AbacusInterface.DEFAULT_SUBBEADS);
	}

	/* Mesoamerican Nepohualtzintzin setting use with base 20 */
	public void toggleAnomalyAbacus() {
		toggleAnomalyRails(2, 0);
	}

	/* Babylonian Watch */
	public void toggleWatchAbacus() {
		toggleAnomalyRails(4, 4);
	}

	/*public void VerticalAbacus() {
		toggleVerticalRails();
	}*/

	public void setBaseAbacus(int newValue) {
		base = newValue;
		formatRails(0);
	}

	public void setDisplayBaseAbacus(int newValue) {
		displayBase = newValue;
		setCounter(0, decimalPosition, 0);
	}

	public void toggleDemoAbacus() {
		if (!((AbacusApplet)applet).mainAbacus(this))
			return;
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_DEMO);
		demo = ((AbacusApplet)applet).getDemo();
		checkBeads();
		resetBeads();
		repaint();
		if (demo) {
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_CLEAR);
			abacusDemo.clearDemo();
		} else {
			((AbacusApplet)applet).killDemoDialog();
		}
	}

	public void toggleTeachAbacus() {
		if (!((AbacusApplet)applet).mainAbacus(this))
			return;
		((AbacusApplet)applet).callbackAbacus(this,
			AbacusInterface.ACTION_TEACH);
		checkBeads();
		resetBeads();
		repaint();
		if (((AbacusApplet)applet).getTeach()) {
			((AbacusApplet)applet).callbackAbacus(this,
				AbacusInterface.ACTION_CLEAR);
		}
	}

	public void toggleRightToLeftAbacusAdd() {
		((AbacusApplet)applet).setRightToLeftAdd(
			!((AbacusApplet)applet).getRightToLeftAdd());
	}

	public boolean getRightToLeftAdd() {
		return ((AbacusApplet)applet).getRightToLeftAdd();
	}

	public void toggleRightToLeftAbacusMult() {
		((AbacusApplet)applet).setRightToLeftMult(
			!((AbacusApplet)applet).getRightToLeftMult());
	}

	public boolean getRightToLeftMult() {
		return ((AbacusApplet)applet).getRightToLeftMult();
	}

	public void showNextAbacus() {
		/*resetBeads();*/
		abacusDemo.queryDemo(abacusDemo.chapter, true);
	}

	public void showRepeatAbacus() {
		abacusDemo.queryDemo(abacusDemo.chapter, false);
	}

	public void showJumpAbacus() {
		abacusDemo.jumpDemo();
	}

	public void showChapterAbacus(int chapt) {
		abacusDemo.chapterDemo(chapt);
	}

	public void showMoreAbacus() {
		abacusDemo.moreDemo();
	}

	public void paint(Graphics g) {
		int deck, rail = 0, bead, j = 0;

		if (firstPaint) {
			initializeAbacus();
			firstPaint = false;
			resizePaint = true;
		}
		if (!framePaint && !resetPaint &&
				!selectPaint && !releasePaint)
			resizePaint = true;
		if (resizePaint) {
			resizeAbacus();
			exposeAbacus();
			resizePaint = false;
		}
		if (framePaint) {
			drawFrame(true);
			framePaint = false;
		}
		if (resetPaint) {
			clearRails();
			resetPaint = false;
		}
		if (selectPaint) {
			if (currentDeck != AbacusInterface.ACTION_IGNORE) {
				deck = currentDeck;
				rail = currentRail;
				j = currentPosition;
				bead = j;
				if (checkSubdeck(3) && rail == decimalPosition - 2) {
					if (bead > subdecks[deck].position)
						bead -= subdecks[deck].spaces;
					bead = j + numberSubbeadsOffset(deck);
					j = bead;
					bead -= AbacusInterface.SUBDECK_SPACE;
					deck = BOTTOM;
				} else {
					if (bead > decks[currentDeck].position[currentRail])
						bead -= decks[currentDeck].spaces;
				}
				drawBead(deck, rail, bead, j,
					true, false, false, 1, 0, 0);
			}
			selectPaint = false;
		}
		if (releasePaint) {
			if (currentDeck != AbacusInterface.ACTION_IGNORE) {
				deck = currentDeck;
				rail = currentRail;
				j = currentPosition;
				bead = j;
				if (checkSubdeck(3) && rail == decimalPosition - 2) {
					if (bead > subdecks[deck].position)
						bead -= subdecks[deck].spaces;
					bead = j + numberSubbeadsOffset(deck);
					j = bead;
					bead -= AbacusInterface.SUBDECK_SPACE;
					deck = BOTTOM;
				} else {
					if (bead > decks[deck].position[rail])
						bead -= decks[deck].spaces;
				}
				drawBead(deck, rail, bead, j,
					false, false, false, 1, 0, 0);
				drawBead(deck, rail, bead, j,
					true, false, false, 0, 0, 0);
				if (currentSpace != -1) {
					moveBeadsByPos(currentDeck, rail,
						currentPosition, false);
				}
				currentDeck = AbacusInterface.ACTION_IGNORE;
			}
			releasePaint = false;
		}
	}

	public void update(Graphics g) {
		paint(g); /* no erase */
	}

	void paintNow() {
		Graphics g = getGraphics();

		try {
			paint(g);
		} catch (Exception e) {
			e.printStackTrace();
			return;
		} finally {
			g.dispose();
		}
	}
}

class AbacusDeck {
	int	 number;
	boolean     orientation;
	int	 factor;
	int	 spaces;
	int[]       position = null;
	int	 room;
	int	 height = 0;
	int	 piece = 0, piecePercent = 0;

	public AbacusDeck(int rails, int number, boolean orientation,
			int factor, int spaces) {
		this.number = number;
		this.orientation = orientation;
		this.factor = factor;
		this.spaces = spaces;
		this.room = spaces + number;
		this.position = new int[rails];
	}

	public AbacusDeck(int rails, int number, boolean orientation,
			int factor, int spaces, int piece) {
		this.number = number;
		this.orientation = orientation;
		this.factor = factor;
		this.spaces = spaces;
		this.room = spaces + number;
		this.position = new int[rails];
		this.piece = piece;
	}

	public AbacusDeck(int rails, int number, boolean orientation,
			int factor, int spaces, int piece, int piecePercent) {
		this.number = number;
		this.orientation = orientation;
		this.factor = factor;
		this.spaces = spaces;
		this.room = spaces + number;
		this.position = new int[rails];
		this.piece = piece;
		this.piecePercent = piecePercent;
	}
}

class AbacusSubdeck {
	int	 number;
	int	 spaces;
	int	 position;
	int	 room;

	public AbacusSubdeck(int number, int spaces) {
		this.number = number;
		this.spaces = spaces;
		this.room = spaces + number;
		this.position = 0;
	}
}
