/*
 * @(#)OKDialog.java
 *
 * Mostly from Java Sourcebook
 * by Ed Anuff, pp 287-289
 */

import java.awt.*;
import java.awt.event.*;

/**
 * The <code>OKDialog</code> class is a  <code>Dialog</code>
 * with an OK button.
 *
 * @author	Ed Anuff
 */

public class OKDialog extends Dialog {
	private static final long serialVersionUID = 42L;
	private static boolean active = false;

	public OKDialog(Frame parent,
			String titleText, String messageText,
			String buttonText, Icon icon, boolean scroll) {
		super(parent, titleText, true);
		if (messageText != null) {
			if (scroll) {
				ScrollPane scrollPane = new ScrollPane();
				scrollPane.setSize(480, 200);
				scrollPane.add(new MultiLineLabel(messageText),
					BorderLayout.CENTER);
				add(scrollPane, BorderLayout.CENTER);
				if (icon != null)
					add(icon, BorderLayout.LINE_START);
			} else {
				Panel messagePanel = new Panel();
				if (icon != null)
					messagePanel.add(icon,
						BorderLayout.LINE_START);
				messagePanel.add(new
					MultiLineLabel(messageText),
					BorderLayout.CENTER);
				add(messagePanel, BorderLayout.LINE_START);
			}
		}
		Button button = new Button(buttonText);
		Panel buttonPanel = new Panel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		buttonPanel.add(button);
		add("South", buttonPanel);
		pack();
		setActive(true);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				dispose();
				//else multiple OKDialogs can be opened
				setActive(false);
			}
		});
		addWindowListener(new WindowListener() {
			public void windowClosed(WindowEvent event) {
			}
			public void windowIconified(WindowEvent event) {
			}
			public void windowDeiconified(WindowEvent event) {
			}
			public void windowActivated(WindowEvent event) {
			}
			public void windowDeactivated(WindowEvent event) {
			}
			public void windowOpened(WindowEvent event) {
			}
			public void windowClosing(WindowEvent event) {
				dispose();
				setActive(false);
			}
		});
	}

	boolean getActive() {
		return OKDialog.active;
	}

	void setActive(boolean value) {
		active = value;
	}
}
